import { jsx as _jsx } from "react/jsx-runtime";
import { useState, useRef, useEffect } from 'react';
import { createResponseTextNotification } from '@brudererdev/web-extensions';
import { LogProvider } from '@brudererdev/web-extensions';
import { VariableContainerRenderer } from './VariableContainerRenderer';
export const ServiceVariableContainerInjector = (props) => {
    // Fields
    const contextName = 'ServiceVariableContainerInjector';
    const logger = LogProvider.getLogger(contextName);
    // States
    const [serviceVersion, setServiceVersion] = useState(-1);
    const [container, setContainer] = useState(props.container);
    const [changedVariables, setChangedVariables] = useState([]);
    const [componentVersion, setComponentVersion] = useState(-1);
    // Refs
    const changedVariablesRef = useRef(changedVariables);
    changedVariablesRef.current = changedVariables;
    // Effects
    useEffect(() => {
        // Mount
        if (props.service)
            props.service.onChanges(contextName, handleServiceChanges);
        // Unmount
        return () => {
            if (props.service)
                props.service.offChanges(contextName);
        };
    }, []);
    useEffect(() => {
        if (!props.container)
            return;
        if (!container) {
            setContainer(props.container);
            return;
        }
    }, [props.container]);
    useEffect(() => {
        if (props.container) {
            if (props.discardChangesCommand === 'All')
                setContainer(props.container);
        }
    }, [props.discardChangesCommand]);
    useEffect(() => {
        if (!props.saveChangesCommand)
            return;
        if (changedVariables.length <= 0) {
            props.onChangesSaved(false);
            return;
        }
        if (props.onWriteVariables) {
            props.onWriteVariables(changedVariables);
            return;
        }
        if (props.service)
            props.service.writeVariables(changedVariables, contextName)
                .then(response => {
                var result = (response.payload != undefined) ? response.payload : false;
                // Create notification
                var notification = undefined;
                if (!result)
                    notification = createResponseTextNotification(response, props.onLocalize);
                props.onChangesSaved(result, notification);
            });
    }, [props.saveChangesCommand]);
    useEffect(() => {
        // Invoke external changed variable list updated method
        if (props.onChangedVariableListUpdated)
            props.onChangedVariableListUpdated(changedVariables);
    }, [componentVersion]);
    const handleServiceChanges = (version, key) => {
        setServiceVersion(version);
        if (props.service)
            setContainer(props.service.variableContainer);
    };
    const handleChangesDiscarded = (result, plcSymbol) => {
        if (!result)
            return;
        var indexToRemove = changedVariablesRef.current.findIndex(v => v.link.key.toLowerCase() === plcSymbol.toLowerCase());
        if (indexToRemove < 0)
            return;
        const newChangedVariableList = [...changedVariablesRef.current];
        newChangedVariableList.splice(indexToRemove, 1);
        changedVariablesRef.current = newChangedVariableList;
        setChangedVariables(newChangedVariableList);
        setComponentVersion(componentVersion + 1);
    };
    const handleVariableChanged = (variable) => {
        switch (variableChangeHandling) {
            case 'external': {
                // Invoke external variable changed method
                if (props.onVariableChanged)
                    props.onVariableChanged(variable, props.service);
                // Push the variable to the change list
                const newChangedVariableList = [...changedVariables];
                var index = newChangedVariableList.findIndex(v => v.link.key === variable.link.key);
                if (index === -1)
                    newChangedVariableList.push(variable);
                else
                    newChangedVariableList[index] = variable;
                setChangedVariables(newChangedVariableList);
                setComponentVersion(componentVersion + 1);
                break;
            }
            case 'direct': {
                var variables = [];
                variables.push(variable);
                if (props.service)
                    props.service.writeVariables(variables, contextName)
                        .then(response => {
                        //Create notification?
                    });
                break;
            }
        }
    };
    // Helpers
    var variableChangeHandling = 'external';
    return (_jsx(VariableContainerRenderer, { containers: container, containerDisplayKeyFilter: props.containerDisplayKeyFilter, 
        // filter={props.filter}
        // minimumFilterLength={props.minimumFilterLength}
        renderType: props.renderType, userRoles: [], locale: props.locale, onLocalize: props.onLocalize, onVariableChanged: handleVariableChanged, onCustomVariableRendering: props.onCustomVariableRendering, onCustomListRendering: props.onCustomListRendering, onCustomListHeaderRendering: props.onCustomListHeaderRendering, onAdditionalListContentRendering: props.onAdditionalListContentRendering }, void 0));
};
