import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useState, useEffect, useRef } from 'react';
import { getFormattedDate } from '@brudererdev/web-extensions';
import { createResponseTextNotification } from '@brudererdev/web-extensions';
import Card from '@material-ui/core/Card';
import CardContent from '@material-ui/core/CardContent';
import { VariableRenderer } from './../renderer/VariableRenderer';
import { Label } from './../elements/Label';
import { WrapContainer } from './../containers/WrapContainer';
import { ElementContainer } from './../containers/ElementContainer';
import { getFilteredVariables } from './../../shared/variableProps';
export const ServiceVariableInjector = (props) => {
    // Fields
    const contextName = 'ServiceVariableInjector';
    // States
    const [serviceVersion, setServiceVersion] = useState(-1);
    const [changedVariables, setChangedVariables] = useState([]);
    const [componentVersion, setComponentVersion] = useState(-1);
    // Refs
    const changedVariablesRef = useRef(changedVariables);
    changedVariablesRef.current = changedVariables;
    // Effects
    useEffect(() => {
        // Mount
        if (props.service)
            props.service.onChanges(contextName, handleServiceChanges);
        // Unmount
        return () => {
            if (props.service)
                props.service.offChanges(contextName);
        };
    }, []);
    useEffect(() => {
        if (!props.saveChangesCommand)
            return;
        if (changedVariables.length <= 0) {
            props.onChangesSaved(false);
            return;
        }
        if (props.service)
            props.service.writeVariables(changedVariables, contextName)
                .then(response => {
                var result = (response.payload != undefined) ? response.payload : false;
                // Create notification
                var notification = undefined;
                if (result) {
                    setChangedVariables([]);
                    setComponentVersion(componentVersion + 1);
                }
                else
                    notification = createResponseTextNotification(response, props.onLocalize);
                props.onChangesSaved(result, notification);
            });
    }, [props.saveChangesCommand]);
    useEffect(() => {
        // Invoke external changed variable list updated method
        if (props.onChangedVariableListUpdated)
            props.onChangedVariableListUpdated(changedVariables);
    }, [componentVersion]);
    const handleServiceChanges = (version, key) => {
        setServiceVersion(version);
    };
    const handleChangesDiscarded = (result, plcSymbol) => {
        if (!result)
            return;
        var indexToRemove = changedVariablesRef.current.findIndex(v => v.link.key.toLowerCase() === plcSymbol.toLowerCase());
        if (indexToRemove < 0)
            return;
        const newChangedVariableList = [...changedVariablesRef.current];
        newChangedVariableList.splice(indexToRemove, 1);
        changedVariablesRef.current = newChangedVariableList;
        setChangedVariables(newChangedVariableList);
        setComponentVersion(componentVersion + 1);
    };
    const handleVariableChanged = (variable) => {
        switch (variableChangeHandling) {
            case 'external': {
                // Invoke external variable changed method
                if (props.onVariableChanged)
                    props.onVariableChanged(variable, props.service);
                // Push the variable to the change list
                const newChangedVariableList = [...changedVariables];
                var index = newChangedVariableList.findIndex(v => v.link.key === variable.link.key);
                if (index === -1)
                    newChangedVariableList.push(variable);
                else
                    newChangedVariableList[index] = variable;
                setChangedVariables(newChangedVariableList);
                setComponentVersion(componentVersion + 1);
                break;
            }
            case 'direct': {
                var variables = [];
                variables.push(variable);
                if (props.service)
                    props.service.writeVariables(variables, contextName)
                        .then(response => {
                        //Create notification?
                    });
                break;
            }
        }
    };
    const renderVariablesNormal = () => {
        return (_jsx(WrapContainer, { children: filteredVariables.map((variable, index) => {
                var discardChanges = false;
                if (props.discardChangesCommand)
                    discardChanges = (props.discardChangesCommand === 'All') || (variable.link.key.toLowerCase() === props.discardChangesCommand.toLowerCase());
                else
                    discardChanges = false;
                return (_jsx(ElementContainer, { children: _jsx(VariableRenderer, { variable: variable, showInitialValueOnChange: true, userRoles: [], locale: props.locale, onLocalize: props.onLocalize, onVariableChanged: handleVariableChanged }, void 0) }, index));
            }) }, void 0));
    };
    const renderVariablesDeveloper = () => {
        return (_jsx(WrapContainer, { children: filteredVariables.map((variable, index) => {
                var discardChanges = false;
                if (props.discardChangesCommand)
                    discardChanges = (props.discardChangesCommand === 'All') || (variable.link.key.toLowerCase() === props.discardChangesCommand.toLowerCase());
                else
                    discardChanges = false;
                const locale = (props.locale !== undefined) ? props.locale : 'de-DE';
                const formattedLastChangeDate = getFormattedDate(new Date(variable.lastUpdate), locale);
                return (_jsx(Card, Object.assign({ classes: {
                        root: 'card-root'
                    } }, { children: _jsxs(CardContent, Object.assign({ classes: {
                            root: 'cardcontent-root'
                        } }, { children: [_jsx(ElementContainer, { children: _jsx(VariableRenderer, { variable: variable, showInitialValueOnChange: true, userRoles: [], locale: props.locale, onLocalize: props.onLocalize, onVariableChanged: handleVariableChanged }, void 0) }, void 0), _jsxs("div", Object.assign({ className: 'm2 text-wrap-word' }, { children: [_jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0 }, { children: `Symbol` }), void 0), _jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0, isStartAdornmentVisible: false, hasStartAdornmentIndent: true }, { children: `${variable.link.key}` }), void 0)] }), void 0), _jsxs("div", Object.assign({ className: 'm2 text-wrap-word' }, { children: [_jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0 }, { children: `Type` }), void 0), _jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0, isStartAdornmentVisible: false, hasStartAdornmentIndent: true }, { children: `${variable.valueType}` }), void 0)] }), void 0), _jsxs("div", Object.assign({ className: 'm2 text-wrap-word' }, { children: [_jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0 }, { children: `Last change` }), void 0), _jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0, isStartAdornmentVisible: false, hasStartAdornmentIndent: true }, { children: formattedLastChangeDate }), void 0)] }), void 0), _jsxs("div", Object.assign({ className: 'm2 text-wrap-word' }, { children: [_jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0 }, { children: `Version` }), void 0), _jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0, isStartAdornmentVisible: false, hasStartAdornmentIndent: true }, { children: `${variable.version}` }), void 0)] }), void 0), _jsxs("div", Object.assign({ className: 'm2 text-wrap-word' }, { children: [_jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0 }, { children: `Service` }), void 0), _jsx(Label, Object.assign({ topOffset: 0, bottomOffset: 0, isStartAdornmentVisible: false, hasStartAdornmentIndent: true }, { children: `${props.service ? props.service.key : 'Service?'}` }), void 0)] }), void 0)] }), void 0) }), index));
            }) }, void 0));
    };
    // Setup service variables
    var variables = [];
    if (props.service)
        variables = props.service.variables;
    // Filter variables
    var filteredVariables = variables;
    if (props.filter)
        filteredVariables = getFilteredVariables(variables, props.filter, props.onLocalize);
    // Helpers
    var variableChangeHandling = 'external';
    const displayMode = props.displayMode ? props.displayMode : 'normal';
    var displayText = props.service.display.value;
    if (props.onLocalize)
        displayText = props.onLocalize(props.service.display);
    return (_jsxs("div", Object.assign({ className: 'd-flex flex-col' }, { children: [props.service && (filteredVariables.length > 0) &&
                _jsx(Label, { children: displayText }, void 0), (displayMode === 'normal') && renderVariablesNormal(), (displayMode === 'developer') && renderVariablesDeveloper()] }), void 0));
};
