import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState } from 'react';
import { generateVariableContainerList } from '@brudererdev/web-extensions';
import { SizeEnumeration } from './../../shared/styleProps';
import { WrapContainer, ElementContainer } from './../containers';
import { Label } from './../elements';
import Card from '@material-ui/core/Card';
import CardContent from '@material-ui/core/CardContent';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import Collapse from '@material-ui/core/Collapse';
import ExpandLess from '@material-ui/icons/ExpandLess';
import ExpandMore from '@material-ui/icons/ExpandMore';
import { VariableRenderer } from './../renderer/VariableRenderer';
const getVariablesFromContainers = (containers) => {
    if (!containers)
        return [];
    var containerList = generateVariableContainerList(containers);
    var variables = [];
    for (var container of containerList) {
        variables.push(...container.variables);
        var childVariables = getVariablesFromContainers(container.variableContainers);
        variables.push(...childVariables);
    }
    return variables;
};
export const VariableContainerRenderer = (props) => {
    // States
    const [openIndex, setOpenIndex] = useState();
    const handleListItemClick = (e, index) => {
        if (openIndex === index) {
            setOpenIndex(-1);
            return;
        }
        setOpenIndex(index);
    };
    const getElementSize = (variable) => {
        if (variable.valueEnumerationKey)
            return SizeEnumeration.l;
        if (variable.valueType === 'System.String')
            return SizeEnumeration.xl;
        return SizeEnumeration.s;
    };
    const renderContainerVariables = (variables) => {
        if (variables.length <= 0)
            return null;
        return (_jsx(WrapContainer, { children: variables.map((variable, index) => {
                // const canBeFiltered = (props.containerDisplayKeyFilter !== undefined && !variable.display.key.toLowerCase().includes(props.containerDisplayKeyFilter.toLowerCase()));
                // if (canBeFiltered)
                //   return undefined;
                var displayText = variable.display.value;
                if (props.onLocalize)
                    displayText = props.onLocalize(variable.display);
                const minimumFilterLength = props.minimumFilterLength !== undefined ? props.minimumFilterLength : 3;
                const canFilter = props.filter !== undefined && (props.filter.length >= minimumFilterLength);
                if (canFilter) {
                    if (!displayText.toLowerCase().includes(props.filter.toLowerCase()))
                        return undefined;
                }
                return (_jsx(ElementContainer, Object.assign({ size: getElementSize(variable) }, { children: _jsx(VariableRenderer, { variable: variable, showInitialValueOnChange: true, onVariableChanged: props.onVariableChanged, userRoles: [], locale: props.locale, onLocalize: props.onLocalize }, void 0) }), index));
            }) }, void 0));
    };
    const renderContainersAsList = (containers) => {
        return (_jsx(List, Object.assign({ component: "nav", classes: {
                root: 'list-root',
            } }, { children: containers.map((container, index) => {
                const hasVariables = container.variables.length > 0;
                const hasContainers = container.variableContainers.length > 0;
                const canBeFiltered = (props.containerDisplayKeyFilter !== undefined && !container.display.link.key.toLowerCase().includes(props.containerDisplayKeyFilter.toLowerCase()));
                if (canBeFiltered) {
                    {
                        !container.isEnumerable && renderContainers(container.variableContainers);
                    }
                    {
                        container.isEnumerable && renderContainersAsList(container.variableContainers);
                    }
                    return undefined;
                }
                var isOpen = openIndex === index;
                var CustomListElement = undefined;
                // Check for custom list element rendering or list element hiding
                if (props.onCustomListRendering) {
                    CustomListElement = props.onCustomListRendering(container, index, isOpen);
                    if (typeof CustomListElement === 'boolean') {
                        if (!CustomListElement)
                            return undefined;
                    }
                    if (typeof CustomListElement !== 'undefined')
                        return CustomListElement;
                }
                // Setup custom header rendering
                var CustomListHeaderElement = undefined;
                if (props.onCustomListHeaderRendering)
                    CustomListHeaderElement = props.onCustomListHeaderRendering(container, index, isOpen);
                const hasCustomHeader = CustomListHeaderElement !== undefined;
                // Setup additional content rendering
                var AdditionalContentElement = undefined;
                if (props.onAdditionalListContentRendering)
                    AdditionalContentElement = props.onAdditionalListContentRendering(container, index, isOpen);
                const hasAdditionalContent = AdditionalContentElement !== undefined;
                var displayText = container.display.value;
                if (props.onLocalize)
                    displayText = props.onLocalize(container.display);
                return (_jsxs(React.Fragment, { children: [_jsx(ListItem, Object.assign({ button: true, classes: {
                                root: 'listitem-root element',
                                selected: 'listitem-selected primary'
                            }, selected: isOpen, onClick: (e) => handleListItemClick(e, index) }, { children: _jsxs("div", Object.assign({ className: 'listitem-commandcontainer-root' }, { children: [hasCustomHeader &&
                                        CustomListHeaderElement, !hasCustomHeader &&
                                        _jsx("div", Object.assign({ className: 'typo-h5 w-100' }, { children: displayText }), void 0), isOpen ? _jsx(ExpandLess, {}, void 0) : _jsx(ExpandMore, {}, void 0)] }), void 0) }), void 0), _jsxs(Collapse, Object.assign({ in: isOpen, timeout: "auto" }, { children: [hasVariables &&
                                    _jsx(Card, Object.assign({ className: 'card-root' }, { children: _jsx(CardContent, Object.assign({ className: 'cardcontent-root' }, { children: renderContainerVariables(container.variables) }), void 0) }), void 0), hasAdditionalContent &&
                                    _jsxs(React.Fragment, { children: [_jsx("div", { className: 'h4' }, void 0), _jsx(Card, Object.assign({ className: 'card-root' }, { children: _jsx(CardContent, Object.assign({ className: 'cardcontent-root' }, { children: AdditionalContentElement }), void 0) }), void 0)] }, void 0), _jsx("div", { className: 'h4' }, void 0), !container.isEnumerable && renderContainers(container.variableContainers), container.isEnumerable && renderContainersAsList(container.variableContainers)] }), void 0)] }, index));
            }) }), void 0));
    };
    const renderContainers = (containers) => {
        return (_jsx(React.Fragment, { children: containers.map((container, index) => {
                const hasVariables = container.variables.length > 0;
                const hasContainers = container.variableContainers.length > 0;
                const canBeFiltered = (props.containerDisplayKeyFilter !== undefined && !container.display.link.key.toLowerCase().includes(props.containerDisplayKeyFilter.toLowerCase()));
                const canRenderVariables = !canBeFiltered && hasVariables;
                var displayText = container.display.value;
                if (props.onLocalize)
                    displayText = props.onLocalize(container.display);
                return (_jsxs(React.Fragment, { children: [canRenderVariables &&
                            _jsxs(React.Fragment, { children: [_jsx(Label, { children: displayText }, void 0), _jsx(Card, Object.assign({ className: 'card-root' }, { children: _jsx(CardContent, Object.assign({ className: 'cardcontent-root' }, { children: renderContainerVariables(container.variables) }), void 0) }), void 0), _jsx("div", { className: 'h2' }, void 0)] }, void 0), !container.isEnumerable && renderContainers(container.variableContainers), container.isEnumerable && renderContainersAsList(container.variableContainers)] }, index));
            }) }, void 0));
    };
    // Helpers
    const renderType = props.renderType ? props.renderType : 'container';
    const containerList = generateVariableContainerList(props.containers);
    const variables = getVariablesFromContainers(containerList);
    if (renderType === 'container')
        return renderContainers(containerList);
    if (renderType === 'variables')
        return renderContainerVariables(variables);
    return null;
};
