import { Component } from 'react';
import { LogProvider } from '@brudererdev/web-extensions';
export class ErrorBoundary extends Component {
    constructor(props) {
        super(props);
        this.state = {
            sourceName: this.props.sourceName,
            error: null,
            errorInfo: null
        };
    }
    componentDidCatch(error, info) {
        const logger = LogProvider.getLogger('ErrorBoundary');
        this.setState({
            error: error,
            errorInfo: info
        });
        if (info)
            logger.error(`${error && error.toString()} ${info.componentStack} @'${this.props.sourceName}'`);
    }
    componentDidUpdate() {
        if (this.state.sourceName !== this.props.sourceName) {
            this.setState({
                sourceName: this.props.sourceName,
                error: null,
                errorInfo: null
            });
        }
    }
    render() {
        if (this.state.errorInfo)
            return this.props.fallback(this.state.sourceName, this.state.error, this.state.errorInfo);
        return this.props.children;
    }
}
