import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useContext, useEffect, useRef, useState } from 'react';
import { Box, Fab } from '@mui/material';
import { StampingPressModelServiceKeys } from "@brudererdev/web-extensions";
import DoneIcon from '@mui/icons-material/Done';
import { BrudererContext } from "../../providers/BrudererProvider";
import { StatusRenderer, StatusContextEnumeration, AssistantRenderer, AssistantContextEnumeration } from "../renderer";
;
export const MessageBar = (props) => {
    // Fields
    const contextName = 'MessageBar';
    // States
    const [statusContext, setStatusContext] = useState({ contextType: StatusContextEnumeration.Unknown });
    const [assistantContext, setAssistantContext] = useState({ contextType: AssistantContextEnumeration.Unknown });
    const [messageServiceVersion, setMessageServiceVersion] = useState(-1);
    // Refs
    const subscriberKeyDictionaryRef = useRef({});
    // Contexts
    const brudererContext = useContext(BrudererContext);
    const messageService = brudererContext.getService(StampingPressModelServiceKeys.MessageService);
    // Effects
    useEffect(() => {
        // Mount
        if (messageService) {
            const registerKey = messageService.onChanges(contextName, handleMessageServiceChanges);
            subscriberKeyDictionaryRef.current[messageService.key] = registerKey;
        }
        ;
        // Unmount
        return () => {
            if (messageService) {
                const registerKey = subscriberKeyDictionaryRef.current[messageService.key];
                messageService.offChanges(registerKey);
            }
            ;
            subscriberKeyDictionaryRef.current = {};
        };
    }, []);
    const handleMessageServiceChanges = (version, key) => {
        setMessageServiceVersion(version);
    };
    const handleStatusContextChanged = (context) => {
        setStatusContext(context);
    };
    const handleAssistantContextChanged = (context) => {
        setAssistantContext(context);
    };
    const getAcknowledgeButtonStyle = () => {
        var buttonStyle = {
            backgroundColor: (theme) => theme.palette.primary.main,
            color: (theme) => theme.palette.primary.contrastText,
            fill: (theme) => theme.palette.primary.contrastText
        };
        if (messageService !== undefined && messageService.acknowledgeableMessages.length > 0)
            buttonStyle = {
                backgroundColor: (theme) => theme.palette.success.main,
                color: (theme) => theme.palette.success.contrastText,
                fill: (theme) => theme.palette.success.contrastText
            };
        return buttonStyle;
    };
    return (_jsxs(Box, { children: [_jsxs(Box, { sx: {
                    height: '100%',
                    padding: (theme) => theme.spacing(1),
                    display: 'flex',
                    flexDirection: 'row',
                    alignItems: 'center',
                    alignContent: 'center',
                    justifyContent: 'space-between',
                    ...statusContext.style
                }, onClick: (e) => {
                    e.stopPropagation();
                    if (props.onStatusClick)
                        props.onStatusClick();
                }, children: [_jsx(StatusRenderer, { type: 'MessageBar', onStatusContextChanged: handleStatusContextChanged }, void 0), _jsx(Fab, { sx: {
                            minHeight: 32,
                            minWidth: 32,
                            height: 32,
                            width: 32,
                            ...getAcknowledgeButtonStyle()
                        }, "aria-label": "Acknowledge message", onClick: (e) => {
                            e.stopPropagation();
                            if (props.onMessageAcknowledge)
                                props.onMessageAcknowledge();
                        }, children: _jsx(DoneIcon, {}, void 0) }, void 0)] }, void 0), _jsx(Box, { sx: { minHeight: (theme) => theme.spacing(1) } }, void 0), _jsx(Box, { sx: {
                    width: '100%',
                    padding: (theme) => theme.spacing(1),
                    display: 'flex',
                    flexDirection: 'row',
                    ...assistantContext.style
                }, onClick: (e) => {
                    e.stopPropagation();
                    if (props.onAssistantClick)
                        props.onAssistantClick();
                }, children: _jsx(Box, { minHeight: 32, children: _jsx(AssistantRenderer, { type: 'MessageBar', onClick: props.onAssistantClick, onAssistantContextChanged: handleAssistantContextChanged }, void 0) }, void 0) }, void 0)] }, void 0));
};
