import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { forwardRef, useState } from 'react';
import { Box, Typography, Card, CardActions, IconButton, Collapse } from '@mui/material';
import { useSnackbar } from 'notistack';
import { NotificationAcknowledgeTypeEnumeration } from '@brudererdev/web-extensions';
import CloseIcon from '@mui/icons-material/Close';
import ExpandMoreIcon from '@mui/icons-material/ExpandMore';
import ExpandLessIcon from '@mui/icons-material/ExpandLess';
import { textBreakWordStyle } from './../../styles/themeExtensions';
import { getNotificationIcon, getNotificationStyle } from './../../shared/notificationProps';
export const SnackBar = forwardRef((props, ref) => {
    // External hooks
    const { closeSnackbar } = useSnackbar();
    // States
    const [expanded, setExpanded] = useState(false);
    const handleExpandClick = () => {
        setExpanded(!expanded);
    };
    const handleDismiss = () => {
        closeSnackbar(props.key);
    };
    // Helpers
    var NotificationIcon = getNotificationIcon(props.notification);
    return (_jsxs(Card, { ref: ref, sx: {
            width: '384px',
            margin: (theme) => theme.spacing(1),
            borderRadius: (theme) => theme.spacing(1)
        }, elevation: 8, children: [_jsx(CardActions, { sx: {
                    ...getNotificationStyle(props.notification)
                }, children: _jsxs(Box, { sx: {
                        flex: 'auto',
                        display: 'flex',
                        flexDirection: 'row',
                        alignContent: 'center',
                        alignItems: 'center'
                    }, children: [_jsx(NotificationIcon, { sx: {
                                minWidth: 32,
                                minHeight: 32
                            } }, void 0), _jsx(Box, { sx: {
                                minWidth: (theme) => theme.spacing(1)
                            } }, void 0), _jsx(Typography, { variant: 'body1', sx: {
                                ...textBreakWordStyle,
                                textAlign: 'left'
                            }, children: props.message }, void 0), _jsx(Box, { sx: {
                                minWidth: (theme) => theme.spacing(1)
                            } }, void 0), _jsxs(Box, { sx: {
                                display: 'flex',
                                flexDirection: 'column',
                                marginLeft: 'auto'
                            }, children: [props.notification.acknowledgeType === NotificationAcknowledgeTypeEnumeration.User &&
                                    _jsx(IconButton, { "aria-label": "Close snackbar", onClick: handleDismiss, children: _jsx(CloseIcon, {}, void 0) }, void 0), props.notification.logText &&
                                    _jsx(IconButton, { "aria-label": "Show more", onClick: handleExpandClick, children: expanded ? _jsx(ExpandLessIcon, {}, void 0) : _jsx(ExpandMoreIcon, {}, void 0) }, void 0)] }, void 0)] }, void 0) }, void 0), _jsx(Collapse, { sx: {
                    '& .MuiCollapse-wrapper': {
                        border: 'solid currentColor 2px',
                        borderRadius: '0px 0px 6px 6px'
                    },
                }, in: expanded, timeout: 'auto', unmountOnExit: true, children: _jsx(Box, { sx: {
                        margin: (theme) => theme.spacing(1),
                        borderRadius: (theme) => theme.spacing(1)
                    }, children: _jsx(Typography, { variant: 'body2', sx: {
                            ...textBreakWordStyle,
                            textAlign: 'left'
                        }, children: props.notification.logText }, void 0) }, void 0) }, void 0)] }, void 0));
});
