import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState } from 'react';
import { LocalizationNamespaces, BrudererUserRoleEnumeration, createEmptyVariable } from '@brudererdev/web-extensions';
import { Box, Typography, Fab, IconButton, Card, CardContent } from '@mui/material';
import AppIcon from '@mui/icons-material/Apps';
import ViewQuiltIcon from '@mui/icons-material/ViewQuilt';
import CloseIcon from '@mui/icons-material/Close';
import SearchIcon from '@mui/icons-material/Search';
import TuneIcon from '@mui/icons-material/Tune';
import { cardStyle, cardContentStyle } from './../../styles/themeExtensions';
import { useDebounceFunction } from '../..';
import { WrapContainer } from '../containers';
import { VarTextIn } from '../elements';
import { NavigationElementRenderer } from './../renderer';
import { MessageBar } from './MessageBar';
export const ViewTopBar = (props) => {
    const handleSearchTextChanged = (input) => {
        if (props.onSearchTextChanged)
            props.onSearchTextChanged(input);
    };
    // States
    const [searchText, setSearchText] = useState('');
    const [debouncedSearchText, cancel, call] = useDebounceFunction(handleSearchTextChanged, 500);
    const handleCloseClick = (e) => {
        if (props.onClose)
            props.onClose(e);
        setSearchText('');
    };
    const handleMenuClick = (e) => {
        if (props.onShowApps)
            props.onShowApps(e);
        setSearchText('');
    };
    const getFilteredElements = () => {
        var filteredElements = [];
        if (!props.elements)
            return filteredElements;
        filteredElements = props.elements.filter((element, index) => {
            if (props.userRoles)
                if (element.visibilityAuthorization)
                    if (element.visibilityAuthorization > BrudererUserRoleEnumeration.Undefined)
                        if (!props.userRoles.some(role => role >= element.visibilityAuthorization))
                            return false;
            if ((element.isVisible !== undefined) && !element.isVisible)
                return false;
            return true;
        });
        return filteredElements;
    };
    const getHeadertyle = () => {
        var headerStyle = {
            backgroundColor: (theme) => theme.palette.primary.main,
            color: (theme) => theme.palette.primary.contrastText,
            fill: (theme) => theme.palette.primary.contrastText,
            ':hover': {
                backgroundColor: (theme) => theme.palette.primary.main,
                color: (theme) => theme.palette.primary.contrastText,
                fill: (theme) => theme.palette.primary.contrastText,
            },
        };
        return headerStyle;
    };
    const renderSearchEndAdornment = () => {
        return (_jsxs(React.Fragment, { children: [_jsx(IconButton, { "aria-label": "Search filter", onClick: props.onShowSearchFilter, children: _jsx(TuneIcon, {}, void 0) }, void 0), _jsx(IconButton, { "aria-label": "Search", onClick: (e) => props.onSearch ? props.onSearch(searchText) : undefined, children: _jsx(SearchIcon, {}, void 0) }, void 0)] }, void 0));
    };
    const renderNavigationElements = () => {
        if (filteredNavigationElements.length <= 1)
            return undefined;
        return (_jsx(NavigationElementRenderer, { elements: filteredNavigationElements, selectedElementIndex: props.selectedElementIndex, userRoles: props.userRoles, locale: (props.locale !== undefined) ? props.locale : 'de-DE', noWrap: true, onLocalize: props.onLocalize, onClick: (element, index) => {
                if (props.onSelectTile)
                    props.onSelectTile(element);
            } }, void 0));
    };
    const renderMessageBar = () => {
        if (!props.isMessageBarVisible)
            return undefined;
        return (_jsx(Box, { sx: {
                marginTop: (theme) => theme.spacing(1),
                marginBottom: (theme) => theme.spacing(1)
            }, children: _jsx(MessageBar, { onAssistantClick: props.onAssistantClick, onStatusClick: props.onStatusClick, onMessageAcknowledge: props.onMessageAcknowledge }, void 0) }, void 0));
    };
    const renderSearchBar = () => {
        if (!props.isSearchVisible)
            return undefined;
        return (_jsx(VarTextIn, { displayTextType: 'Label', displayTextResetOnBlur: false, localizedPlaceholderText: placeholderText, modelVariable: searchVariable, EndAdornment: renderSearchEndAdornment(), userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onTextInputChanged: (input) => debouncedSearchText(input), onTextInput: (input) => {
                setSearchText(input);
                debouncedSearchText(input);
            } }, void 0));
    };
    // Helpers
    const rtl = props.rtl !== undefined ? props.rtl : true;
    const filteredNavigationElements = getFilteredElements();
    const showActionTileButton = props.hasActionTiles !== undefined ? props.hasActionTiles : false;
    const showAppButton = props.onShowApps !== undefined;
    const showCloseButton = props.onClose !== undefined;
    const showHeaderCard = (filteredNavigationElements.length > 1) || props.isSearchVisible || props.isMessageBarVisible;
    // Setup search variable
    const searchVariable = createEmptyVariable();
    searchVariable.display = {
        link: {
            key: 'global.search',
            path: 'global',
            name: 'search',
            seperator: '.'
        },
        keyNamespace: LocalizationNamespaces.UIGeneric,
        value: 'Suche'
    };
    searchVariable.valueType = 'System.String';
    searchVariable.value = searchText;
    // Setup placeholder text
    var placeholderText = 'Geben sie einen Suchbegriff ein';
    if (props.onLocalize)
        placeholderText = props.onLocalize({
            link: {
                key: 'global.searchterm',
                path: 'global',
                name: 'searchterm',
                seperator: '.'
            },
            keyNamespace: LocalizationNamespaces.UIGeneric,
            value: 'Geben sie einen Suchbegriff ein'
        });
    return (_jsxs(Box, { sx: {
            display: 'flex',
            flexDirection: 'column',
            position: 'relative',
            zIndex: 10
        }, children: [_jsx(Box, { sx: { minHeight: (theme) => theme.spacing(2) } }, void 0), _jsxs(Box, { sx: {
                    flex: 'auto',
                    display: 'flex',
                    flexDirection: 'column',
                    backgroundColor: (theme) => theme.palette.view.main,
                    color: (theme) => theme.palette.view.contrastText,
                    fill: (theme) => theme.palette.view.contrastText
                }, children: [_jsx(Box, { sx: { minHeight: (theme) => theme.spacing(6) } }, void 0), showHeaderCard &&
                        _jsxs(Box, { sx: {
                                flex: 'auto',
                                marginLeft: (theme) => theme.spacing(1),
                                marginRight: (theme) => theme.spacing(1)
                            }, children: [_jsx(Card, { sx: { ...cardStyle }, elevation: 5, children: _jsxs(CardContent, { sx: { ...cardContentStyle }, children: [renderNavigationElements(), renderMessageBar(), renderSearchBar()] }, void 0) }, void 0), _jsx(Box, { sx: { minHeight: (theme) => theme.spacing(2) } }, void 0)] }, void 0)] }, void 0), _jsx(Box, { sx: {
                    width: '100%',
                    position: 'absolute'
                }, children: _jsx(Box, { sx: {
                        position: 'relative',
                        minHeight: 56,
                        marginLeft: (theme) => theme.spacing(1),
                        marginRight: (theme) => theme.spacing(1)
                    }, children: _jsxs(WrapContainer, { reverse: rtl, children: [showCloseButton &&
                                _jsx(Box, { children: _jsx(Fab, { sx: {
                                            alignSelf: 'center',
                                            ...getHeadertyle()
                                        }, color: 'primary', "aria-label": "Close", onClick: handleCloseClick, children: _jsx(CloseIcon, {}, void 0) }, void 0) }, void 0), showAppButton &&
                                _jsx(Box, { children: _jsx(Fab, { sx: {
                                            alignSelf: 'center',
                                            ...getHeadertyle()
                                        }, color: 'primary', "aria-label": "Show apps", onClick: handleMenuClick, children: _jsx(AppIcon, {}, void 0) }, void 0) }, void 0), showActionTileButton &&
                                _jsx(Box, { children: _jsx(Fab, { sx: {
                                            alignSelf: 'center',
                                            ...getHeadertyle()
                                        }, color: 'primary', "aria-label": "Show action tiles", onClick: props.onShowActionTiles, children: _jsx(ViewQuiltIcon, {}, void 0) }, void 0) }, void 0), _jsx(Box, { sx: {
                                    flex: 'auto',
                                    alignSelf: 'center',
                                    display: 'flex',
                                    alignContent: 'center',
                                    alignItems: 'center',
                                    minHeight: 56
                                }, children: _jsx(Box, { sx: {
                                        flex: 'auto',
                                        height: 40,
                                        ...getHeadertyle()
                                    }, boxShadow: 3, children: _jsx(Box, { sx: {
                                            height: '100%',
                                            display: 'flex',
                                            flexDirection: 'column',
                                            justifyItems: 'center',
                                            justifyContent: 'center',
                                            marginLeft: (theme) => theme.spacing(1),
                                            marginRight: (theme) => theme.spacing(1)
                                        }, children: _jsx(Typography, { variant: 'h5', children: props.headerText ? props.headerText : '' }, void 0) }, void 0) }, void 0) }, void 0)] }, void 0) }, void 0) }, void 0)] }, void 0));
};
