import React, { ReactElement } from 'react';
import { DotProps } from 'recharts';
import { IStandardChartConnector, IStandardChartData, IStandardChartReferenceLine, LocalizeMethod } from '@brudererdev/web-extensions';
export interface IStandardChartLabelContent {
    value: number | string;
    offset?: number;
    position?: 'top' | 'left' | 'right' | 'bottom' | 'inside' | 'outside' | 'insideLeft' | 'insideRight' | 'insideTop' | 'insideBottom' | 'insideTopLeft' | 'insideBottomLeft' | 'insideTopRight' | 'insideBottomRight' | 'insideStart' | 'insideEnd' | 'end' | 'center' | 'centerTop' | 'centerBottom' | 'middle' | {
        x?: number;
        y?: number;
    };
    content?: () => ReactElement;
    textBreakAll?: boolean;
    angle?: number;
}
export interface IEnvelopeStyle {
    stroke: string;
    fill: string;
    fillOpacity: number;
}
export interface IStandardChartSettings {
    maxHeight: number;
    maxWidth: number;
    topOffset?: number;
    leftOffset?: number;
    rightOffset?: number;
    hasEnvelope?: boolean;
    hasZoom?: boolean;
    showToolTip?: boolean;
    showLegend?: boolean;
    showDots?: boolean;
    domainTop?: number | string;
    domainBottom?: number | string;
    domainLeft?: number | string;
    domainRight?: number | string;
    xAxisHide?: boolean;
    xAxisAllowDecimals?: boolean;
    yAxisAffiliation?: 'Global' | 'PerConnector';
    yAxisHide?: boolean;
    yAxisAllowDecimals?: boolean;
    locale: string;
    labelFontSize?: number | string;
    envelopeStyle?: IEnvelopeStyle;
}
interface ILocalProps {
    chartData: Array<IStandardChartData>;
    connectors: Array<IStandardChartConnector>;
    xAxisReferenceLines?: Array<IStandardChartReferenceLine>;
    yAxisReferenceLines?: Array<IStandardChartReferenceLine>;
    settings: IStandardChartSettings;
    onXAxisTickCount?: () => number;
    onXAxisTickFormat?: (value: any, tickIndex: number) => any;
    onXAxisLabel?: () => IStandardChartLabelContent | undefined;
    onYAxisTickCount?: (axisIndex: number) => number;
    onYAxisTickFormat?: (value: any, tickIndex: number, axisIndex: number) => any;
    onYAxisLabel?: (axisIndex: number) => IStandardChartLabelContent | undefined;
    onClickActiveDot?: (dotProps: DotProps, dot: any) => void;
    onLocalize: LocalizeMethod;
}
declare type Props = ILocalProps;
export declare const StandardChart: React.FC<Props>;
export {};
