import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
// /* Recharts documentation on this link  https://recharts.org/en-US/ */
import React, { useEffect, useState } from 'react';
import { CartesianGrid, XAxis, YAxis, Line, Tooltip, Legend, Area, ComposedChart, ReferenceLine, ReferenceArea } from 'recharts';
import { TypeValidatorNET, createEmptyVariable, LocalizationNamespaces } from '@brudererdev/web-extensions';
import { Box, Typography, Fab } from '@mui/material';
import { useTheme } from '@mui/material/styles';
import ZoomOutMapIcon from '@mui/icons-material/ZoomOutMap';
import ImportExportIcon from '@mui/icons-material/ImportExport';
;
;
;
export const StandardChart = (props) => {
    // Fields
    const contextName = 'StandardChart';
    // External hooks
    const themeStyle = useTheme();
    // Helpers
    const topOffset = props.settings.topOffset !== undefined ? props.settings.topOffset : 8;
    const leftOffset = props.settings.leftOffset !== undefined ? props.settings.leftOffset : 8;
    const rightOffset = props.settings.rightOffset !== undefined ? props.settings.rightOffset : 8;
    const hasEnvelope = props.settings.hasEnvelope !== undefined ? props.settings.hasEnvelope : false;
    const hasZoom = props.settings.hasZoom !== undefined ? props.settings.hasZoom : true;
    const showToolTip = props.settings.showToolTip !== undefined ? props.settings.showToolTip : true;
    const showLegend = props.settings.showLegend !== undefined ? props.settings.showLegend : false;
    const showDots = props.settings.showDots !== undefined ? props.settings.showDots : false;
    const showSettingsArea = hasEnvelope || hasZoom;
    const connectors = props.connectors;
    const chartdata = props.chartData;
    const domainTop = props.settings.domainTop !== undefined ? props.settings.domainTop : 'auto';
    const domainBottom = props.settings.domainBottom !== undefined ? props.settings.domainBottom : 'auto';
    const domainLeft = props.settings.domainLeft !== undefined ? props.settings.domainLeft : 'auto';
    const domainRight = props.settings.domainRight !== undefined ? props.settings.domainRight : 'auto';
    const xAxisHide = props.settings.xAxisHide !== undefined ? props.settings.xAxisHide : false;
    const xAxisAllowDecimals = props.settings.xAxisAllowDecimals !== undefined ? props.settings.xAxisAllowDecimals : false;
    const yAxisAffiliation = props.settings.yAxisAffiliation !== undefined ? props.settings.yAxisAffiliation : 'Global';
    const yAxisHide = props.settings.yAxisHide !== undefined ? props.settings.yAxisHide : false;
    const yAxisAllowDecimals = props.settings.yAxisAllowDecimals !== undefined ? props.settings.yAxisAllowDecimals : false;
    const xAxisReferenceLines = props.xAxisReferenceLines !== undefined ? props.xAxisReferenceLines : [];
    const yAxisReferenceLines = props.yAxisReferenceLines !== undefined ? props.yAxisReferenceLines : [];
    const locale = props.settings.locale;
    const labelFontSize = props.settings.labelFontSize !== undefined ? props.settings.labelFontSize : 20;
    const envelopeStyle = (props.settings.envelopeStyle !== undefined) ? props.settings.envelopeStyle : {
        stroke: "#008B00",
        fill: "#00EE00",
        fillOpacity: 0.5,
    };
    const settingsAreaWidth = 72;
    // States
    const [isEnvelopeVisible, setShowEnvelope] = useState(hasEnvelope);
    const [domain, setDomain] = useState({
        refAreaLeft: 0, refAreaRight: 0, bottom: domainBottom, top: domainTop, left: domainLeft, right: domainRight
    });
    // Effects
    useEffect(() => {
        setDomain({ ...domain, left: domainLeft, right: domainRight, bottom: domainBottom, top: domainTop });
    }, [props.settings.domainTop, props.settings.domainBottom, props.settings.domainLeft, props.settings.domainRight]);
    // Variable
    const showEnvelopeVariable = createEmptyVariable();
    showEnvelopeVariable.display = {
        link: {
            key: 'views.elements.chart.envelope',
            path: 'views.elements.chart',
            name: 'envelope',
            seperator: '.'
        },
        keyNamespace: LocalizationNamespaces.UIGeneric,
        value: 'Hüllkurve'
    };
    showEnvelopeVariable.valueType = 'System.Boolean';
    showEnvelopeVariable.value = isEnvelopeVisible;
    showEnvelopeVariable.modelLink.key = "chart.envelope";
    const zoomIn = () => {
        var { refAreaLeft, refAreaRight } = domain;
        if (refAreaLeft === refAreaRight || refAreaRight === 0) {
            setDomain({ ...domain, refAreaLeft: 0, refAreaRight: 0 });
            return;
        }
        if (refAreaLeft > refAreaRight)
            [refAreaLeft, refAreaRight] = [refAreaRight, refAreaLeft];
        setDomain({ ...domain, refAreaLeft: 0, refAreaRight: 0, left: refAreaLeft, right: refAreaRight, top: domainTop, bottom: domainBottom });
    };
    const zoomOut = () => {
        setDomain({ ...domain, left: domainLeft, right: domainRight, bottom: domainBottom, top: domainTop });
    };
    const handleActiveDotClick = (dotProps, dot) => {
        if (props.onClickActiveDot)
            props.onClickActiveDot(dotProps, dot);
    };
    const getChartWidth = () => {
        if (!showSettingsArea)
            return props.settings.maxWidth;
        return props.settings.maxWidth - settingsAreaWidth;
    };
    const getXAxisTickCount = () => {
        if (props.onXAxisTickCount)
            return props.onXAxisTickCount();
        const chartWidth = getChartWidth();
        if (chartWidth < 300)
            return 1;
        return 5;
    };
    const getXAxisTickFormat = (value, tickIndex) => {
        if (props.onXAxisTickFormat)
            return props.onXAxisTickFormat(value, tickIndex);
        return value;
    };
    const getXAxisLabel = () => {
        if (props.onXAxisLabel)
            return props.onXAxisLabel();
        return undefined;
    };
    const getYAxisTickCount = (axisIndex) => {
        if (props.onYAxisTickCount)
            return props.onYAxisTickCount(axisIndex);
        return undefined;
    };
    const getYAxisTickFormat = (value, tickIndex, axisIndex) => {
        if (props.onYAxisTickFormat)
            return props.onYAxisTickFormat(value, tickIndex, axisIndex);
        return value;
    };
    const getYAxisLabel = (axisIndex) => {
        if (props.onYAxisLabel)
            return props.onYAxisLabel(axisIndex);
        return undefined;
    };
    const renderLegend = () => {
        if (!showLegend)
            return undefined;
        return (_jsx(Legend, { align: 'center', verticalAlign: 'bottom', content: (contentTypes) => {
                if (!contentTypes.payload || !contentTypes.payload.length)
                    return _jsx(React.Fragment, {}, void 0);
                // Set y Axis name
                var text = `${contentTypes.payload[0].value}`;
                return (_jsxs(Box, { sx: {
                        display: 'flex',
                        flexDirection: 'row',
                        alignContent: 'center',
                        alignItems: 'center'
                    }, children: [_jsx(Box, { sx: {
                                backgroundColor: contentTypes.payload[0].color,
                                margin: (theme) => theme.spacing(1),
                                width: 16,
                                height: 16
                            } }, void 0), _jsx(Typography, { variant: 'body1', children: text }, void 0)] }, void 0));
            } }, void 0));
    };
    const renderTooltip = () => {
        if (!showToolTip)
            return undefined;
        return (_jsx(Tooltip, { content: (contentTypes) => {
                if (!contentTypes.active || !contentTypes.payload || !contentTypes.payload.length)
                    return _jsx(React.Fragment, {}, void 0);
                // helpers
                var yAxisName = '';
                var yAxisColor;
                var envelopeName = '';
                var envelopeColor;
                var showEnvelope = false;
                // Set y Axis values (default axis value or envelope values)
                var variableY = chartdata[0].yAxes[0].modelVariable;
                var values = contentTypes.payload[0].value;
                if (variableY && values.length > 1) { //envelope available
                    yAxisName = `${contentTypes.payload[1].name}:` + ` ${TypeValidatorNET.formatValue(contentTypes.payload[1].value, variableY.valueType, locale, variableY.valuePrecision)} ${variableY.engineeringUnit?.displayName}`;
                    yAxisColor = contentTypes.payload[1].color;
                    envelopeName = `${contentTypes.payload[0].name}:`;
                    envelopeColor = contentTypes.payload[0].color;
                    var envelopePos = TypeValidatorNET.formatValue(values[0], variableY.valueType, locale, variableY.valuePrecision);
                    var envelopeNeg = TypeValidatorNET.formatValue(values[1], variableY.valueType, locale, variableY.valuePrecision);
                    envelopeName = envelopeName + ` [${envelopeNeg}, ${envelopePos}] ${variableY.engineeringUnit?.displayName}`;
                    showEnvelope = contentTypes.payload.length > 1 && !envelopeName.includes('NaN');
                }
                else { //no envelope available
                    yAxisName = `${contentTypes.payload[0].name}:` + ` ${TypeValidatorNET.formatValue(contentTypes.payload[0].value, variableY.valueType, locale, variableY.valuePrecision)} ${variableY.engineeringUnit?.displayName}`;
                    yAxisColor = contentTypes.payload[0].color;
                }
                ;
                // Set x Axis value
                var variableX = chartdata[0].xAxis.modelVariable;
                if (variableX)
                    yAxisName = yAxisName + ` @ ${TypeValidatorNET.formatValue(contentTypes.label, variableX.valueType, locale, variableX.valuePrecision)} ${variableX.engineeringUnit?.displayName}`;
                return (_jsxs(Box, { sx: {
                        display: 'flex',
                        flexDirection: 'column'
                    }, children: [_jsxs(Box, { sx: {
                                display: 'flex',
                                flexDirection: 'row',
                                alignContent: 'center',
                                alignItems: 'center'
                            }, children: [_jsx(Box, { sx: {
                                        backgroundColor: yAxisColor,
                                        margin: (theme) => theme.spacing(1),
                                        width: 16,
                                        height: 16
                                    } }, void 0), _jsx(Typography, { variant: 'body1', children: yAxisName }, void 0)] }, void 0), showEnvelope &&
                            _jsxs(Box, { sx: {
                                    display: 'flex',
                                    flexDirection: 'row',
                                    alignContent: 'center',
                                    alignItems: 'center'
                                }, children: [_jsx(Box, { sx: {
                                            backgroundColor: envelopeColor,
                                            margin: (theme) => theme.spacing(1),
                                            width: 16,
                                            height: 16
                                        } }, void 0), _jsx(Typography, { variant: 'body1', children: envelopeName }, void 0)] }, void 0)] }, void 0));
            } }, void 0));
    };
    const renderChartSettings = () => {
        if (!showSettingsArea)
            return undefined;
        return (_jsxs(Box, { sx: {
                height: props.settings.maxHeight,
                width: settingsAreaWidth,
                display: 'flex',
                flexDirection: 'column',
                alignContent: 'center',
                alignItems: 'center',
                justifyContent: 'center',
                justifyItems: 'center',
            }, children: [hasZoom &&
                    _jsx(Fab, { color: 'primary', "aria-label": "Reset zoom", onClick: () => {
                            zoomOut();
                        }, children: _jsx(ZoomOutMapIcon, {}, void 0) }, void 0), hasEnvelope &&
                    _jsx(Fab, { sx: {
                            marginTop: '16px'
                        }, color: isEnvelopeVisible ? 'secondary' : 'primary', "aria-label": "Show or hide envelope", onClick: () => {
                            setShowEnvelope(!isEnvelopeVisible);
                        }, children: _jsx(ImportExportIcon, {}, void 0) }, void 0)] }, void 0));
    };
    return (_jsxs(Box, { sx: {
            width: '100%',
            display: 'flex',
            flexDirection: 'row',
            fontSize: labelFontSize,
            height: props.settings.maxHeight
        }, children: [renderChartSettings(), _jsxs(ComposedChart, { data: props.chartData, margin: { top: topOffset, right: rightOffset, bottom: 16 }, width: getChartWidth(), height: props.settings.maxHeight, onMouseDown: (e) => {
                    if (!hasZoom)
                        return;
                    if (e && e.activeLabel)
                        setDomain({ ...domain, refAreaLeft: e.activeLabel });
                }, onMouseMove: (e) => {
                    if (!hasZoom)
                        return;
                    if (e && e.activeLabel)
                        setDomain({ ...domain, refAreaRight: e.activeLabel });
                }, onMouseUp: (e) => {
                    if (!hasZoom)
                        return;
                    zoomIn();
                }, children: [_jsx(CartesianGrid, { stroke: themeStyle.palette.grey[400], strokeDasharray: '3 3' }, void 0), _jsx(XAxis, { hide: xAxisHide, allowDataOverflow: true, allowDecimals: xAxisAllowDecimals, type: "number", dataKey: "xAxis.value", interval: 'preserveStartEnd', label: getXAxisLabel(), stroke: themeStyle.palette.grey[400], domain: [domain.left, domain.right], tickCount: getXAxisTickCount(), tickFormatter: getXAxisTickFormat, tick: {
                            fontSize: themeStyle.typography.body1.fontSize
                        } }, void 0), yAxisAffiliation === 'Global' &&
                        _jsx(YAxis, { hide: yAxisHide, allowDataOverflow: true, allowDecimals: yAxisAllowDecimals, type: "number", label: getYAxisLabel(0), stroke: themeStyle.palette.grey[400], domain: [domain.bottom, domain.top], tickCount: getYAxisTickCount(0), tickFormatter: (value, tickIndex) => getYAxisTickFormat(value, tickIndex, 0), tick: {
                                fontSize: themeStyle.typography.body1.fontSize
                            } }, void 0), ";", yAxisAffiliation === 'PerConnector' &&
                        connectors.map((connector, index) => {
                            const hide = (connector.yAxisHide !== undefined) ? connector.yAxisHide : yAxisHide;
                            const allowDecimals = (connector.yAxisAllowDecimals !== undefined) ? connector.yAxisAllowDecimals : yAxisAllowDecimals;
                            const domainBottom = (connector.domainBottom !== undefined) ? connector.domainBottom : 'auto';
                            const domainTop = (connector.domainTop !== undefined) ? connector.domainTop : 'auto';
                            return (_jsx(YAxis, { yAxisId: `yAxis_${index}`, hide: hide, allowDataOverflow: true, allowDecimals: allowDecimals, type: "number", label: getYAxisLabel(index), stroke: themeStyle.palette.grey[400], domain: [domainBottom, domainTop], tickCount: getYAxisTickCount(index), tickFormatter: (value, tickIndex) => getYAxisTickFormat(value, tickIndex, index), tick: {
                                    fontSize: themeStyle.typography.body1.fontSize
                                } }, `yAxis_${index}`));
                        }), ";", connectors.map((connector, index) => {
                        if (!isEnvelopeVisible)
                            return undefined;
                        var yAxisIndex = props.chartData[0].yAxes.findIndex((yAxisChartData) => yAxisChartData.modelLinkKey.toLowerCase() === connector.yAxisLinkKey.toLowerCase());
                        var localizedContendEnvelope = props.onLocalize({
                            link: {
                                key: 'views.elements.chart.envelope',
                                path: 'views.elements.chart',
                                name: 'envelope',
                                seperator: '.'
                            },
                            keyNamespace: LocalizationNamespaces.UIGeneric,
                            value: 'Hüllkurve'
                        });
                        var name = props.onLocalize(connector.label);
                        if (yAxisIndex < 0)
                            return undefined;
                        return (_jsx(Area, { yAxisId: (yAxisAffiliation === 'PerConnector') ? `yAxis_${index}` : undefined, animationDuration: 0, dataKey: `yAxes[${yAxisIndex}].envelopeValues`, stroke: envelopeStyle.stroke, strokeWidth: connector.strokeWidth, fill: envelopeStyle.fill, name: `${localizedContendEnvelope} ${name}`, fillOpacity: envelopeStyle.fillOpacity }, `Area_${index}`));
                    }), ";", connectors.map((connector, index) => {
                        var yAxisIndex = props.chartData[0].yAxes.findIndex((yAxisChartData) => yAxisChartData.modelLinkKey.toLowerCase() === connector.yAxisLinkKey.toLowerCase());
                        var name = "?";
                        var unit = undefined;
                        if (yAxisIndex < 0)
                            return undefined;
                        var variable = props.chartData[0].yAxes[yAxisIndex].modelVariable;
                        if (variable) {
                            name = props.onLocalize(variable.display);
                            unit = variable.engineeringUnit ? variable.engineeringUnit.displayName : undefined;
                        }
                        ;
                        if (showDots) {
                            return (_jsx(Line, { yAxisId: (yAxisAffiliation === 'PerConnector') ? `yAxis_${index}` : undefined, animationDuration: 10, type: connector.type, dataKey: `yAxes[${yAxisIndex}].value`, stroke: connector.stroke.toString(), name: name, unit: unit, strokeWidth: connector.strokeWidth, dot: {
                                    strokeWidth: 8
                                }, activeDot: {
                                    stroke: '#F73378', strokeWidth: 4, r: 8,
                                    onMouseDown: handleActiveDotClick
                                } }, `LineDots_${index}`));
                        }
                        return (_jsx(Line, { yAxisId: (yAxisAffiliation === 'PerConnector') ? `yAxis_${index}` : undefined, animationDuration: 0, type: connector.type, dataKey: `yAxes[${yAxisIndex}].value`, stroke: connector.stroke.toString(), name: name, unit: unit, strokeWidth: connector.strokeWidth, dot: false }, `Line_${index}`));
                    }), ";", renderTooltip(), renderLegend(), xAxisReferenceLines.map((referenceLine, index) => {
                        var localizedLabel = '';
                        if (referenceLine.label)
                            localizedLabel = props.onLocalize(referenceLine.label);
                        return (_jsx(ReferenceLine, { x: referenceLine.value, fontFamily: 'Roboto', stroke: referenceLine.stroke, strokeDasharray: referenceLine.strokeDasharray, strokeWidth: referenceLine.strokeWidth, label: localizedLabel }, `ReferenceLineX_${index}`));
                    }), ";", yAxisReferenceLines.map((referenceLine, index) => {
                        var localizedLabel = '';
                        if (referenceLine.label)
                            localizedLabel = props.onLocalize(referenceLine.label);
                        return (_jsx(ReferenceLine, { y: referenceLine.value, fontFamily: 'Roboto', stroke: referenceLine.stroke, strokeDasharray: referenceLine.strokeDasharray, strokeWidth: referenceLine.strokeWidth, label: localizedLabel }, `ReferenceLineY_${index}`));
                    }), ";", (domain.refAreaLeft && domain.refAreaRight) ? (_jsx(ReferenceArea, { x1: domain.refAreaLeft, x2: domain.refAreaRight, strokeOpacity: 0.3 }, void 0)) : null] }, void 0)] }, void 0));
};
