import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/* eslint-disable react/react-in-jsx-scope -- Unaware of jsxImportSource */
/** @jsxImportSource @emotion/react */
import { useEffect, useState } from 'react';
import { Box, Fab, MobileStepper } from '@mui/material';
import MoreHorizIcon from '@mui/icons-material/MoreHoriz';
import SwapHorizIcon from '@mui/icons-material/SwapHoriz';
import AutorenewIcon from '@mui/icons-material/Autorenew';
import { horizontalLineStyle } from './../../styles/themeExtensions';
import { defaultActionTileWidth, defaultActionTileHeight } from './../../shared/tileProps';
import { SizeInjectorContainer } from './../containers/SizeInjectorContainer';
import { SelectableMenu } from './../SelectableMenu';
export const ActionTile = (props) => {
    // States
    const [dropoutMenuAnchor, setDropoutMenuAnchor] = useState(null);
    const [layerIndex, setLayerIndex] = useState(1);
    const [contentIndex, setContentIndex] = useState(1);
    const [navigationElements, setNavigationElements] = useState([]);
    // Helpers
    const numberOfLayers = props.numberOfLayers !== undefined ? props.numberOfLayers : 1;
    const numberOfContents = props.numberOfContents !== undefined ? props.numberOfContents : 1;
    // Effects
    useEffect(() => {
        // Mount
        if (props.preSelectedContentIndex !== undefined)
            setContentIndex(props.preSelectedContentIndex);
        if (props.preSelectedLayerindex !== undefined)
            setLayerIndex(props.preSelectedLayerindex);
        if (props.elements !== undefined) {
            setNavigationElements(getSelectableElements(props.elements));
        }
    }, []);
    const getSelectableElements = (elements) => {
        var selectableElements = [];
        elements.forEach((element) => {
            if (element.isSelectable !== undefined) {
                if (element.isSelectable)
                    selectableElements.push(element);
            }
            else {
                selectableElements.push(element);
            }
        });
        return selectableElements;
    };
    const handleLayerBottomClick = (e) => {
        setLayerIndex((layerIndex % numberOfLayers) + 1);
    };
    const handleContentBottomClick = (e) => {
        setContentIndex((contentIndex % numberOfContents) + 1);
    };
    const handleLabelButtonClick = (e) => {
        if (props.onLabelButtonClick)
            props.onLabelButtonClick(e);
        if (props.elements !== undefined)
            setDropoutMenuAnchor(dropoutMenuAnchor ? null : e.currentTarget);
    };
    const handleMenuSelect = (e, element, index) => {
        setDropoutMenuAnchor(null);
        if (props.onElementSelect)
            props.onElementSelect(e, element, index);
    };
    const getContentStyle = () => {
        var contentStyle = props.contentStyle;
        if (!contentStyle)
            contentStyle = {
                backgroundColor: (theme) => theme.palette.view.main,
                color: (theme) => theme.palette.view.contrastText,
                fill: (theme) => theme.palette.view.contrastText
            };
        return contentStyle;
    };
    const getLabelStyle = () => {
        var labelStyle = props.labelStyle;
        if (!labelStyle)
            labelStyle = {
                backgroundColor: (theme) => theme.palette.primary.main,
                color: (theme) => theme.palette.primary.contrastText,
                fill: (theme) => theme.palette.primary.contrastText,
                ':hover': {
                    backgroundColor: (theme) => theme.palette.primary.main,
                    color: (theme) => theme.palette.primary.contrastText,
                    fill: (theme) => theme.palette.primary.contrastText,
                },
            };
        return labelStyle;
    };
    const getLabelButtonStyle = () => {
        var labelButtonStyle = props.labelButtonStyle;
        if (!labelButtonStyle)
            labelButtonStyle = {
                backgroundColor: (theme) => theme.palette.primary.main,
                color: (theme) => theme.palette.primary.contrastText,
                fill: (theme) => theme.palette.primary.contrastText,
                ':hover': {
                    backgroundColor: (theme) => theme.palette.primary.main,
                    color: (theme) => theme.palette.primary.contrastText,
                    fill: (theme) => theme.palette.primary.contrastText,
                },
            };
        return labelButtonStyle;
    };
    const renderIcon = (IconElement) => {
        if (!IconElement)
            return;
        if (typeof IconElement === 'string')
            return (_jsx(Box, { component: 'span', sx: { fontSize: 24 }, children: IconElement }, void 0));
        return (_jsx(IconElement, { css: {
                width: 32,
                height: 32,
                minWidth: 32,
                minHeight: 32
            } }, void 0));
    };
    const renderFooter = () => {
        if (!props.onRenderFooter)
            return undefined;
        return (_jsxs(Box, { sx: {
                minHeight: 40,
                justifySelf: 'flex-end',
                margin: (theme) => theme.spacing(1)
            }, children: [_jsx(Box, { component: 'hr', sx: {
                        ...horizontalLineStyle
                    } }, void 0), _jsx(Box, { sx: {
                        display: 'flex',
                        flexDirection: 'row',
                        alignContent: 'center',
                        alignItems: 'center',
                        justifyContent: 'center',
                        justifyItems: 'center',
                        minHeight: (theme) => theme.spacing(1)
                    } }, void 0), props.onRenderFooter()] }, void 0));
    };
    const renderStepper = () => {
        if (numberOfContents < 2)
            return undefined;
        return (_jsx(MobileStepper, { variant: "dots", sx: {
                backgroundColor: (theme) => theme.palette.view.main,
                paddingTop: 0,
                paddingBottom: (theme) => theme.spacing(1),
                paddingLeft: (theme) => theme.spacing(1),
                paddingRight: (theme) => theme.spacing(1)
            }, steps: numberOfContents, position: "static", activeStep: contentIndex - 1, nextButton: _jsx("div", {}, void 0), backButton: _jsx("div", {}, void 0) }, void 0));
    };
    const renderLabel = () => {
        return (_jsx(Box, { sx: {
                flex: 'auto',
                height: 40,
                display: 'flex',
                flexDirection: 'row',
                alignContent: 'center',
                alignItems: 'center',
                justifyContent: 'center',
                justifyItems: 'center',
                marginRight: (theme) => theme.spacing(1),
                ...getLabelStyle()
            }, boxShadow: 3, children: props.onRenderHeader &&
                _jsx(Box, { sx: {
                        flex: 'auto',
                        marginLeft: (theme) => theme.spacing(1)
                    }, onClick: props.onHeaderClick, children: props.onRenderHeader() }, void 0) }, void 0));
    };
    const renderLayerSwapButton = () => {
        if (numberOfLayers == 1)
            return;
        return (_jsx(Fab, { sx: {
                marginRight: (theme) => theme.spacing(1),
                ...getLabelButtonStyle()
            }, "aria-label": "interate layers", onClick: handleLayerBottomClick, children: renderIcon(SwapHorizIcon) }, void 0));
    };
    const renderContentSwapButton = () => {
        if (numberOfContents == 1)
            return;
        return (_jsx(Fab, { sx: {
                marginRight: (theme) => theme.spacing(1),
                ...getLabelButtonStyle()
            }, "aria-label": "interate content views", onClick: handleContentBottomClick, children: renderIcon(AutorenewIcon) }, void 0));
    };
    const renderContextSelectionButton = () => {
        var IconElement = props.labelButtonIcon;
        if (IconElement === undefined)
            IconElement = MoreHorizIcon;
        return (_jsx(Fab, { sx: {
                marginRight: (theme) => theme.spacing(1),
                ...getLabelButtonStyle()
            }, "aria-label": "Select a subview", onClick: handleLabelButtonClick, children: renderIcon(IconElement) }, void 0));
    };
    // Helpers
    const minHeight = (props.minHeight != undefined) ? props.minHeight : defaultActionTileHeight;
    const minWidth = (props.minWidth != undefined) ? props.minWidth : defaultActionTileWidth;
    const height = (props.height != undefined) ? props.height : defaultActionTileHeight;
    return (_jsxs(Box, { sx: {
            position: 'relative',
            width: '100%',
            height: '100%',
            display: 'flex',
            flexDirection: 'column'
        }, children: [_jsx(Box, { sx: { minHeight: (theme) => theme.spacing(2) } }, void 0), _jsxs(Box, { sx: {
                    minHeight: minHeight,
                    minWidth: minWidth,
                    height: height,
                    width: props.width,
                    flex: 'auto',
                    display: 'flex',
                    flexDirection: 'column',
                    overflow: 'hidden',
                    ...getContentStyle()
                }, boxShadow: 5, children: [_jsx(Box, { sx: { minHeight: (theme) => theme.spacing(2) } }, void 0), _jsx(Box, { sx: {
                            overflow: 'hidden',
                            flex: 'auto',
                            margin: (theme) => theme.spacing(1)
                        }, onClick: props.onContentClick, children: _jsx(SizeInjectorContainer, { isScrollLocked: props.isScrollLocked, children: props.onRenderContent && props.onRenderContent(layerIndex, contentIndex) }, void 0) }, void 0), renderFooter(), renderStepper()] }, void 0), _jsx(Box, { sx: {
                    position: 'absolute',
                    width: '100%'
                }, children: _jsxs(Box, { sx: {
                        position: 'relative',
                        display: 'flex',
                        flexDirection: 'row',
                        alignContent: 'center',
                        alignItems: 'center',
                        minHeight: 56,
                        marginLeft: (theme) => theme.spacing(1),
                    }, children: [renderLabel(), renderLayerSwapButton(), renderContentSwapButton(), renderContextSelectionButton()] }, void 0) }, void 0), _jsx(SelectableMenu, { anchor: dropoutMenuAnchor, elements: navigationElements, selectedElementIndex: props.selectedElementIndex, onLocalize: props.onLocalize, onSelect: handleMenuSelect, onClose: () => setDropoutMenuAnchor(null) }, void 0)] }, void 0));
};
