import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useContext } from 'react';
import { LocalizationNamespaces } from '@brudererdev/web-extensions';
import { makeStyles, createStyles } from '@material-ui/core/styles';
import SettingsInputHdmiIcon from '@material-ui/icons/SettingsInputHdmi';
import InfoIcon from '@material-ui/icons/Info';
import { Box, Typography } from '@material-ui/core';
import { BrudererContext } from '../..';
import { SizeInjectorContainer } from '../containers';
const useStyles = makeStyles((theme) => createStyles({
    default: {
        backgroundColor: theme.palette.view.main,
        color: theme.palette.view.contrastText,
        fill: theme.palette.view.contrastText
    },
    info: {
        backgroundColor: theme.palette.primary.main,
        color: theme.palette.primary.contrastText,
        fill: theme.palette.primary.contrastText
    },
    warning: {
        backgroundColor: theme.palette.warning.main,
        color: theme.palette.warning.contrastText,
        fill: theme.palette.warning.contrastText
    },
    error: {
        backgroundColor: theme.palette.error.main,
        color: theme.palette.error.contrastText,
        fill: theme.palette.error.contrastText
    },
    assistantScrollContainer: {
        display: 'flex',
        flexDirection: 'row',
        alignContent: 'center',
        alignItems: 'center'
    },
    assistantContent: {
        minHeight: 0,
        textAlign: 'left'
    }
}));
export const AssistantMessages = (props) => {
    // Fields
    const contextName = 'Assistant';
    // External hooks
    const styleClasses = useStyles();
    // Contexts
    const brudererContext = useContext(BrudererContext);
    const getSizes = () => {
        switch (props.type) {
            case 'messageBar':
                iconSizeValue = 32;
                iconWrapperWidth = 64;
                typographyVariant = 'h6';
                break;
            case 'actionTile':
            default:
                iconSizeValue = 64;
                iconWrapperWidth = 100;
                typographyVariant = 'h5';
                break;
        }
    };
    const getContextIcon = (context) => {
        switch (context) {
            case 'BackendConnectionError':
            case 'PLCConnectionError':
                return (_jsx(SettingsInputHdmiIcon, { style: {
                        minWidth: `${iconSizeValue}`,
                        minHeight: `${iconSizeValue}`,
                        width: `${iconSizeValue}`,
                        height: `${iconSizeValue}`
                    }, className: 'animated1' }, void 0)); /* TO DO: change animation to new style */
            case 'InfoMessages':
            case 'ToolContext':
            case 'ErrorMessages':
                return (_jsx(InfoIcon, { style: {
                        minWidth: `${iconSizeValue}`,
                        minHeight: `${iconSizeValue}`,
                        width: `${iconSizeValue}`,
                        height: `${iconSizeValue}`
                    }, className: 'animated1' }, void 0)); /* TO DO: change animation to new style */
            case 'Default':
            default:
                return null;
        }
    };
    const getAssistanceText = (context) => {
        switch (context) {
            case 'BackendConnectionError':
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.backenderrorhelp', 'Es scheint ein Problem mit der Verbindung zum Backend zu geben. Tippen Sie hier, um Hilfe zu erhalten.'));
            case 'PLCConnectionError':
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.plcconnectionerrorhelp', 'Es scheint ein Problem mit der Verbindung zur SPS zu geben. Tippen Sie hier, um Hilfe zu erhalten.'));
            case 'InfoMessages':
                var localizedMessageText = '?';
                if (activeInfoMessages.length > 0) {
                    localizedMessageText = activeInfoMessages[0].display.value;
                    if (!activeInfoMessages[0].hasCustomText)
                        localizedMessageText = brudererContext.localize(activeInfoMessages[0].display);
                }
                return (localizedMessageText);
            case 'ToolContext':
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.tooldataerrorhelp', 'Es scheint ein Problem mit ihren Werkzeugdaten zu geben. Tippen Sie hier um die Daten zu editieren.'));
            case 'ErrorMessages':
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.errormessagehelp', 'Es stehen ein oder mehrere Fehler an. Tippen Sie hier, um Hilfe zu erhalten.'));
            case 'Default':
            default:
                return null;
        }
    };
    const getContext = () => {
        if (brudererContext.failedServices.length > 0) {
            if (props.onContextStyleClassesChange)
                props.onContextStyleClassesChange(styleClasses.error);
            return 'BackendConnectionError';
        }
        ;
        if (!brudererContext.isPLCConnected) {
            if (props.onContextStyleClassesChange)
                props.onContextStyleClassesChange(styleClasses.error);
            return 'PLCConnectionError';
        }
        ;
        if (activeInfoMessages.length > 0) {
            if (props.onContextStyleClassesChange)
                props.onContextStyleClassesChange(styleClasses.info);
            return 'InfoMessages';
        }
        ;
        if (props.noToolLoaded) {
            if (props.onContextStyleClassesChange)
                props.onContextStyleClassesChange(styleClasses.info);
            return 'ToolContext';
        }
        ;
        if (activeErrorMessages.length > 0) {
            if (props.onContextStyleClassesChange)
                props.onContextStyleClassesChange(styleClasses.error);
            return 'ErrorMessages';
        }
        ;
        if (props.onContextStyleClassesChange)
            props.onContextStyleClassesChange(styleClasses.default);
        return 'Default';
    };
    const renderContent = () => {
        return (_jsxs(Box, Object.assign({ style: {
                height: '100%',
                display: 'flex',
                flexDirection: 'row',
                alignContent: 'center',
                alignItems: 'center'
            }, onClick: () => {
                if (props.onClick)
                    props.onClick(assistantContext);
            } }, { children: [_jsx(Box, Object.assign({ minWidth: iconWrapperWidth, display: 'flex', flexDirection: 'column', alignContent: 'center', alignItems: 'center' }, { children: getContextIcon(assistantContext) }), void 0), _jsx(SizeInjectorContainer, Object.assign({ className: styleClasses.assistantScrollContainer }, { children: _jsx(Box, Object.assign({ className: `${styleClasses.assistantContent}` }, { children: _jsx(Typography, Object.assign({ variant: `${typographyVariant}` }, { children: getAssistanceText(assistantContext) }), void 0) }), void 0) }), void 0)] }), void 0));
    };
    // Helpers
    var iconSizeValue;
    var iconWrapperWidth;
    var typographyVariant;
    getSizes();
    const activeInfoMessages = props.activeInfoMessages ? props.activeInfoMessages : [];
    const activeErrorMessages = props.activeErrorMessages ? props.activeErrorMessages : [];
    const assistantContext = getContext();
    return (_jsx(React.Fragment, { children: renderContent() }, void 0));
};
