import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState, useRef, useEffect } from 'react';
import { Box, Typography } from '@mui/material';
;
export const CircleSegment = (props) => {
    // States
    const [height, setHeight] = useState(0);
    const [width, setWidth] = useState(0);
    // Refs
    const containerRef = useRef(null);
    // Effects
    useEffect(() => {
        if (containerRef.current !== null)
            if (containerRef.current.offsetHeight !== height)
                setHeight(containerRef.current.offsetHeight);
        if (containerRef.current !== null)
            if (containerRef.current.offsetWidth !== width)
                setWidth(containerRef.current.offsetWidth);
    });
    // Helpers
    const backgroundColor = (props.backgroundColor !== undefined) ? props.backgroundColor : '#555555';
    const labelColor = (props.labelColor !== undefined) ? props.labelColor : '#bbbbbb';
    const rotation = (props.rotation !== undefined) ? props.rotation : 'CW';
    const showAngleSteps = (props.showAngleSteps !== undefined) ? props.showAngleSteps : true;
    const angleStepSize = (props.angleStepSize !== undefined) ? props.angleStepSize : 10.0;
    const showAngleLabels = (props.showAngleLabels !== undefined) ? props.showAngleLabels : true;
    const angleLabels = (props.angleLabels !== undefined) ? props.angleLabels : [0, 90, 180, 270];
    const getMaxSize = () => {
        var maxSize = width < height ? width : height;
        if (maxSize <= 0)
            maxSize = 64;
        return maxSize;
    };
    const getSize = () => {
        var size = maxSize;
        if (props.size && props.size <= maxSize)
            size = props.size;
        size -= margin;
        if (showAngleLabels)
            size -= 40;
        return size;
    };
    const renderSegments = () => {
        return (_jsxs(React.Fragment, { children: [_jsx(Box, { sx: {
                        position: 'absolute',
                        height: size,
                        width: size,
                        borderRadius: '50%',
                        background: `conic-gradient(${backgroundColor} 0deg 0deg)`
                    } }, void 0), props.segments.map((segment, index) => {
                    var startAngle = segment.startAngle;
                    var endAngle = segment.endAngle;
                    if (rotation === 'CCW') {
                        startAngle = 360.0 - segment.endAngle;
                        endAngle = 360.0 - segment.startAngle;
                    }
                    var conicGradientCSS = 'conic-gradient(';
                    if (startAngle > endAngle) {
                        conicGradientCSS += `${segment.color} 0deg ${endAngle}deg, `;
                        conicGradientCSS += `transparent ${endAngle}deg ${startAngle}deg, `;
                        conicGradientCSS += `${segment.color} 0deg)`;
                    }
                    else {
                        if (startAngle > 0) {
                            conicGradientCSS += `transparent 0deg ${startAngle}deg, `;
                            conicGradientCSS += `${segment.color} ${startAngle}deg ${endAngle}deg, transparent 0deg)`;
                        }
                        else {
                            conicGradientCSS += `${segment.color} ${startAngle}deg ${endAngle}deg, transparent 0deg)`;
                        }
                    }
                    return (_jsx(Box, { sx: {
                            position: 'absolute',
                            height: size,
                            width: size,
                            borderRadius: '50%',
                            background: conicGradientCSS
                        } }, index));
                })] }, void 0));
    };
    const renderAngleSteps = () => {
        const angleSteps = 360 / angleStepSize;
        const stepLineSize = 10.0;
        const stepLineThickness = 3.0;
        const stepLineRadius = (size / 2) - (stepLineSize / 2);
        const stepElements = [];
        for (var i = 0; i < angleSteps; i++) {
            stepElements.push(_jsx(Box, { sx: {
                    height: stepLineSize,
                    width: stepLineThickness,
                    backgroundColor: labelColor,
                    position: 'absolute',
                    transform: `rotate(${(i * angleStepSize) + 180}deg) translate(${0}px, ${stepLineRadius}px)`,
                    filter: 'blur(0)'
                } }, `Step_${i}`));
        }
        return stepElements;
    };
    const renderAngleLabels = () => {
        const labelRadius = (size / 2) + 20;
        return (_jsxs(React.Fragment, { children: [angleLabels.map((rawAngle, index) => {
                    var angle = rawAngle;
                    if (rotation === 'CCW')
                        angle *= -1;
                    return (_jsx(Box, { sx: {
                            zIndex: 3,
                            position: 'absolute',
                            transform: `rotate(${angle + 180}deg) translate(${0}px, ${labelRadius}px) rotate(${(angle * -1) + 180}deg)`,
                            filter: 'blur(0)'
                        }, children: _jsx(Typography, { variant: 'body1', children: `${angle}°` }, void 0) }, `Label_${index}`));
                }), ";"] }, void 0));
    };
    // Helpers
    const margin = 16;
    var maxSize = getMaxSize();
    const size = getSize();
    return (_jsxs(Box, { ref: containerRef, sx: {
            position: 'relative',
            height: '100%',
            width: '100%',
            display: 'flex',
            flexDirection: 'column',
            alignContent: 'center',
            alignItems: 'center',
            justifyContent: 'center',
            justifyItems: 'center'
        }, children: [renderSegments(), showAngleSteps && renderAngleSteps(), showAngleLabels && renderAngleLabels()] }, void 0));
};
