import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState, useRef, useEffect } from 'react';
import { Box } from '@mui/material';
import { defaultNumericLayout } from './../../shared/keyboardLayouts';
import { Tile } from './Tile';
export const Keyboard = (props) => {
    // States
    const [isShiftActive, setShiftActive] = useState(false);
    const [height, setHeight] = useState(0);
    const [width, setWidth] = useState(0);
    // Refs
    const keyboardContainerRef = useRef(null);
    // Effects
    useEffect(() => {
        if (keyboardContainerRef.current !== null)
            if (keyboardContainerRef.current.offsetHeight !== height)
                setHeight(keyboardContainerRef.current.offsetHeight);
        if (keyboardContainerRef.current !== null)
            if (keyboardContainerRef.current.offsetWidth !== width)
                setWidth(keyboardContainerRef.current.offsetWidth);
    });
    const getKeyStyle = (keyValue) => {
        var keyStyle = {
            backgroundColor: (theme) => theme.palette.primary.main,
            color: (theme) => theme.palette.primary.contrastText,
            fill: (theme) => theme.palette.primary.contrastText
        };
        if (isShiftActive && (keyValue === '{shift}'))
            keyStyle = {
                backgroundColor: (theme) => theme.palette.secondary.main,
                color: (theme) => theme.palette.secondary.contrastText,
                fill: (theme) => theme.palette.secondary.contrastText
            };
        var keyRootStyle = {
            borderRadius: (theme) => theme.spacing(1),
            ...keyStyle
        };
        return keyRootStyle;
    };
    const renderKeys = (keyRow) => {
        return (_jsx(React.Fragment, { children: keyRow.keys.map((key, index) => {
                // Setup key props
                const growSize = key.growSize ? key.growSize : 1;
                const keyWidth = key.width ? key.width : keySize;
                const keyHeight = key.height ? key.height : keySize;
                const basisSize = (growSize * keyWidth) + ((growSize - 1) * keyGapSize);
                const canRenderGap = (keyRow.keys.length > 1) && (index < (keyRow.keys.length - 1));
                // Define key value
                var keyValue = key.value;
                if (isShiftActive)
                    if (key.valueShift !== undefined)
                        keyValue = key.valueShift;
                // Define key sign
                var keySign = key.sign;
                if (keySign === undefined)
                    if (isShiftActive && (key.valueShift !== undefined))
                        keySign = key.valueShift;
                    else
                        keySign = key.value;
                return (_jsxs(React.Fragment, { children: [_jsx(Box, { sx: {
                                maxWidth: `${basisSize}px`,
                                display: 'flex',
                                flexGrow: growSize,
                                flexBasis: `${basisSize}px`
                            }, children: _jsx(Tile, { tileStyle: getKeyStyle(keyValue), minHeight: keyHeight, minWidth: keyWidth, height: '100%', width: '100%', onClick: (e) => {
                                    if (keyValue === '{shift}') {
                                        setShiftActive(!isShiftActive);
                                        return;
                                    }
                                    if (keyValue === '{space}')
                                        keyValue = ' ';
                                    props.onKeyClick(keyValue, Date.now());
                                }, children: _jsx(Box, { sx: {
                                        maxHeight: keyHeight,
                                        fontSize: key.signSize ? key.signSize : (theme) => theme.typography.h3.fontSize
                                    }, children: keySign }, void 0) }, void 0) }, void 0), canRenderGap &&
                            _jsx(Box, { sx: { width: keyGapSize } }, void 0)] }, index));
            }) }, void 0));
    };
    const renderKeyRows = (keyRows) => {
        return (_jsx(React.Fragment, { children: keyRows.map((keyRow, index) => {
                const growSize = keyRow.growSize ? keyRow.growSize : 1;
                const basisSize = (growSize * keySize) + ((growSize - 1) * keyGapSize);
                const canRenderGap = (keyRows.length > 1) && (index < (keyRows.length - 1));
                return (_jsxs(React.Fragment, { children: [_jsx(Box, { sx: {
                                display: 'flex',
                                flexGrow: growSize,
                                flexBasis: `${basisSize}px`,
                                justifyContent: 'center',
                                justifyItems: 'center'
                            }, children: renderKeys(keyRow) }, void 0), canRenderGap &&
                            _jsx(Box, { sx: { height: keyGapSize } }, void 0)] }, index));
            }) }, void 0));
    };
    const renderLayout = () => {
        return (_jsx(Box, { ref: keyboardContainerRef, sx: {
                display: 'flex',
                flexDirection: 'row'
            }, children: layout.map((keyRows, index) => {
                const canRenderGap = (keyRows.length > 1) && (index < (keyRows.length - 1));
                return (_jsxs(React.Fragment, { children: [_jsx(Box, { sx: {
                                display: 'flex',
                                flexDirection: 'column'
                            }, children: renderKeyRows(keyRows) }, void 0), canRenderGap &&
                            _jsx(Box, { sx: {
                                    minWidth: keyGapSize,
                                    width: keyGapSize
                                } }, void 0)] }, index));
            }) }, void 0));
    };
    // Helpers
    const layout = props.layout ? props.layout : defaultNumericLayout;
    const keySize = props.keySize ? props.keySize : 60;
    const keyGapSize = props.keyGapSize ? props.keyGapSize : 8;
    return (_jsx(Box, { sx: {
            height: '100%',
            display: 'flex',
            flexDirection: 'column'
        }, children: renderLayout() }, void 0));
};
