import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, Typography } from '@mui/material';
export const LabelMark = (props) => {
    const getMarkerTopOffset = () => {
        return ((props.height / 2) - (markerSize / 2)) * 100 / props.height;
    };
    const getMarkerLeftOffset = () => {
        switch (markerOrientation) {
            case 'Left':
                return 0;
            case 'Bottom':
                return ((props.width / 2) - (markerSize / 2)) * 100 / props.width;
            case 'Top':
                return ((props.width / 2) - (markerSize / 2)) * 100 / props.width;
            case 'Right':
                return (props.width - markerSize) * 100 / props.width;
            default:
                return ((props.width / 2) - (markerSize / 2)) * 100 / props.width;
        }
    };
    const getMarkerRotation = () => {
        const rotationOffset = 45;
        switch (markerOrientation) {
            case 'Left':
                return rotationOffset;
            case 'Bottom':
                return rotationOffset + 270;
            case 'Top':
                return rotationOffset + 90;
            case 'Right':
                return rotationOffset + 180;
            default:
                if (typeof (markerOrientation) === 'number')
                    return rotationOffset + markerOrientation;
                return rotationOffset + 270;
        }
    };
    const getMarkerStyle = () => {
        var markerStyle = props.markerStyle;
        if (!markerStyle)
            markerStyle = {
                backgroundColor: (theme) => theme.palette.primary.main,
                color: (theme) => theme.palette.primary.contrastText,
                fill: (theme) => theme.palette.primary.contrastText
            };
        return markerStyle;
    };
    const getCorrectionLabelPosition = () => {
        if (!props.correctionLabelValue) {
            return null;
        }
        switch (markerOrientation) {
            case 'Left':
            case 'Right':
            case 'None':
                {
                    if (props.correctionLabelValue > 0 || (!(props.correctionLabelValue > 0) && props.correctionLabelInvertPosition))
                        return 'Top';
                    return 'Bottom';
                }
            case 'Bottom':
            case 'Top':
                {
                    if (props.correctionLabelValue > 0 || (!(props.correctionLabelValue > 0) && props.correctionLabelInvertPosition))
                        return 'Right';
                    return 'Left';
                }
            default:
                return null;
        }
    };
    const getCorrectionLabelTopOffset = () => {
        switch (correctionLabelPosition) {
            case 'Top':
                return -((correctionLabelHeight) * 100 / props.height);
            case 'Bottom':
                return ((props.height) * 100 / props.height);
            case 'Right':
            case 'Left':
                {
                    return ((props.height - correctionLabelHeight) / 2 * 100 / props.height);
                }
            default:
                return null;
        }
    };
    const getCorrectionLabelLeftOffset = () => {
        switch (correctionLabelPosition) {
            case 'Bottom':
            case 'Top':
                return ((props.width - correctionLabelWidth) / 2 * 100 / props.width);
            case 'Right':
                return ((props.width - borderRadius / 4) * 100 / props.width);
            case 'Left':
            default:
                return ((borderRadius / 4 - correctionLabelWidth) * 100 / props.width);
        }
    };
    const getCorrectionLabelRadius = () => {
        switch (correctionLabelPosition) {
            case 'Bottom':
                return `0px 0px ${correctionLabelRadiusSize}px ${correctionLabelRadiusSize}px`;
            case 'Top':
                return `${correctionLabelRadiusSize}px ${correctionLabelRadiusSize}px 0px 0px`;
            case 'Right':
                return `0px ${correctionLabelRadiusSize}px ${correctionLabelRadiusSize}px 0px`;
            case 'Left':
                return `${correctionLabelRadiusSize}px 0px 0px ${correctionLabelRadiusSize}px`;
            default:
                return `${correctionLabelRadiusSize}px ${correctionLabelRadiusSize}px ${correctionLabelRadiusSize}px ${correctionLabelRadiusSize}px`;
        }
    };
    const getCorrectionLabelStyle = () => {
        var correctionLabelStyle = props.correctionLabelStyle;
        if (!correctionLabelStyle)
            correctionLabelStyle = {
                backgroundColor: (theme) => theme.palette.secondary.main,
                color: (theme) => theme.palette.secondary.contrastText,
                fill: (theme) => theme.palette.secondary.contrastText
            };
        return correctionLabelStyle;
    };
    const renderCorrectionLabel = () => {
        if (!(props.correctionLabelValue) || !(Math.abs(props.correctionLabelValue) > correctionLabelLimit))
            return null;
        return (_jsx(Box, { sx: {
                position: 'absolute',
                display: 'flex',
                alignContent: 'center',
                alignItems: 'center',
                justifyContent: 'center',
                justifyItems: 'center',
                top: `${getCorrectionLabelTopOffset()}%`,
                left: `${getCorrectionLabelLeftOffset()}%`,
                height: correctionLabelHeight,
                width: correctionLabelWidth,
                borderRadius: getCorrectionLabelRadius(),
                ...getCorrectionLabelStyle()
            }, children: _jsxs(Box, { sx: {
                    display: 'flex',
                    flexDirection: 'row',
                    alignItems: 'baseline'
                }, children: [_jsx(Typography, { variant: correctionLabelTypographyVariant, children: (props.correctionLabelValue > 0) ? `+${props.correctionLabelValueFormatted}` : `${props.correctionLabelValueFormatted}` }, void 0), _jsx(Box, { sx: { minWidth: 4 } }, void 0), _jsx(Typography, { variant: 'h6', children: props.correctionLabelUnit }, void 0)] }, void 0) }, void 0));
    };
    // Helpers
    const borderRadius = props.borderRadius ? props.borderRadius : props.height / 100 * 30;
    const top = (props.top !== undefined) ? props.top : undefined;
    const left = (props.left !== undefined) ? props.left : undefined;
    const markerSize = (props.markerSize !== undefined) ? props.markerSize : props.height;
    const markerOrientation = (props.markerOrientation !== undefined) ? props.markerOrientation : 'Bottom';
    const correctionLabelPosition = props.correctionLabelFixedPosition ? props.correctionLabelFixedPosition : getCorrectionLabelPosition();
    const correctionLabelHeight = props.height - borderRadius;
    const correctionLabelWidth = props.width * 3 / 4;
    const correctionLabelRadiusSize = 0.2 * correctionLabelHeight;
    const correctionLabelTypographyVariant = props.correctionLabelTypographyVariant ? props.correctionLabelTypographyVariant : 'h6';
    const correctionLabelLimit = (props.correctionLabelLimit !== undefined) ? props.correctionLabelLimit : 0.005;
    return (_jsxs(Box, { sx: {
            position: 'absolute',
            height: 'fit-content',
            width: 'fit-content',
            top: top,
            left: left
        }, children: [(markerOrientation !== 'None') &&
                _jsx(Box, { sx: {
                        position: 'absolute',
                        top: `${getMarkerTopOffset()}%`,
                        left: `${getMarkerLeftOffset()}%`,
                        height: markerSize,
                        width: markerSize,
                        borderRadius: '50% 50% 50% 0%',
                        transform: `rotate(${getMarkerRotation()}deg)`,
                        ...getMarkerStyle()
                    } }, void 0), renderCorrectionLabel(), _jsx(Box, { sx: {
                    position: 'relative',
                    display: 'flex',
                    alignContent: 'center',
                    alignItems: 'center',
                    justifyContent: 'center',
                    justifyItems: 'center',
                    borderRadius: borderRadius,
                    height: props.height,
                    width: props.width,
                    ...getMarkerStyle()
                }, onClick: props.onLabelClick, children: props.children }, void 0)] }, void 0));
};
