import { jsx as _jsx } from "@emotion/react/jsx-runtime";
/* eslint-disable react/react-in-jsx-scope -- Unaware of jsxImportSource */
/** @jsxImportSource @emotion/react */
import { useState, useRef, useEffect } from 'react';
import { Box } from '@mui/material';
import RemoveCircleOutlineIcon from '@mui/icons-material/RemoveCircleOutline';
import CheckCircleOutlineIcon from '@mui/icons-material/CheckCircleOutline';
import WarningIcon from '@mui/icons-material/Warning';
import InfoIcon from '@mui/icons-material/Info';
import ErrorOutlineIcon from '@mui/icons-material/ErrorOutline';
export var ElementStatusEnumeration;
(function (ElementStatusEnumeration) {
    ElementStatusEnumeration[ElementStatusEnumeration["Inactive"] = 0] = "Inactive";
    ElementStatusEnumeration[ElementStatusEnumeration["Active"] = 1] = "Active";
    ElementStatusEnumeration[ElementStatusEnumeration["Warning"] = 2] = "Warning";
    ElementStatusEnumeration[ElementStatusEnumeration["Info"] = 3] = "Info";
    ElementStatusEnumeration[ElementStatusEnumeration["Error"] = 4] = "Error";
})(ElementStatusEnumeration || (ElementStatusEnumeration = {}));
;
;
;
;
export const StatusIcon = (props) => {
    // States
    const [flashingIntervalID, setFlashingIntervalID] = useState(0);
    const [isFlashingActive, setFlashingActive] = useState(false);
    // Refs
    const isFlashingActiveRef = useRef(isFlashingActive);
    isFlashingActiveRef.current = isFlashingActive;
    const onFlashingInterval = () => {
        setFlashingActive(!isFlashingActiveRef.current);
    };
    // Effects
    useEffect(() => {
        if (props.isFlashing === undefined)
            return;
        if (props.isFlashing) {
            var intervalID = window.setInterval(onFlashingInterval, flashingInterval);
            setFlashingIntervalID(intervalID);
            return;
        }
        if (!props.isFlashing) {
            clearInterval(flashingIntervalID);
            setFlashingActive(false);
            return;
        }
    }, [props.isFlashing]);
    const getStatusStyle = () => {
        if (isFlashingActive)
            return props.customStatusStyles?.flshingStyle ? props.customStatusStyles.flshingStyle : {
                backgroundColor: (theme) => theme.palette.primary.main,
                color: (theme) => theme.palette.primary.contrastText,
                fill: (theme) => theme.palette.primary.contrastText,
            };
        switch (status) {
            case ElementStatusEnumeration.Active:
                return props.customStatusStyles?.activeStyle ? props.customStatusStyles.activeStyle : {
                    backgroundColor: (theme) => theme.palette.success.main,
                    color: (theme) => theme.palette.success.contrastText,
                    fill: (theme) => theme.palette.success.contrastText,
                };
            case ElementStatusEnumeration.Warning:
                return props.customStatusStyles?.warningStyle ? props.customStatusStyles.warningStyle : {
                    backgroundColor: (theme) => theme.palette.warning.main,
                    color: (theme) => theme.palette.warning.contrastText,
                    fill: (theme) => theme.palette.warning.contrastText,
                };
            case ElementStatusEnumeration.Info:
                return props.customStatusStyles?.infoStyle ? props.customStatusStyles.infoStyle : {
                    backgroundColor: (theme) => theme.palette.info.main,
                    color: (theme) => theme.palette.info.contrastText,
                    fill: (theme) => theme.palette.info.contrastText,
                };
            case ElementStatusEnumeration.Error:
                return props.customStatusStyles?.errorStyle ? props.customStatusStyles.errorStyle : {
                    backgroundColor: (theme) => theme.palette.error.main,
                    color: (theme) => theme.palette.error.contrastText,
                    fill: (theme) => theme.palette.error.contrastText,
                };
            case ElementStatusEnumeration.Inactive:
            default:
                return props.customStatusStyles?.inactiveStyle ? props.customStatusStyles.inactiveStyle : {};
        }
    };
    const getStatusIcon = () => {
        switch (status) {
            case ElementStatusEnumeration.Active:
                return props.customStatusIcons?.activeIcon ? props.customStatusIcons?.activeIcon : CheckCircleOutlineIcon;
            case ElementStatusEnumeration.Warning:
                return props.customStatusIcons?.warningIcon ? props.customStatusIcons?.warningIcon : WarningIcon;
            case ElementStatusEnumeration.Info:
                return props.customStatusIcons?.infoIcon ? props.customStatusIcons?.infoIcon : InfoIcon;
            case ElementStatusEnumeration.Error:
                return props.customStatusIcons?.errorIcon ? props.customStatusIcons?.errorIcon : ErrorOutlineIcon;
            case ElementStatusEnumeration.Inactive:
            default:
                return props.customStatusIcons?.inactiveIcon ? props.customStatusIcons?.inactiveIcon : RemoveCircleOutlineIcon;
        }
    };
    const renderIcon = (IconElement) => {
        if (!IconElement)
            return;
        if (typeof IconElement === 'string')
            return (_jsx(Box, { component: 'span', sx: {
                    fontSize: fontSize
                }, children: IconElement }, void 0));
        return (_jsx(IconElement, { css: {
                minWidth: iconSize,
                minHeight: iconSize,
                width: iconSize,
                height: iconSize
            } }, void 0));
    };
    // Helpers
    const status = props.status ? props.status : ElementStatusEnumeration.Inactive;
    const StatusIcon = getStatusIcon();
    const iconSize = (props.iconSize !== undefined) ? props.iconSize : 40;
    const fontSize = (iconSize * 24) / 32;
    const flashingInterval = (props.flashingInterval !== undefined) ? props.flashingInterval : 500;
    if (props.onRenderCustomStatus)
        return props.onRenderCustomStatus(status);
    return (_jsx(Box, { sx: {
            height: '100%',
            display: 'flex',
            flexDirection: 'row',
            alignItems: 'center',
            alignContent: 'center',
            justifyItems: 'center',
            justifyContent: 'center',
            minHeight: iconSize + 16,
            minWidth: iconSize + 16,
            width: iconSize + 16,
            ...getStatusStyle()
        }, children: renderIcon(StatusIcon) }, void 0));
};
