import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { CrossCuttingServiceKeys, LocalizationNamespaces, MessageLevelEnumeration, StampingPressModelServiceKeys } from "@brudererdev/web-extensions";
import { Badge, Box, createStyles, makeStyles, Typography, useTheme } from "@material-ui/core";
import { useContext, useEffect, useRef, useState } from "react";
import React from "react";
import ErrorIcon from '@material-ui/icons/Error';
import WarningIcon from '@material-ui/icons/Warning';
import InfoIcon from '@material-ui/icons/Info';
import { BrudererContext, SizeEnumeration } from "../..";
import { SizeInjectorContainer } from "../containers";
import { Indicator2 } from "../indicators";
const useStyles = makeStyles((theme) => createStyles({
    default: {
        backgroundColor: theme.palette.view.main,
        color: theme.palette.view.contrastText,
        fill: theme.palette.view.contrastText
    },
    info: {
        backgroundColor: theme.palette.primary.main,
        color: theme.palette.primary.contrastText,
        fill: theme.palette.primary.contrastText
    },
    engaged: {
        backgroundColor: theme.palette.success.main,
        color: theme.palette.success.contrastText,
        fill: theme.palette.success.contrastText
    },
    engagedIndicator: {
        backgroundColor: theme.palette.success.contrastText,
    },
    warning: {
        backgroundColor: theme.palette.warning.main,
        color: theme.palette.warning.contrastText,
        fill: theme.palette.warning.contrastText
    },
    error: {
        backgroundColor: theme.palette.error.main,
        color: theme.palette.error.contrastText,
        fill: theme.palette.error.contrastText
    },
    buttonDefault: {
        backgroundColor: theme.palette.primary.main,
        color: theme.palette.primary.contrastText,
        fill: theme.palette.primary.contrastText
    },
    buttonAcknowledgeable: {
        backgroundColor: theme.palette.success.main,
        color: theme.palette.success.contrastText,
        fill: theme.palette.success.contrastText
    },
    statusScrollContainerCard: {
        display: 'flex',
        flexDirection: 'column',
        justifyContent: 'center'
    },
    statusScrollContainerBar: {
        display: 'flex',
        flexDirection: 'row',
        justifyContent: 'center'
    },
    messageContainer: {
        height: '100%',
        display: 'flex',
        flexDirection: 'row',
    },
}));
export const StatusMessages = (props) => {
    // Fields
    const contextName = 'MessageBar';
    // External hooks
    const styleClasses = useStyles();
    const themeClasses = useTheme();
    // States
    const [messageServiceVersion, setMessageServiceVersion] = useState(-1);
    const [clutchServiceVersion, setClutchServiceVersion] = useState(-1);
    const [toolServiceVersion, setToolServiceVersion] = useState(-1);
    // Refs
    const serviceChangeSubscriberKeyDictionaryRef = useRef({});
    // Contexts
    const brudererContext = useContext(BrudererContext);
    const messageService = brudererContext.getService(StampingPressModelServiceKeys.MessageService);
    const clutchService = brudererContext.getService(StampingPressModelServiceKeys.ClutchService);
    const navigationService = brudererContext.getService(CrossCuttingServiceKeys.NavigationService);
    const toolService = brudererContext.getService(StampingPressModelServiceKeys.ToolService);
    // Effects
    useEffect(() => {
        // Mount
        if (messageService) {
            const registerKey = messageService.onChanges(contextName, handleMessageServiceChanges);
            serviceChangeSubscriberKeyDictionaryRef.current[messageService.key] = registerKey;
        }
        ;
        if (clutchService) {
            const registerKey = clutchService.onChanges(contextName, handleClutchServiceChanges);
            serviceChangeSubscriberKeyDictionaryRef.current[clutchService.key] = registerKey;
        }
        ;
        if (toolService) {
            const registerKey = toolService.onChanges(contextName, handleToolServiceChanges);
            serviceChangeSubscriberKeyDictionaryRef.current[toolService.key] = registerKey;
        }
        ;
        // Unmount
        return () => {
            if (messageService) {
                const registerKey = serviceChangeSubscriberKeyDictionaryRef.current[messageService.key];
                messageService.offChanges(registerKey);
            }
            ;
            if (clutchService) {
                const registerKey = serviceChangeSubscriberKeyDictionaryRef.current[clutchService.key];
                clutchService.offChanges(registerKey);
            }
            ;
            if (toolService) {
                const registerKey = serviceChangeSubscriberKeyDictionaryRef.current[toolService.key];
                toolService.offChanges(registerKey);
            }
            ;
            serviceChangeSubscriberKeyDictionaryRef.current = {};
        };
    }, []);
    const handleMessageServiceChanges = (version, key) => {
        setMessageServiceVersion(version);
    };
    const handleClutchServiceChanges = (version, key) => {
        setClutchServiceVersion(version);
    };
    const handleToolServiceChanges = (version, key) => {
        setToolServiceVersion(version);
    };
    const getSizes = () => {
        switch (props.type) {
            case 'messageBar':
                iconSize = SizeEnumeration.m;
                iconSizeValue = 32;
                iconWrapperWidth = 64;
                numberTypographyVariant = 'h5';
                textTypographyVariant = 'h6';
                break;
            case 'actionTile':
            default:
                iconSize = SizeEnumeration.l;
                iconSizeValue = 64;
                iconWrapperWidth = 100;
                numberTypographyVariant = 'h4';
                textTypographyVariant = 'h5';
                break;
        }
    };
    const getMessageIcon = (message) => {
        switch (message.level) {
            case MessageLevelEnumeration.Error:
                return ErrorIcon;
            case MessageLevelEnumeration.Warning:
                return WarningIcon;
            case MessageLevelEnumeration.Info:
                return InfoIcon;
            default:
                return InfoIcon;
        }
    };
    const renderStatusMessage = (message) => {
        const MessageIcon = getMessageIcon(message);
        if (props.onContextStyleClassesChange)
            switch (message.level) {
                case MessageLevelEnumeration.Error:
                    props.onContextStyleClassesChange(styleClasses.error);
                    break;
                case MessageLevelEnumeration.Warning:
                    props.onContextStyleClassesChange(styleClasses.warning);
                    break;
                case MessageLevelEnumeration.Info:
                    props.onContextStyleClassesChange(styleClasses.info);
                    break;
                default:
                    props.onContextStyleClassesChange(styleClasses.info);
                    break;
            }
        var localizedMessageText = message.display.value;
        if (!message.hasCustomText)
            localizedMessageText = brudererContext.localize(message.display);
        var messageCount = 1;
        if (messageService)
            messageCount = messageService.activeErrorMessages.length;
        return (_jsxs(Box, Object.assign({ height: '100%', display: 'flex', flexDirection: 'row', justifyContent: 'center', justifyItems: 'center', alignContent: 'center', alignItems: 'center' }, { children: [_jsx(Box, Object.assign({ height: '100%', minWidth: iconWrapperWidth, display: 'flex', flexDirection: 'column', justifyContent: 'center', justifyItems: 'center', alignContent: 'center', alignItems: 'center' }, { children: _jsx(Badge, Object.assign({ badgeContent: messageCount, overlap: "circle", color: 'primary' }, { children: _jsx(MessageIcon, { style: {
                                minWidth: `${iconSizeValue}`,
                                minHeight: `${iconSizeValue}`,
                                width: `${iconSizeValue}`,
                                height: `${iconSizeValue}`
                            } }, void 0) }), void 0) }), void 0), _jsxs(SizeInjectorContainer, Object.assign({ className: props.type === 'actionTile' ? styleClasses.statusScrollContainerCard : styleClasses.statusScrollContainerBar }, { children: [_jsx(Typography, Object.assign({ variant: numberTypographyVariant }, { children: message.messageID.toString() }), void 0), _jsx(Box, { style: { minWidth: 8 } }, void 0), _jsx(Typography, Object.assign({ variant: textTypographyVariant }, { children: localizedMessageText }), void 0)] }), void 0)] }), void 0));
    };
    const renderEngagedIndicator = () => {
        if (props.onContextStyleClassesChange)
            props.onContextStyleClassesChange(styleClasses.engaged);
        const clutchStateText = isEngaged ?
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.engaged', 'Eingekuppelt') :
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.disengaged', 'Ausgekuppelt');
        return (_jsxs(Box, Object.assign({ height: '100%', display: 'flex', flexDirection: 'row', justifyContent: 'center', justifyItems: 'center', alignContent: 'center', alignItems: 'center' }, { children: [_jsx(Box, Object.assign({ height: '100%', minWidth: iconWrapperWidth, width: iconWrapperWidth, display: 'flex', flexDirection: 'column', justifyContent: 'center', justifyItems: 'center' }, { children: _jsx(Indicator2, { className: styleClasses.engagedIndicator, size: iconSize }, void 0) }), void 0), _jsx(Box, Object.assign({ flex: 'auto', display: 'flex', flexDirection: 'column' }, { children: _jsx(Typography, Object.assign({ variant: numberTypographyVariant }, { children: clutchStateText }), void 0) }), void 0)] }), void 0));
    };
    const renderDisengagedIndicator = () => {
        if (props.onContextStyleClassesChange)
            props.onContextStyleClassesChange(styleClasses.default);
        const clutchStateText = isEngaged ?
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.engaged', 'Eingekuppelt') :
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.disengaged', 'Ausgekuppelt');
        return (_jsxs(Box, Object.assign({ height: '100%', display: 'flex', flexDirection: 'row', alignContent: 'center', alignItems: 'center' }, { children: [_jsx(Box, Object.assign({ height: '100%', minWidth: iconWrapperWidth, width: iconWrapperWidth, display: 'flex', flexDirection: 'column', justifyContent: 'center', alignItems: 'center' }, { children: _jsx(Box, { className: styleClasses.engagedIndicator, style: { opacity: 0.6 }, borderRadius: '50%', height: iconSizeValue, width: iconSizeValue }, void 0) }), void 0), _jsx(Box, Object.assign({ flex: 'auto', display: 'flex', flexDirection: 'column' }, { children: _jsx(Typography, Object.assign({ variant: numberTypographyVariant }, { children: clutchStateText }), void 0) }), void 0)] }), void 0));
    };
    const renderContent = () => {
        const primaryMessage = (activeErrorMessages.length > 0) ? activeErrorMessages[0] : undefined;
        if (primaryMessage)
            return renderStatusMessage(primaryMessage);
        if (isEngaged)
            return renderEngagedIndicator();
        return renderDisengagedIndicator();
    };
    // Helpers
    var iconSize;
    var iconSizeValue;
    var iconWrapperWidth;
    var numberTypographyVariant;
    var textTypographyVariant;
    getSizes();
    const activeErrorMessages = props.activeErrorMessages ? props.activeErrorMessages : [];
    const isEngaged = clutchService ? clutchService.isEngaged : false;
    return (_jsx(React.Fragment, { children: renderContent() }, void 0));
};
