import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useState, useRef, useEffect } from 'react';
import { Box, Typography } from '@mui/material';
import { SizeEnumeration } from './../../shared/styleProps';
import { defaultTileSize } from './../../shared/tileProps';
import { InfoBadge, getInfoBadgeSizeValue } from './../badges/InfoBadge';
import { TileBase } from './TileBase';
export const Tile = (props) => {
    // States
    const [tileWidth, setTileWidth] = useState(0);
    const [tileHeight, setTileHeight] = useState(0);
    // Refs
    const tileBaseRef = useRef(null);
    // Effects
    useEffect(() => {
        // Set new width for the region label after a region change
        if (tileBaseRef.current !== null)
            if (tileBaseRef.current.offsetWidth !== tileWidth)
                setTileWidth(tileBaseRef.current.offsetWidth);
        // Set new width for the view label after a region change
        if (tileBaseRef.current !== null)
            if (tileBaseRef.current.offsetHeight !== tileHeight)
                setTileHeight(tileBaseRef.current.offsetHeight);
    });
    const getBadgeTopOffset = () => {
        var verticalOffset = Math.round(getInfoBadgeSizeValue(badgeSize) / 3) * -1;
        return verticalOffset;
    };
    const getBadgeLeftOffset = () => {
        var horizontalOffset = Math.round(getInfoBadgeSizeValue(badgeSize) / 3) * -1;
        if (badgeOrientation === 'right')
            horizontalOffset = (horizontalOffset * 2) + tileWidth;
        return horizontalOffset;
    };
    // Helpers
    const width = (props.width !== undefined) ? props.width : defaultTileSize;
    const height = (props.height !== undefined) ? props.height : defaultTileSize;
    const badgeOrientation = (props.badgeOrientation !== undefined) ? props.badgeOrientation : 'right';
    const badgeSize = (props.badgeSize !== undefined) ? props.badgeSize : SizeEnumeration.m;
    const badgeTopOffset = (props.badgeTopOffset !== undefined) ? props.badgeTopOffset : 0;
    const badgeLeftOffset = (props.badgeLeftOffset !== undefined) ? props.badgeLeftOffset : 0;
    const isBadgeVisible = (props.isBadgeVisible !== undefined) ? props.isBadgeVisible : false;
    return (_jsxs(Box, { flex: 'auto', children: [isBadgeVisible &&
                _jsx(InfoBadge, { badgeStyle: props.badgeStyle, size: badgeSize, topOffset: getBadgeTopOffset() + badgeTopOffset, leftOffset: getBadgeLeftOffset() + badgeLeftOffset, children: _jsx(Typography, { variant: 'h6', children: props.badgeContent ? props.badgeContent : '' }, void 0) }, void 0), _jsx(TileBase, { ref: tileBaseRef, ...props, width: width, height: height }, void 0)] }, void 0));
};
