import { jsx as _jsx } from "react/jsx-runtime";
import { useState, useRef, useEffect, forwardRef } from 'react';
import { Button } from '@mui/material';
export const TileBase = forwardRef((props, ref) => {
    // States
    const [isFlashingActive, setFlashingActive] = useState(false);
    // Refs
    const flashingIntervalIDRef = useRef(0);
    const isFlashingActiveRef = useRef(isFlashingActive);
    isFlashingActiveRef.current = isFlashingActive;
    const onFlashingInterval = () => {
        setFlashingActive(!isFlashingActiveRef.current);
    };
    const getTileStyle = () => {
        var tileStyle = props.tileStyle;
        if (!tileStyle)
            tileStyle = {
                backgroundColor: (theme) => theme.palette.primary.main,
                color: (theme) => theme.palette.primary.contrastText,
                fill: (theme) => theme.palette.primary.contrastText
            };
        if (isFlashingActive) {
            tileStyle = props.flashingStyle;
            if (!tileStyle)
                tileStyle = {
                    backgroundColor: (theme) => theme.palette.success.main,
                    color: (theme) => theme.palette.success.contrastText,
                    fill: (theme) => theme.palette.success.contrastText
                };
        }
        return tileStyle;
    };
    // Effects
    useEffect(() => {
        // Mount
        // Unmount
        return () => {
            clearInterval(flashingIntervalIDRef.current);
        };
    }, []);
    useEffect(() => {
        if (props.isFlashing === undefined)
            return;
        if (props.isFlashing) {
            flashingIntervalIDRef.current = window.setInterval(onFlashingInterval, flashingInterval);
            return;
        }
        if (!props.isFlashing) {
            clearInterval(flashingIntervalIDRef.current);
            setFlashingActive(false);
            return;
        }
    }, [props.isFlashing]);
    // Helpers
    const flashingInterval = (props.flashingInterval !== undefined) ? props.flashingInterval : 500;
    return (_jsx(Button, { ref: ref, sx: {
            minWidth: props.minWidth,
            width: props.width,
            minHeight: props.minHeight,
            height: props.height,
            borderRadius: 0,
            padding: 0,
            textTransform: 'none',
            ...getTileStyle(),
            ':hover': (theme) => ({
                ...getTileStyle()
            }),
            '.Mui-disabled': (theme) => ({
                color: theme.palette.action.disabled,
                fill: theme.palette.action.disabled,
            })
        }, variant: 'contained', disabled: props.isDisabled, onClick: props.onClick, onMouseDown: props.onMouseDown, onMouseUp: props.onMouseUp, onTouchStart: props.onTouchStart, onTouchEnd: props.onTouchEnd, children: props.children ? props.children : _jsx("div", {}, void 0) }, void 0));
});
