import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/* eslint-disable react/react-in-jsx-scope -- Unaware of jsxImportSource */
/** @jsxImportSource @emotion/react */
import React, { useState } from 'react';
import { LocalizationNamespaces, SelectionMaps } from '@brudererdev/web-extensions';
import { FormControl, InputLabel, Select, InputAdornment, FormHelperText, MenuItem, Box, Typography } from '@mui/material';
import ArrowRightIcon from '@mui/icons-material/ArrowRight';
import InfoOutlinedIcon from '@mui/icons-material/InfoOutlined';
export const VarSelector = (props) => {
    // States
    const [selectionMap, setSelectionMap] = useState(SelectionMaps.find(map => map.key === props.modelVariable.valueEnumerationKey));
    const handleSelectorChange = (e, child) => {
        if (isReadOnly)
            return;
        if (!selectionMap)
            return;
        var varClone = { ...props.modelVariable };
        varClone.value = e.target.value;
        // Invoke changed method
        if (props.onModelVariableChanged)
            props.onModelVariableChanged(varClone);
    };
    const getLabelText = () => {
        if (props.newValue === undefined || props.newValue === null)
            return '';
        if (showInitialValueOnChange && (selectionMap !== undefined))
            if (props.modelVariable.value !== props.newValue) {
                var initialValueMap = selectionMap.values.find((map, index) => map.value === props.modelVariable.value);
                if (initialValueMap === undefined || initialValueMap === null)
                    return '';
                var labelText = initialValueMap.display.value;
                if (props.onLocalize)
                    labelText = props.onLocalize(initialValueMap.display);
                return labelText;
            }
        return '';
    };
    const renderStartAdornmentIcon = () => {
        var IconElement = props.StartAdornmentIcon;
        if (IconElement == undefined)
            IconElement = InfoOutlinedIcon;
        if (!IconElement)
            return;
        const iconSize = 24;
        const fontSize = (iconSize * 24) / 32;
        if (typeof IconElement === 'string')
            return (_jsx(Box, { component: 'span', sx: {
                    fontSize: fontSize
                }, children: IconElement }, void 0));
        return (_jsx(IconElement, { css: {
                minWidth: iconSize,
                minHeight: iconSize,
                width: iconSize,
                height: iconSize
            } }, void 0));
    };
    const renderStartAdornment = () => {
        if (!props.showStartAdornment)
            return undefined;
        return (_jsx(InputAdornment, { sx: {
                height: '100%',
                display: 'flex',
                flexDirection: 'column',
                justifyContent: 'flex-end',
                justifyItems: 'flex-end'
            }, position: "start", children: renderStartAdornmentIcon() }, void 0));
    };
    // Get actual value
    var value = (props.newValue !== undefined) ? props.newValue : props.modelVariable.value;
    if (!selectionMap)
        value = '';
    // Helpers
    const showInitialValueOnChange = (props.showInitialValueOnChange !== undefined) ? props.showInitialValueOnChange : false;
    const isReadOnlyForced = (props.forceReadOnly !== undefined) ? props.forceReadOnly : false;
    const isReadOnly = isReadOnlyForced ? isReadOnlyForced : props.modelVariable.isReadOnly;
    // Setup "NoValidData" text
    var noValidDataText = 'No vaild data';
    if (props.onLocalize)
        noValidDataText = props.onLocalize({
            link: {
                key: 'global.novaliddata',
                path: 'global',
                name: 'novaliddata',
                seperator: '.'
            },
            keyNamespace: LocalizationNamespaces.UIGeneric,
            value: 'No vaild data'
        });
    return (_jsx(React.Fragment, { children: _jsxs(FormControl, { variant: 'filled', margin: 'dense', id: props.modelVariable.modelLink.key, disabled: props.isDisabled, sx: {
                width: '100%'
            }, children: [_jsx(InputLabel, { color: 'secondary', disabled: props.isDisabled, children: _jsxs(Box, { sx: {
                            display: 'flex',
                            flexDirection: 'row',
                            alignContent: 'center',
                            alignItems: 'center'
                        }, children: [showInitialValueOnChange && (props.newValue !== undefined) && (props.modelVariable.value !== props.modelVariable.value) && _jsx(ArrowRightIcon, {}, void 0), getLabelText()] }, void 0) }, void 0), _jsxs(Select, { labelId: "label", variant: 'filled', color: 'secondary', sx: {
                        '& .MuiSelect-filled': {
                            paddingBottom: '2px',
                            paddingTop: '23px'
                        },
                        '& .MuiSelect-nativeInput': {},
                        '& .MuiSelect-icon': {},
                        '& .MuiSelect-select': {
                            ":focus": {
                                backgroundColor: (theme) => theme.palette.card.main
                            }
                        }
                    }, value: value, disabled: props.isDisabled || isReadOnly, onChange: handleSelectorChange, startAdornment: renderStartAdornment(), children: [selectionMap &&
                            selectionMap.values.map((selectionValueMap, index) => {
                                return (_jsx(MenuItem, { value: selectionValueMap.value, sx: {
                                        '&.MuiMenuItem-root': {
                                            backgroundColor: (theme) => theme.palette.menu.main,
                                            color: (theme) => theme.palette.menu.contrastText,
                                            fill: (theme) => theme.palette.menu.contrastText,
                                            ':hover': {
                                                backgroundColor: (theme) => theme.palette.grey[700],
                                                color: (theme) => theme.palette.menu.contrastText,
                                                fill: (theme) => theme.palette.menu.contrastText
                                            },
                                        },
                                        '&.Mui-selected': {
                                            backgroundColor: (theme) => theme.palette.secondary.main,
                                            color: (theme) => theme.palette.secondary.contrastText,
                                            fill: (theme) => theme.palette.secondary.contrastText
                                        }
                                    }, children: _jsx(Typography, { variant: 'h5', children: (props.onLocalize !== undefined) ? props.onLocalize(selectionValueMap.display) : selectionValueMap.display.value }, void 0) }, index));
                            }), !selectionMap &&
                            _jsx(MenuItem, { value: '', children: _jsx(Typography, { variant: 'h5', children: noValidDataText }, void 0) }, void 0)] }, void 0), _jsx(FormHelperText, { disabled: props.isDisabled, sx: {
                        '&.MuiFormHelperText-root': (theme) => ({
                            opacity: 1.0,
                            overflowWrap: 'anywhere',
                            fontWeight: 'normal',
                            ...theme.typography.body1
                        })
                    }, children: (props.onLocalize !== undefined) ? props.onLocalize(props.modelVariable.display) : props.modelVariable.display.value }, void 0)] }, void 0) }, void 0));
};
