import { jsx as _jsx } from "react/jsx-runtime";
import React, { useState, useEffect, useRef, Suspense } from 'react';
import { Grid, Box } from '@mui/material';
import { scrollVerticalStyle } from './../../styles/themeExtensions';
import { SizeEnumeration } from './../../shared/styleProps';
import { ActionTile } from './../elements/ActionTile';
import { ErrorBoundary } from './../ErrorBoundary';
import { ErrorContent } from './../ErrorContent';
import { Indicator1 } from './../indicators/Indicator1';
import { ActionTileRenderer } from './../renderer/ActionTileRenderer';
const ActionTileInjectorMemoized = (props) => {
    // States
    const [actionTileContexts, setActionTileContexts] = useState([]);
    const [sizeCount, setSizeCount] = useState(0);
    // Effects
    useEffect(() => {
        // Mount
        const newActionTileContexts = props.onLoadActionTiles();
        setActionTileContexts(newActionTileContexts);
        setSizeCount(getSizeCount(newActionTileContexts));
        actionTileNavigationElementsRef.current = props.onLoadNavigationElements();
    }, []);
    useEffect(() => {
        if (!props.loadActionTiles)
            return;
        const newActionTileContexts = props.onLoadActionTiles();
        setSizeCount(getSizeCount(newActionTileContexts));
        setActionTileContexts(newActionTileContexts);
    }, [props.loadActionTiles]);
    // Refs
    var actionTileContextsRef = useRef(actionTileContexts);
    actionTileContextsRef.current = actionTileContexts;
    var actionTileNavigationElementsRef = useRef([]);
    const searchNavigationElement = (fileName) => {
        return actionTileNavigationElementsRef.current.find((element) => element.fileName === fileName);
    };
    const getSizeCount = (contexts) => {
        var count = 0;
        contexts.forEach((context) => {
            var existingElement = searchNavigationElement(context.fileName);
            if (existingElement)
                count += existingElement.size;
        });
        return count;
    };
    /**
     * validate if all conditions are met for the dynamically loaded action tile.
     *
     * @param fileName fileName of the new actiontile
     */
    const validateNewActionTile = (fileName) => {
        var validatingResult = true;
        const existingNavigationElement = searchNavigationElement(fileName);
        if (!existingNavigationElement)
            return false;
        if (!checkActionTileNavigationElementSize(existingNavigationElement))
            validatingResult = false;
        return validatingResult;
    };
    const handleReloadActionTile = (newFileName, lastFileName, index) => {
        var activeActionTileContexts = [...actionTileContextsRef.current];
        const existingNavigationElement = searchNavigationElement(newFileName);
        const oldExistingNavigationElement = searchNavigationElement(lastFileName);
        if (!existingNavigationElement)
            return;
        if (!oldExistingNavigationElement)
            return;
        if (index >= activeActionTileContexts.length)
            return;
        if (!validateNewActionTile(newFileName))
            return;
        reloadActionTileContexts(existingNavigationElement, oldExistingNavigationElement, index);
        const reloadedActionTileContexts = [...actionTileContextsRef.current];
        setActionTileContexts(reloadedActionTileContexts);
    };
    /**
      * adapt existing contexts for the new actiontile.
      */
    const reloadActionTileContexts = (newElement, lastElement, index) => {
        const currentNavigationElements = [];
        actionTileContextsRef.current.forEach((context) => {
            var existingElement = searchNavigationElement(context.fileName);
            if (existingElement)
                currentNavigationElements.push(existingElement);
        });
        if (currentNavigationElements.length !== actionTileContextsRef.current.length)
            return false;
        var indexCount = 0;
        for (var elementIndex = 0; elementIndex < index; elementIndex++) {
            indexCount += currentNavigationElements[elementIndex].size;
        }
        if (lastElement.size == 2) {
            if (newElement.size == 2) {
                actionTileContextsRef.current[index].fileName = newElement.fileName;
            }
            else {
                actionTileContextsRef.current[index].fileName = newElement.fileName;
                const newActionTileNavigationElement = actionTileNavigationElementsRef.current.find((element) => element.size == 1 && element.isSelectable !== false);
                if (newActionTileNavigationElement)
                    actionTileContextsRef.current.splice(index + 1, 0, { fileName: newActionTileNavigationElement.fileName });
            }
        }
        else {
            if (newElement.size == 1) {
                actionTileContextsRef.current[index].fileName = newElement.fileName;
            }
            else {
                if ((indexCount % 2) == 0) {
                    actionTileContextsRef.current[index].fileName = newElement.fileName;
                    actionTileContextsRef.current.splice(index + 1, 1);
                }
                else {
                    if (indexCount < sizeCount - 1) {
                        actionTileContextsRef.current[index + 1].fileName = newElement.fileName;
                        actionTileContextsRef.current.splice(index + 2, 1);
                    }
                }
            }
        }
    };
    /**
       * check if there is enought space to display passed navigation element.
       */
    const checkActionTileNavigationElementSize = (element) => {
        var result = true;
        if (element.size == 2) {
            // check whether there is enought space to display double actiontile.
            if (!window.matchMedia("(min-width: 960px)").matches) {
                result = false;
            }
        }
        return result;
    };
    // Helpers
    const xs = (props.xs !== undefined) ? props.xs : 12; // xs = 0px - 600px
    const sm = (props.sm !== undefined) ? props.sm : 12; // sm = 600px 960px
    var md = (props.md !== undefined) ? props.md : 6; // md = 960px - 1280px
    var lg = (props.lg !== undefined) ? props.lg : 6; // lg = 1280px -1920px
    var xl = (props.xl !== undefined) ? props.xl : 6; // xl = 1920px -
    return (_jsx(Grid, { sx: {
            ...scrollVerticalStyle
        }, container: true, spacing: 2, children: actionTileContexts.map((actionTileContext, index) => {
            const existingNavigationElement = searchNavigationElement(actionTileContext.fileName);
            if (existingNavigationElement) {
                if (existingNavigationElement.size == 2) {
                    md = 12;
                    lg = 12;
                    xl = 12;
                }
                else {
                    md = 6;
                    lg = 6;
                    xl = 6;
                }
            }
            return (_jsx(Grid, { item: true, xs: xs, sm: sm, md: md, lg: lg, xl: xl, children: _jsx(ErrorBoundary, { sourceName: actionTileContext.fileName, fallback: (source, error, errorInfo) => {
                        return (_jsx(ActionTile, { onLocalize: props.onLocalize, onRenderContent: (layerIndex, contentIndex) => {
                                return (_jsx(ErrorContent, { sourceName: source, errorMessage: error && error.toString(), stackInfo: errorInfo.componentStack }, void 0));
                            } }, void 0));
                    }, children: _jsx(Suspense, { fallback: _jsx(ActionTile, { onLocalize: props.onLocalize, onRenderContent: (layerIndex, contentIndex) => {
                                return (_jsx(Box, { sx: {
                                        height: '100%',
                                        width: '100%',
                                        display: 'flex',
                                        flexDirection: 'column',
                                        alignItems: 'center',
                                        alignContent: 'center',
                                        justifyItems: 'center',
                                        justifyContent: 'center'
                                    }, children: _jsx(Indicator1, { size: SizeEnumeration.s }, void 0) }, void 0));
                            } }, void 0), children: _jsx(ActionTileRenderer, { context: actionTileContext, onImportActionTile: props.onImportActionTile, onReloadActionTile: (newFileName, lastFileName) => handleReloadActionTile(newFileName, lastFileName, index) }, void 0) }, void 0) }, void 0) }, index));
        }) }, void 0));
};
export const ActionTileInjector = React.memo(ActionTileInjectorMemoized, (prevProps, nextProps) => { return true; });
