import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState, useRef, useMemo, useEffect, Suspense, useContext } from 'react';
import { Box } from '@mui/material';
import { CrossCuttingServiceKeys, NavigationTypeEnumeration } from '@brudererdev/web-extensions';
import { ActionTileInjector } from '.';
import { BrudererContext, SizeEnumeration } from '../..';
import { ViewTopBar } from '../bars';
import { ErrorBoundary } from '../ErrorBoundary';
import { ErrorContent } from '../ErrorContent';
import { Indicator1 } from '../indicators';
import { View } from '../View';
export const SubViewInjector = (props) => {
    // States
    const [selectedViewElement, setSelectedViewElement] = useState(props.preselectedNavigationElement ? props.preselectedNavigationElement : props.defaultNavigationElement);
    const [searchText, setSearchText] = useState(undefined);
    const [showActionTiles, setShowActionTiles] = useState(props.showActionTilesByDefault ? props.showActionTilesByDefault : false);
    // Contexts
    const brudererContext = useContext(BrudererContext);
    const navigationService = brudererContext.getService(CrossCuttingServiceKeys.NavigationService);
    // Refs
    const lastPayloadRef = useRef();
    // Effects
    useEffect(() => {
        // Mount
        // Unmount
        return () => {
        };
    }, []);
    const handleSelectByView = (elementKey, payload) => {
        var navigationElement = props.navigationElements.find(element => element.key === elementKey);
        if (!navigationElement)
            return false;
        lastPayloadRef.current = payload;
        setSelectedViewElement(navigationElement);
        setSearchText(undefined);
        return true;
    };
    const handleSelectByViewTopBar = (element) => {
        if (!navigationService || !element)
            return false;
        var elementKey = element.key;
        if (element.type === NavigationTypeEnumeration.Action) {
            if (props.onInvokeAction)
                return props.onInvokeAction(elementKey, lastPayloadRef.current);
            return false;
        }
        if (element.type === NavigationTypeEnumeration.Dialog) {
            if (props.onInvokeDialog)
                return props.onInvokeDialog(elementKey, lastPayloadRef.current);
            return false;
        }
        var navigationElement = props.navigationElements.find(element => element.key === elementKey);
        if (!navigationElement)
            return false;
        setSelectedViewElement(navigationElement);
        setSearchText(undefined);
        return true;
    };
    const handleSetPayload = (payload) => {
        lastPayloadRef.current = payload;
    };
    const handleOnLoadActionTiles = () => {
        if (props.onLoadActionTiles)
            return props.onLoadActionTiles();
        return [];
    };
    const handleOnLoadNavigationElements = () => {
        if (props.onLoadNavigationElements)
            return props.onLoadNavigationElements();
        return [];
    };
    const getViewHeaderText = () => {
        var defaultHeaderText = props.contextName;
        var selectedElementHeaderText = '';
        var headerText = defaultHeaderText;
        if (selectedViewElement)
            selectedElementHeaderText = (props.onLocalize !== undefined) ? props.onLocalize(selectedViewElement.display) : selectedViewElement.display.value;
        if ((selectedElementHeaderText !== '') && (selectedElementHeaderText !== defaultHeaderText))
            headerText = `${defaultHeaderText} - ${selectedElementHeaderText}`;
        return headerText;
    };
    // Memorized lazy import of the selected view component
    const GenericView = useMemo(() => {
        return props.onImportView(selectedViewElement);
    }, [selectedViewElement.key]);
    var selectedElementIndex = undefined;
    if (props.navigationElements) {
        var foundedIndex = props.navigationElements.findIndex((element, index) => element.key === selectedViewElement.key);
        if (foundedIndex >= 0)
            selectedElementIndex = foundedIndex;
    }
    ;
    // Helpers
    const showHeader = (props.showHeader !== undefined) ? props.showHeader : false;
    return (_jsxs(Box, { sx: {
            display: 'flex',
            flexDirection: 'column',
            height: '100%'
        }, children: [_jsx(Box, { sx: { minHeight: 0 } }, void 0), _jsx(ViewTopBar, { elements: props.navigationElements, defaultNavigationElement: props.defaultNavigationElement, selectedElementIndex: selectedElementIndex, headerText: getViewHeaderText(), isMessageBarVisible: selectedViewElement.isMessageBarVisible, isSearchVisible: selectedViewElement.isSearchable, hasActionTiles: (handleOnLoadActionTiles().length > 0), userRoles: props.userRoles, locale: props.locale, onAssistantClick: props.onAssistantClick, onStatusClick: props.onStatusClick, onMessageAcknowledge: props.onMessageAcknowledge, onLocalize: props.onLocalize, onSelectTile: handleSelectByViewTopBar, onShowApps: props.onShowApps, onShowActionTiles: () => setShowActionTiles(!showActionTiles), onClose: props.onClose, onSearch: (input) => setSearchText(input), onSearchTextChanged: (input) => setSearchText(input) }, void 0), _jsx(Box, { sx: {
                    overflow: 'hidden',
                    flex: 'auto'
                }, children: _jsx(ErrorBoundary, { sourceName: selectedViewElement.fileName, fallback: (source, error, errorInfo) => {
                        return (_jsx(View, { children: _jsx(ErrorContent, { sourceName: source, errorMessage: error && error.toString(), stackInfo: errorInfo.componentStack }, void 0) }, void 0));
                    }, children: _jsx(Suspense, { fallback: _jsx(View, { contentStyle: {
                                height: '100%',
                                width: '100%',
                                display: 'flex',
                                flexDirection: 'column',
                                alignItems: 'center',
                                alignContent: 'center',
                                justifyItems: 'center',
                                justifyContent: 'center'
                            }, children: _jsx(Indicator1, { size: SizeEnumeration.l }, void 0) }, void 0), children: _jsx(GenericView, { searchText: searchText, payload: lastPayloadRef.current, onSelect: handleSelectByView, onSetPayload: handleSetPayload }, void 0) }, void 0) }, void 0) }, void 0), showActionTiles && props.onImportActionTile &&
                _jsxs(React.Fragment, { children: [_jsx(Box, { sx: { minHeight: 16 } }, void 0), _jsx(ActionTileInjector, { onLoadActionTiles: handleOnLoadActionTiles, onLoadNavigationElements: handleOnLoadNavigationElements, onLocalize: props.onLocalize, onImportActionTile: props.onImportActionTile }, void 0)] }, void 0)] }, void 0));
};
