import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useState, useContext, useRef, useEffect } from 'react';
import { Box, Typography } from '@mui/material';
import SettingsInputHdmiIcon from '@mui/icons-material/SettingsInputHdmi';
import InfoIcon from '@mui/icons-material/Info';
import { LocalizationNamespaces, StampingPressModelServiceKeys } from '@brudererdev/web-extensions';
import { BrudererContext } from '../..';
import { SizeInjectorContainer } from '../containers';
export var AssistantContextEnumeration;
(function (AssistantContextEnumeration) {
    AssistantContextEnumeration[AssistantContextEnumeration["Unknown"] = 0] = "Unknown";
    AssistantContextEnumeration[AssistantContextEnumeration["Default"] = 1] = "Default";
    AssistantContextEnumeration[AssistantContextEnumeration["BackendConnectionError"] = 2] = "BackendConnectionError";
    AssistantContextEnumeration[AssistantContextEnumeration["PLCConnectionError"] = 3] = "PLCConnectionError";
    AssistantContextEnumeration[AssistantContextEnumeration["InfoMessagePending"] = 4] = "InfoMessagePending";
    AssistantContextEnumeration[AssistantContextEnumeration["NoLoadedToolAvailable"] = 5] = "NoLoadedToolAvailable";
    AssistantContextEnumeration[AssistantContextEnumeration["ErrorMessagePending"] = 6] = "ErrorMessagePending";
})(AssistantContextEnumeration || (AssistantContextEnumeration = {}));
;
;
;
;
export const AssistantRendererComponent = (props) => {
    // Fields
    const contextName = 'AssistantRenderer';
    // States
    const [messageServiceVersion, setMessageServiceVersion] = useState(-1);
    const [toolServiceVersion, setToolServiceVersion] = useState(-1);
    // Refs
    const subscriberKeyDictionaryRef = useRef({});
    // Contexts
    const brudererContext = useContext(BrudererContext);
    const messageService = brudererContext.getService(StampingPressModelServiceKeys.MessageService);
    const toolService = brudererContext.getService(StampingPressModelServiceKeys.ToolService);
    // Conditions
    const activeInfoMessages = messageService?.activeInfoMessages ? messageService.activeInfoMessages : [];
    const activeErrorMessages = messageService?.activeErrorMessages ? messageService.activeErrorMessages : [];
    const loadedToolName = toolService?.loadedToolContext.toolName ? toolService.loadedToolContext.toolName : '';
    // Effects
    useEffect(() => {
        // Mount
        if (messageService) {
            const registerKey = messageService.onChanges(contextName, handleMessageServiceChanges);
            subscriberKeyDictionaryRef.current[messageService.key] = registerKey;
        }
        ;
        if (toolService) {
            const registerKey = toolService.onChanges(contextName, handleToolServiceChanges);
            subscriberKeyDictionaryRef.current[toolService.key] = registerKey;
        }
        ;
        // Unmount
        return () => {
            if (messageService) {
                const registerKey = subscriberKeyDictionaryRef.current[messageService.key];
                messageService.offChanges(registerKey);
            }
            ;
            if (toolService) {
                const registerKey = subscriberKeyDictionaryRef.current[toolService.key];
                toolService.offChanges(registerKey);
            }
            ;
            subscriberKeyDictionaryRef.current = {};
        };
    }, []);
    useEffect(() => {
        const assistantContext = getContext();
        assistantContext.style = geContextStyle(assistantContext);
        if (props.onAssistantContextChanged)
            props.onAssistantContextChanged(assistantContext);
    }, [
        brudererContext.failedServices.length,
        brudererContext.isPLCConnected,
        messageServiceVersion,
        toolServiceVersion
    ]);
    const handleMessageServiceChanges = (version, key) => {
        setMessageServiceVersion(version);
    };
    const handleToolServiceChanges = (version, key) => {
        setToolServiceVersion(version);
    };
    const getSizes = () => {
        switch (props.type) {
            case 'MessageBar':
                iconSizeValue = 32;
                iconWrapperWidth = 64;
                typographyVariant = 'h6';
                break;
            case 'ActionTile':
            default:
                iconSizeValue = 64;
                iconWrapperWidth = 100;
                typographyVariant = 'h5';
                break;
        }
    };
    const getContentContainerStyle = () => {
        var containerStyle = {
            display: 'flex',
            flexDirection: 'row',
            alignContent: 'center',
            alignItems: 'center'
        };
        return containerStyle;
    };
    const getContextIcon = (context) => {
        switch (context.contextType) {
            case AssistantContextEnumeration.BackendConnectionError:
            case AssistantContextEnumeration.PLCConnectionError:
                return (_jsx(SettingsInputHdmiIcon, { sx: {
                        minWidth: iconSizeValue,
                        minHeight: iconSizeValue,
                        width: iconSizeValue,
                        height: iconSizeValue
                    }, className: 'asssistant-icon animated1' }, void 0) /* TO DO: change animation to new style */);
            case AssistantContextEnumeration.InfoMessagePending:
            case AssistantContextEnumeration.NoLoadedToolAvailable:
            case AssistantContextEnumeration.ErrorMessagePending:
                return (_jsx(InfoIcon, { sx: {
                        minWidth: iconSizeValue,
                        minHeight: iconSizeValue,
                        width: iconSizeValue,
                        height: iconSizeValue
                    }, className: 'asssistant-icon animated1' }, void 0) /* TO DO: change animation to new style */);
            case AssistantContextEnumeration.Default:
            default:
                return null;
        }
    };
    const geContextStyle = (context) => {
        var statusStyle = {};
        switch (context.contextType) {
            case AssistantContextEnumeration.BackendConnectionError:
            case AssistantContextEnumeration.PLCConnectionError:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.error.main,
                    color: (theme) => theme.palette.error.contrastText,
                    fill: (theme) => theme.palette.error.contrastText
                };
            case AssistantContextEnumeration.NoLoadedToolAvailable:
            case AssistantContextEnumeration.ErrorMessagePending:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.warning.main,
                    color: (theme) => theme.palette.warning.contrastText,
                    fill: (theme) => theme.palette.warning.contrastText
                };
            case AssistantContextEnumeration.InfoMessagePending:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.info.main,
                    color: (theme) => theme.palette.info.contrastText,
                    fill: (theme) => theme.palette.info.contrastText
                };
            case AssistantContextEnumeration.Default:
            default:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.view.main,
                    color: (theme) => theme.palette.view.contrastText,
                    fill: (theme) => theme.palette.view.contrastText
                };
        }
        ;
    };
    const getContextContent = (context) => {
        switch (context.contextType) {
            case AssistantContextEnumeration.BackendConnectionError:
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.backenderrorhelp', 'Es scheint ein Problem mit der Verbindung zum Backend zu geben. Tippen Sie hier, um Hilfe zu erhalten.'));
            case AssistantContextEnumeration.PLCConnectionError:
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.plcconnectionerrorhelp', 'Es scheint ein Problem mit der Verbindung zur SPS zu geben. Tippen Sie hier, um Hilfe zu erhalten.'));
            case AssistantContextEnumeration.InfoMessagePending:
                var localizedMessageText = '?';
                if (activeInfoMessages.length > 0) {
                    localizedMessageText = activeInfoMessages[0].display.value;
                    if (!activeInfoMessages[0].hasCustomText)
                        localizedMessageText = brudererContext.localize(activeInfoMessages[0].display);
                }
                return localizedMessageText;
            case AssistantContextEnumeration.NoLoadedToolAvailable:
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.tooldataerrorhelp', 'Es scheint ein Problem mit ihren Werkzeugdaten zu geben. Tippen Sie hier um die Daten zu editieren.'));
            case AssistantContextEnumeration.ErrorMessagePending:
                return (brudererContext.localizeRaw(LocalizationNamespaces.UIGeneric, 'elements.assistant.errormessagehelp', 'Es stehen ein oder mehrere Fehler an. Tippen Sie hier, um Hilfe zu erhalten.'));
            case AssistantContextEnumeration.Default:
            default:
                return null;
        }
    };
    const getContext = () => {
        var context = {
            contextType: AssistantContextEnumeration.Default
        };
        if (brudererContext.failedServices.length > 0) {
            context.contextType = AssistantContextEnumeration.BackendConnectionError;
            return context;
        }
        ;
        if (!brudererContext.isPLCConnected) {
            context.contextType = AssistantContextEnumeration.PLCConnectionError;
            return context;
        }
        ;
        if (activeInfoMessages.length > 0) {
            context.contextType = AssistantContextEnumeration.InfoMessagePending;
            context.context = {
                message: activeInfoMessages[0]
            };
            return context;
        }
        ;
        if (loadedToolName === '') {
            context.contextType = AssistantContextEnumeration.NoLoadedToolAvailable;
            return context;
        }
        ;
        if (activeErrorMessages.length > 0) {
            context.contextType = AssistantContextEnumeration.ErrorMessagePending;
            context.context = {
                message: activeErrorMessages[0]
            };
            return context;
        }
        ;
        return context;
    };
    const renderContent = () => {
        const assistantContext = getContext();
        return (_jsxs(Box, { sx: {
                height: '100%',
                display: 'flex',
                flexDirection: 'row',
                alignContent: 'center',
                alignItems: 'center'
            }, onClick: () => {
                if (props.onClick)
                    props.onClick(assistantContext);
            }, children: [_jsx(Box, { sx: {
                        minWidth: iconWrapperWidth,
                        display: 'flex',
                        flexDirection: 'column',
                        alignContent: 'center',
                        alignItems: 'center'
                    }, children: getContextIcon(assistantContext) }, void 0), _jsx(SizeInjectorContainer, { containerStyle: getContentContainerStyle(), children: _jsx(Box, { sx: {
                            minHeight: 0,
                            textAlign: 'left'
                        }, children: _jsx(Typography, { variant: `${typographyVariant}`, children: getContextContent(assistantContext) }, void 0) }, void 0) }, void 0)] }, void 0));
    };
    // Helpers
    var iconSizeValue;
    var iconWrapperWidth;
    var typographyVariant;
    getSizes();
    return (_jsx(React.Fragment, { children: renderContent() }, void 0));
};
export const AssistantRenderer = React.memo(AssistantRendererComponent);
