import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React from 'react';
import { TypeValidatorNET, buildModelContainers, BrudererUserRoleEnumeration } from '@brudererdev/web-extensions';
import { Accordion, AccordionSummary, AccordionDetails, Box, Typography, Card, CardContent } from '@mui/material';
import ExpandMoreIcon from '@mui/icons-material/ExpandMore';
import { cardStyle, cardContentStyle, accordionSummaryRootStyle, accordionSummaryExpandedStyle, accordionSummaryContentStyle, accordionDetailsRootStyle } from './../../styles/themeExtensions';
import { ModelVariableRenderer } from './../renderer/ModelVariableRenderer';
import { Label } from './../elements/Label';
import { WrapContainer } from './../containers/WrapContainer';
import { ElementContainer } from './../containers/ElementContainer';
import { getFilteredVariables } from './../../shared/variableProps';
import { SizeEnumeration } from './../../shared/styleProps';
import { ErrorContent } from './../ErrorContent';
export const GenericModelVariableRenderer = (props) => {
    // Fields
    const contextName = 'GenericModelVariableRenderer';
    const defaultItemSize = 96;
    const renderModelVariables = (modelVariables) => {
        return (_jsx(WrapContainer, { children: modelVariables.map((variable, index) => {
                var changedValue = undefined;
                if (props.changedModelVariables) {
                    const changedVariable = props.changedModelVariables.find(changedVariable => changedVariable.modelLink.key.toLowerCase() === variable.modelLink.key.toLowerCase());
                    if (changedVariable)
                        changedValue = changedVariable.value;
                }
                var elementContainerSize = undefined;
                if (variable.valueEnumerationKey ||
                    TypeValidatorNET.isString(variable.valueType)) {
                    elementContainerSize = SizeEnumeration.l;
                }
                // Check authorized visibility
                var isVisible = true;
                if (variable.visibilityAuthorization > BrudererUserRoleEnumeration.Undefined) {
                    if (props.userRoles) {
                        isVisible = props.userRoles.some(role => role >= variable.visibilityAuthorization);
                        if (!isVisible)
                            return null;
                    }
                    ;
                }
                ;
                // Check for start adornemnt
                var startAdornment = {};
                if (props.onStartAdornment)
                    startAdornment = props.onStartAdornment(variable);
                return (_jsx(ElementContainer, { size: elementContainerSize, children: _jsx(ModelVariableRenderer, { modelVariable: variable, newValue: changedValue, showInitialValueOnChange: true, forceReadOnly: props.forceReadOnly, showStartAdornment: startAdornment.showStartAdornment, StartAdornmentIcon: startAdornment.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onValidationError: props.onValidationError, onModelVariableChanged: props.onModelVariableChanged, onRenderModelVariable: props.onRenderModelVariable }, void 0) }, index));
            }) }, void 0));
    };
    const renderModelContainer = (modelContainerData) => {
        // Check for custom container rendering. If the returned custom element is undefined we render a default one.
        if (props.onRenderModelContainer) {
            const CustomModelContainer = props.onRenderModelContainer(modelContainerData);
            if (CustomModelContainer === null)
                return _jsx(React.Fragment, {}, void 0);
            if (CustomModelContainer !== undefined)
                return _jsx(CustomModelContainer, { changedModelVariables: props.changedModelVariables, onModelVariableChanged: props.onModelVariableChanged }, void 0);
        }
        ;
        return (_jsxs(React.Fragment, { children: [_jsx(Box, { sx: { minHeight: 8 } }, void 0), _jsx(Card, { sx: { ...cardStyle }, elevation: 5, children: _jsx(CardContent, { sx: { ...cardContentStyle }, children: renderModelVariables(modelContainerData.modelContainer.modelVariables) }, void 0) }, void 0), _jsx(Box, { sx: { minHeight: 8 } }, void 0)] }, void 0));
    };
    const renderEnumerableModelContainers = (modelContainerDataStructures) => {
        return (_jsxs(React.Fragment, { children: [_jsx(Box, { sx: { minHeight: 8 } }, void 0), modelContainerDataStructures.map((modelContainerData, renderIndex) => {
                    // Get model container
                    const modelContainer = modelContainerData.modelContainer;
                    // Unshift enumerable model container in child model container list
                    // So we can render the whole content of the enumerable container in a default render pass
                    const subModelComponenContainers = modelContainer.modelContainers;
                    subModelComponenContainers.unshift(modelContainer);
                    // Setup model container data
                    var subModelContainerDataStructures = subModelComponenContainers.map((subModelContainer, index) => {
                        const subModelContainerData = {
                            modelContainer: subModelContainer,
                            renderIndex: index,
                            enumerableParentData: [...modelContainerData.enumerableParentData]
                        };
                        // Add enumeration parent data
                        subModelContainerData.enumerableParentData.push({ enumerationIndex: index, containerModelLink: subModelContainer.modelLink.key });
                        return subModelContainerData;
                    });
                    // Check for custom container header rendering. If the returned custom element is undefined we render a default one.
                    var CustomModelContainerHeader = undefined;
                    if (props.onRenderModelContainerHeader)
                        CustomModelContainerHeader = props.onRenderModelContainerHeader(modelContainerData);
                    var headerText = `${modelContainer.display.value} ${renderIndex + 1}`;
                    if (props.onLocalize)
                        headerText = props.onLocalize(modelContainer.display);
                    return (_jsxs(Accordion, { TransitionProps: {
                            unmountOnExit: true
                        }, square: true, elevation: 5, children: [_jsxs(AccordionSummary, { expandIcon: _jsx(ExpandMoreIcon, {}, void 0), id: renderIndex.toString(), sx: [
                                    (theme) => ({
                                        '&.MuiAccordionSummary-root': {
                                            ...accordionSummaryRootStyle,
                                        },
                                        '& .MuiAccordionSummary-content': {
                                            ...accordionSummaryContentStyle
                                        },
                                        '&.Mui-expanded': {
                                            ...accordionSummaryExpandedStyle
                                        }
                                    }),
                                    {
                                        '&.MuiAccordionSummary-root': {
                                            minHeight: defaultItemSize,
                                            height: defaultItemSize,
                                        }
                                    }
                                ], children: [(CustomModelContainerHeader !== null) && (CustomModelContainerHeader !== undefined) && CustomModelContainerHeader, (CustomModelContainerHeader === undefined) &&
                                        _jsxs(Box, { sx: {
                                                width: '100%',
                                                display: 'flex',
                                                flexDirection: 'row',
                                                alignContent: 'center',
                                                alignItems: 'center'
                                            }, children: [_jsx(Box, { sx: { minWidth: 8 } }, void 0), _jsx(Box, { sx: {
                                                        display: 'flex',
                                                        flexDirection: 'column',
                                                        flex: 'auto'
                                                    }, children: _jsx(Typography, { variant: 'h5', children: headerText }, void 0) }, void 0)] }, void 0)] }, void 0), _jsx(AccordionDetails, { sx: (theme) => ({
                                    '&.MuiAccordionDetails-root': {
                                        ...accordionDetailsRootStyle
                                    }
                                }), children: renderModelContainers(subModelContainerDataStructures) }, void 0)] }, renderIndex));
                }), _jsx(Box, { sx: { minHeight: 8 } }, void 0)] }, void 0));
    };
    const renderModelContainers = (modelContainerDataStructures) => {
        return (_jsx(React.Fragment, { children: modelContainerDataStructures.map((modelContainerData, renderIndex) => {
                // Get model container
                const modelContainer = modelContainerData.modelContainer;
                // No rendering if no content is present
                var hasContent = (modelContainer.modelVariables.length > 0) || modelContainer.isEnumerableContainer;
                if (!hasContent)
                    return undefined;
                // Check for custom container header rendering. If the returned custom element is undefined we render a default one.
                var CustomModelContainerHeader = undefined;
                if (props.onRenderModelContainerHeader)
                    CustomModelContainerHeader = props.onRenderModelContainerHeader(modelContainerData);
                // Setup label text
                var hasLabelText = modelContainer.display.link.key !== '';
                var headerText = modelContainer.display.value;
                if (props.onLocalize && hasLabelText)
                    headerText = props.onLocalize(modelContainer.display);
                // Setup enumerable container data
                var enumerableModelContainerDataStructures = [];
                if (modelContainer.isEnumerableContainer) {
                    enumerableModelContainerDataStructures = modelContainer.modelContainers.map((enumerableModelContainer, index) => {
                        const enumerableModelContainerData = {
                            modelContainer: enumerableModelContainer,
                            renderIndex: index,
                            enumerableParentData: [...modelContainerData.enumerableParentData]
                        };
                        return enumerableModelContainerData;
                    });
                }
                return (_jsxs(React.Fragment, { children: [(CustomModelContainerHeader !== null) && (CustomModelContainerHeader !== undefined) && CustomModelContainerHeader, (CustomModelContainerHeader === undefined) && hasLabelText &&
                            _jsx(Label, { bottomOffset: 0, children: headerText }, void 0), _jsxs(React.Fragment, { children: [!modelContainer.isEnumerableContainer && renderModelContainer(modelContainerData), modelContainer.isEnumerableContainer && renderEnumerableModelContainers(enumerableModelContainerDataStructures)] }, void 0)] }, renderIndex));
            }) }, void 0));
    };
    // Setup service model variables
    var modelVariables = [];
    if (props.modelVariables)
        modelVariables = props.modelVariables;
    // Filter model variables
    var filteredModelVariables = modelVariables;
    if (props.filter)
        filteredModelVariables = getFilteredVariables(modelVariables, props.filter, props.onLocalize, props.filterFilledByStart);
    if (filteredModelVariables.length <= 0)
        return null;
    // Helpers
    const renderingType = props.renderingType ? props.renderingType : 'auto';
    // Rendering 'auto'
    if ((renderingType === 'auto')) {
        const modelComponentContainers = buildModelContainers(filteredModelVariables, props.modelStructureContainer, props.stackEnumerableItemVariables);
        // Setup container data
        var modelContainerDataStructures = modelComponentContainers.map((modelContainer, index) => {
            const modelContainerData = {
                modelContainer: modelContainer,
                renderIndex: index,
                enumerableParentData: (props.enumerableParentData !== undefined) ? props.enumerableParentData : []
            };
            return modelContainerData;
        });
        return renderModelContainers(modelContainerDataStructures);
    }
    ;
    // Rendering 'flat'
    if (renderingType === 'flat') {
        return (_jsx(Card, { sx: {
                ...cardStyle
            }, elevation: 5, children: _jsx(CardContent, { sx: {
                    ...cardContentStyle
                }, children: renderModelVariables(filteredModelVariables) }, void 0) }, void 0));
    }
    ;
    return (_jsx(ErrorContent, { sourceName: contextName, errorMessage: `Cannot render variables!` }, void 0));
};
