import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React from 'react';
import { TypeValidatorNET, buildVariableContainers, BrudererUserRoleEnumeration } from '@brudererdev/web-extensions';
import { Accordion, AccordionSummary, AccordionDetails, Box, Typography, Card, CardContent } from '@mui/material';
import ExpandMoreIcon from '@mui/icons-material/ExpandMore';
import { cardStyle, cardContentStyle, accordionSummaryRootStyle, accordionSummaryExpandedStyle, accordionSummaryContentStyle, accordionDetailsRootStyle } from './../../styles/themeExtensions';
import { VariableRenderer } from './../renderer/VariableRenderer';
import { Label } from './../elements/Label';
import { WrapContainer } from './../containers/WrapContainer';
import { ElementContainer } from './../containers/ElementContainer';
import { getFilteredVariables } from './../../shared/variableProps';
import { SizeEnumeration } from './../../shared/styleProps';
import { ErrorContent } from './../ErrorContent';
export const GenericVariableRenderer = (props) => {
    // Fields
    const contextName = 'GenericVariableRenderer';
    const defaultItemSize = 96;
    const renderVariablesWrapped = (variables) => {
        return (_jsx(WrapContainer, { children: variables.map((variable, index) => {
                var changedValue = undefined;
                if (props.changedVariables) {
                    const changedVariable = props.changedVariables.find(changedVariable => changedVariable.link.key.toLowerCase() === variable.link.key.toLowerCase());
                    if (changedVariable)
                        changedValue = changedVariable.value;
                }
                var elementContainerSize = undefined;
                if (variable.valueEnumerationKey ||
                    TypeValidatorNET.isString(variable.valueType)) {
                    elementContainerSize = SizeEnumeration.l;
                }
                // Check authorized visibility
                var isVisible = true;
                if (variable.visibilityAuthorization > BrudererUserRoleEnumeration.Undefined) {
                    if (props.userRoles) {
                        isVisible = props.userRoles.some(role => role >= variable.visibilityAuthorization);
                        if (!isVisible)
                            return null;
                    }
                    ;
                }
                ;
                // Check for start adornemnt
                var startAdornment = {};
                if (props.onRenderStartAdornment)
                    startAdornment = props.onRenderStartAdornment(variable);
                return (_jsx(ElementContainer, { size: elementContainerSize, children: _jsx(VariableRenderer, { variable: variable, newValue: changedValue, showInitialValueOnChange: true, forceReadOnly: props.forceReadOnly, showStartAdornment: startAdornment.showStartAdornment, StartAdornmentIcon: startAdornment.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onValidationError: props.onValidationError, onVariableChanged: props.onVariableChanged, onRenderVariable: props.onRenderVariable }, void 0) }, index));
            }) }, void 0));
    };
    const renderVariableContainer = (containerData) => {
        // Check for custom container rendering. If the returned custom element is undefined we render a default one.
        var CustomVariableContainer = undefined;
        if (props.onRenderVariableContainer)
            CustomVariableContainer = props.onRenderVariableContainer(containerData, props.changedVariables, props.onVariableChanged);
        return (_jsxs(React.Fragment, { children: [(CustomVariableContainer !== null) && (CustomVariableContainer !== undefined) && CustomVariableContainer, (CustomVariableContainer === undefined) &&
                    _jsxs(React.Fragment, { children: [_jsx(Box, { sx: { minHeight: 8 } }, void 0), _jsx(Card, { sx: { ...cardStyle }, elevation: 5, children: _jsx(CardContent, { sx: { ...cardContentStyle }, children: renderVariablesWrapped(containerData.container.variables) }, void 0) }, void 0), _jsx(Box, { sx: { minHeight: 8 } }, void 0)] }, void 0)] }, void 0));
    };
    const renderEnumerableVariableContainers = (enumerableVariableContainers, enumerableParents) => {
        return (_jsxs(React.Fragment, { children: [_jsx(Box, { sx: { minHeight: 8 } }, void 0), enumerableVariableContainers.map((enumerableVariableContainer, renderIndex) => {
                    // Setup container data
                    const variableContainerData = {
                        container: enumerableVariableContainer,
                        renderIndex: renderIndex,
                        enumerableParents: [...enumerableParents]
                    };
                    // Check for custom container header rendering. If the returned custom element is undefined we render a default one.
                    var CustomVariableContainerHeader = undefined;
                    if (props.onRenderVariableContainerHeader)
                        CustomVariableContainerHeader = props.onRenderVariableContainerHeader(variableContainerData);
                    var headerText = `${enumerableVariableContainer.display.value} ${renderIndex + 1}`;
                    if (props.onLocalize)
                        headerText = props.onLocalize(enumerableVariableContainer.display);
                    return (_jsxs(Accordion, { TransitionProps: {
                            unmountOnExit: true
                        }, square: true, elevation: 5, children: [_jsxs(AccordionSummary, { expandIcon: _jsx(ExpandMoreIcon, {}, void 0), id: renderIndex.toString(), sx: [
                                    (theme) => ({
                                        '&.MuiAccordionSummary-root': {
                                            ...accordionSummaryRootStyle,
                                        },
                                        '& .MuiAccordionSummary-content': {
                                            ...accordionSummaryContentStyle
                                        },
                                        '&.Mui-expanded': {
                                            ...accordionSummaryExpandedStyle
                                        }
                                    }),
                                    {
                                        '&.MuiAccordionSummary-root': {
                                            minHeight: defaultItemSize,
                                            height: defaultItemSize,
                                        }
                                    }
                                ], children: [(CustomVariableContainerHeader !== null) && (CustomVariableContainerHeader !== undefined) && CustomVariableContainerHeader, (CustomVariableContainerHeader === undefined) &&
                                        _jsxs(Box, { sx: {
                                                width: '100%',
                                                display: 'flex',
                                                flexDirection: 'row',
                                                alignContent: 'center',
                                                alignItems: 'center'
                                            }, children: [_jsx(Box, { sx: { minWidth: 8 } }, void 0), _jsx(Box, { sx: {
                                                        display: 'flex',
                                                        flexDirection: 'column',
                                                        flex: 'auto'
                                                    }, children: _jsx(Typography, { variant: 'h5', children: headerText }, void 0) }, void 0)] }, void 0)] }, void 0), _jsxs(AccordionDetails, { sx: (theme) => ({
                                    '&.MuiAccordionDetails-root': {
                                        ...accordionDetailsRootStyle
                                    }
                                }), children: [renderVariableContainer(variableContainerData), (enumerableVariableContainer.variableContainers.length > 0) &&
                                        _jsx(React.Fragment, { children: renderVariableContainers(enumerableVariableContainer.variableContainers, variableContainerData.enumerableParents) }, void 0)] }, void 0)] }, renderIndex));
                }), _jsx(Box, { sx: { minHeight: 8 } }, void 0)] }, void 0));
    };
    const renderVariableContainers = (variableContainers, enumerableParents) => {
        return (_jsx(React.Fragment, { children: variableContainers.map((variableContainer, renderIndex) => {
                // No rendering if no content is present
                var hasContent = (variableContainer.variables.length > 0) || variableContainer.isEnumerableContainer;
                if (!hasContent)
                    return undefined;
                // Setup container data
                const variableContainerData = {
                    container: variableContainer,
                    renderIndex: renderIndex,
                    enumerableParents: [...enumerableParents]
                };
                // Check for custom container header rendering. If the returned custom element is undefined we render a default one.
                var CustomVariableContainerHeader = undefined;
                if (props.onRenderVariableContainerHeader)
                    CustomVariableContainerHeader = props.onRenderVariableContainerHeader(variableContainerData);
                var headerText = variableContainer.display.value;
                if (props.onLocalize)
                    headerText = props.onLocalize(variableContainer.display);
                return (_jsxs(React.Fragment, { children: [(CustomVariableContainerHeader !== null) && (CustomVariableContainerHeader !== undefined) && CustomVariableContainerHeader, (CustomVariableContainerHeader === undefined) &&
                            _jsx(Label, { bottomOffset: 0, children: headerText }, void 0), _jsxs(React.Fragment, { children: [!variableContainer.isEnumerableContainer && renderVariableContainer(variableContainerData), variableContainer.isEnumerableContainer && renderEnumerableVariableContainers(variableContainer.variableContainers, variableContainerData.enumerableParents)] }, void 0)] }, renderIndex));
            }) }, void 0));
    };
    // Setup service variables
    var variables = [];
    if (props.variables)
        variables = props.variables;
    // Filter variables
    var filteredVariables = variables;
    if (props.filter)
        filteredVariables = getFilteredVariables(variables, props.filter, props.onLocalize, props.filterFilledByStart);
    if (filteredVariables.length <= 0)
        return null;
    // Helpers
    const renderingType = props.renderingType ? props.renderingType : 'auto';
    // Rendering 'custom
    if ((renderingType === 'custom') && props.onRenderVariablesCustom) {
        return props.onRenderVariablesCustom(filteredVariables);
    }
    ;
    // Rendering 'auto'
    if ((renderingType === 'auto')) {
        const variableContainerList = buildVariableContainers(filteredVariables, props.variableContainer, props.stackEnumerableItemVariables);
        const enumerableParents = [];
        if (props.enumerableParents)
            enumerableParents.push(...props.enumerableParents);
        return renderVariableContainers(variableContainerList, enumerableParents);
    }
    ;
    // Rendering 'flat'
    if (renderingType === 'flat') {
        return (_jsx(Card, { sx: {
                ...cardStyle
            }, elevation: 5, children: _jsx(CardContent, { sx: {
                    ...cardContentStyle
                }, children: renderVariablesWrapped(filteredVariables) }, void 0) }, void 0));
    }
    ;
    return (_jsx(ErrorContent, { sourceName: contextName, errorMessage: `Cannot render variables!` }, void 0));
};
