import { jsx as _jsx } from "react/jsx-runtime";
import React from 'react';
import { TypeValidatorNET, BrudererUserRoleEnumeration } from '@brudererdev/web-extensions';
import { VarSelector, VarSwitch, VarStatus, VarTextIn, VarTextOut } from './../elements';
;
export const ModelVariableRenderer = (props) => {
    // Helpers
    const modelVariableProps = props;
    const isReadOnlyForced = (props.forceReadOnly !== undefined) ? props.forceReadOnly : false;
    const isReadOnly = isReadOnlyForced ? isReadOnlyForced : props.modelVariable.isReadOnly;
    // Return empty fragment for a undefined variable
    if (!props.modelVariable)
        return _jsx(React.Fragment, {}, void 0);
    // Check authorized visibility
    var isVisible = true;
    if (props.modelVariable.visibilityAuthorization > BrudererUserRoleEnumeration.Undefined) {
        if (props.userRoles) {
            isVisible = props.userRoles.some(role => role >= props.modelVariable.visibilityAuthorization);
            if (!isVisible)
                return _jsx(React.Fragment, {}, void 0);
        }
        ;
    }
    ;
    // Check authorized manipulation
    var canBeManipulated = true;
    if (props.modelVariable.manipulationAuthorization > BrudererUserRoleEnumeration.Undefined) {
        if (props.userRoles) {
            canBeManipulated = props.userRoles.some(role => role >= props.modelVariable.manipulationAuthorization);
        }
        ;
    }
    ;
    // Return the custom rendered variable
    if (props.onRenderModelVariable) {
        const CustomVariableElement = props.onRenderModelVariable(props.modelVariable);
        if (CustomVariableElement === null)
            return _jsx(React.Fragment, {}, void 0);
        if (CustomVariableElement !== undefined)
            return _jsx(CustomVariableElement, { ...modelVariableProps }, void 0);
    }
    ;
    // Render a selector if the value is an enumeration an not read only
    if (!isReadOnly && props.modelVariable.valueEnumerationKey &&
        canBeManipulated) {
        return (_jsx(VarSelector, { modelVariable: props.modelVariable, newValue: props.newValue, forceReadOnly: props.forceReadOnly, isDisabled: props.isDisabled, showInitialValueOnChange: props.showInitialValueOnChange, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onModelVariableChanged: props.onModelVariableChanged, onValidationError: props.onValidationError }, void 0));
    }
    // Render a text output if the value is an enumeration and read only
    if (isReadOnly && props.modelVariable.valueEnumerationKey &&
        canBeManipulated) {
        return (_jsx(VarTextOut, { modelVariable: props.modelVariable, newValue: props.newValue, isDisabled: props.isDisabled, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onValidationError: props.onValidationError }, void 0));
    }
    // Render a text field for string or a number
    if (!isReadOnly &&
        canBeManipulated &&
        (TypeValidatorNET.isString(props.modelVariable.valueType) ||
            TypeValidatorNET.isNumeric(props.modelVariable.valueType))) {
        return (_jsx(VarTextIn, { modelVariable: props.modelVariable, newValue: props.newValue, isDisabled: props.isDisabled, showInitialValueOnChange: props.showInitialValueOnChange, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onModelVariableChanged: props.onModelVariableChanged, onValidationError: props.onValidationError }, void 0));
    }
    // Render a text out for a read-only string or number
    if ((isReadOnly || !canBeManipulated) &&
        (TypeValidatorNET.isString(props.modelVariable.valueType) ||
            TypeValidatorNET.isNumeric(props.modelVariable.valueType))) {
        return (_jsx(VarTextOut, { modelVariable: props.modelVariable, newValue: props.newValue, isDisabled: props.isDisabled, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onValidationError: props.onValidationError }, void 0));
    }
    // Render a switch for a boolean
    if (!isReadOnly &&
        canBeManipulated &&
        TypeValidatorNET.isBoolean(props.modelVariable.valueType)) {
        return (_jsx(VarSwitch, { modelVariable: props.modelVariable, newValue: props.newValue, forceReadOnly: props.forceReadOnly, isDisabled: props.isDisabled, userRoles: props.userRoles, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, locale: props.locale, onLocalize: props.onLocalize, onModelVariableChanged: props.onModelVariableChanged, onValidationError: props.onValidationError }, void 0));
    }
    // Render a status for a read-only boolean 
    if ((isReadOnly || !canBeManipulated) &&
        TypeValidatorNET.isBoolean(props.modelVariable.valueType)) {
        return (_jsx(VarStatus, { modelVariable: props.modelVariable, newValue: props.newValue, isDisabled: props.isDisabled, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onValidationError: props.onValidationError }, void 0));
    }
    return _jsx(React.Fragment, {}, void 0);
};
