import { jsx as _jsx } from "@emotion/react/jsx-runtime";
/* eslint-disable react/react-in-jsx-scope -- Unaware of jsxImportSource */
/** @jsxImportSource @emotion/react */
import { useContext } from 'react';
import { NavigationTypeEnumeration, BrudererUserRoleEnumeration } from '@brudererdev/web-extensions';
import { Box } from '@mui/material';
import { LabelTile } from '../elements';
import { ScrollContainer } from '../containers/ScrollContainer';
import { ElementContainer } from '../containers/ElementContainer';
import { WrapContainer } from '../containers/WrapContainer';
import { BrudererContext } from '../..';
export const NavigationElementRenderer = (props) => {
    // Contexts
    const brudererContext = useContext(BrudererContext);
    const getTileStyle = (index, type) => {
        var tileStyle = {
            backgroundColor: (theme) => theme.palette.primary.main,
            color: (theme) => theme.palette.primary.contrastText,
            fill: (theme) => theme.palette.primary.contrastText
        };
        if (index === props.selectedElementIndex)
            tileStyle = {
                backgroundColor: (theme) => theme.palette.secondary.main,
                color: (theme) => theme.palette.secondary.contrastText,
                fill: (theme) => theme.palette.secondary.contrastText
            };
        if (type === NavigationTypeEnumeration.Action)
            tileStyle = {
                backgroundColor: (theme) => theme.palette.info.main,
                color: (theme) => theme.palette.info.contrastText,
                fill: (theme) => theme.palette.info.contrastText
            };
        if (type === NavigationTypeEnumeration.Dialog)
            tileStyle = {
                backgroundColor: (theme) => theme.palette.info.main,
                color: (theme) => theme.palette.info.contrastText,
                fill: (theme) => theme.palette.info.contrastText
            };
        return tileStyle;
    };
    const renderIcon = (IconElement) => {
        if (!IconElement)
            return;
        if (typeof IconElement === 'string')
            return (_jsx(Box, { component: 'span', sx: { fontSize: 36 }, children: IconElement }, void 0));
        return (_jsx(IconElement, { css: {
                minWidth: 48,
                minHeight: 48,
                width: 48,
                height: 48
            } }, void 0));
    };
    const renderTiles = () => {
        return (props.elements.map((element, index) => {
            if ((element.visibilityAuthorization !== undefined) && element.visibilityAuthorization > BrudererUserRoleEnumeration.Undefined) {
                if (props.userRoles) {
                    var isVisible = props.userRoles.some(role => role >= element.visibilityAuthorization);
                    if (!isVisible)
                        return null;
                }
                ;
            }
            var localizedTitle = (props.onLocalize !== undefined) ? props.onLocalize(element.display) : element.display.value;
            return (_jsx(ElementContainer, { children: _jsx(LabelTile, { tileStyle: getTileStyle(index, element.type), text: localizedTitle, onClick: () => { props.onClick(element, index); }, children: renderIcon(element.icon) }, void 0) }, index));
        }));
    };
    // Helpers
    const noWrap = (props.noWrap) ? props.noWrap : false;
    return ((!noWrap) ?
        _jsx(WrapContainer, { children: renderTiles() }, void 0) :
        _jsx(ScrollContainer, { orientation: 'horizontal', children: renderTiles() }, void 0));
};
