import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import React, { useContext, useEffect, useRef, useState } from "react";
import { useTheme } from '@mui/material/styles';
import { Badge, Box, Typography } from '@mui/material';
import { CrossCuttingServiceKeys, LocalizationNamespaces, MessageLevelEnumeration, StampingPressModelServiceKeys } from "@brudererdev/web-extensions";
import ErrorIcon from '@mui/icons-material/Error';
import WarningIcon from '@mui/icons-material/Warning';
import InfoIcon from '@mui/icons-material/Info';
import { BrudererContext, SizeEnumeration } from "../..";
import { SizeInjectorContainer } from "../containers";
import { Indicator2 } from "../indicators";
export var StatusContextEnumeration;
(function (StatusContextEnumeration) {
    StatusContextEnumeration[StatusContextEnumeration["Unknown"] = 0] = "Unknown";
    StatusContextEnumeration[StatusContextEnumeration["Message"] = 1] = "Message";
    StatusContextEnumeration[StatusContextEnumeration["Clutch"] = 2] = "Clutch";
})(StatusContextEnumeration || (StatusContextEnumeration = {}));
;
;
;
;
;
const StatusRendererComponent = (props) => {
    // External hooks
    const themeStyles = useTheme();
    // Fields
    const contextName = 'StatusRenderer';
    // States
    const [messageServiceVersion, setMessageServiceVersion] = useState(-1);
    const [clutchServiceVersion, setClutchServiceVersion] = useState(-1);
    // Refs
    const subscriberKeyDictionaryRef = useRef({});
    // Contexts
    const brudererContext = useContext(BrudererContext);
    const messageService = brudererContext.getService(StampingPressModelServiceKeys.MessageService);
    const clutchService = brudererContext.getService(StampingPressModelServiceKeys.ClutchService);
    const navigationService = brudererContext.getService(CrossCuttingServiceKeys.NavigationService);
    // Conditions
    const activeErrorMessages = messageService?.activeErrorMessages ? messageService.activeErrorMessages : [];
    const isEngaged = clutchService ? clutchService.isEngaged : false;
    // Effects
    useEffect(() => {
        // Mount
        if (messageService) {
            const registerKey = messageService.onChanges(contextName, handleMessageServiceChanges);
            subscriberKeyDictionaryRef.current[messageService.key] = registerKey;
        }
        ;
        if (clutchService) {
            const registerKey = clutchService.onChanges(contextName, handleClutchServiceChanges);
            subscriberKeyDictionaryRef.current[clutchService.key] = registerKey;
        }
        ;
        // Unmount
        return () => {
            if (messageService) {
                const registerKey = subscriberKeyDictionaryRef.current[messageService.key];
                messageService.offChanges(registerKey);
            }
            ;
            if (clutchService) {
                const registerKey = subscriberKeyDictionaryRef.current[clutchService.key];
                clutchService.offChanges(registerKey);
            }
            ;
            subscriberKeyDictionaryRef.current = {};
        };
    }, []);
    useEffect(() => {
        var statusContext = {
            contextType: StatusContextEnumeration.Unknown
        };
        const primaryMessage = (activeErrorMessages.length > 0) ? activeErrorMessages[0] : undefined;
        if (primaryMessage) {
            statusContext = {
                contextType: StatusContextEnumeration.Message,
                style: getMessageStatusStyle(primaryMessage),
                context: {
                    message: primaryMessage
                }
            };
            if (props.onStatusContextChanged)
                props.onStatusContextChanged(statusContext);
            return;
        }
        ;
        var statusContext = {
            contextType: StatusContextEnumeration.Clutch,
            style: getClutchStatusStyle(isEngaged),
            context: {
                isEngaged: isEngaged
            }
        };
        if (props.onStatusContextChanged)
            props.onStatusContextChanged(statusContext);
    }, [messageServiceVersion, clutchServiceVersion]);
    const handleMessageServiceChanges = (version, key) => {
        setMessageServiceVersion(version);
    };
    const handleClutchServiceChanges = (version, key) => {
        setClutchServiceVersion(version);
    };
    const getSizes = () => {
        switch (props.type) {
            case 'MessageBar':
                iconSize = SizeEnumeration.m;
                iconSizeValue = 32;
                iconWrapperWidth = 64;
                numberTypographyVariant = 'h5';
                textTypographyVariant = 'h6';
                break;
            case 'ActionTile':
            default:
                iconSize = SizeEnumeration.l;
                iconSizeValue = 64;
                iconWrapperWidth = 100;
                numberTypographyVariant = 'h4';
                textTypographyVariant = 'h5';
                break;
        }
    };
    const getMessageIcon = (message) => {
        switch (message.level) {
            case MessageLevelEnumeration.Error:
                return ErrorIcon;
            case MessageLevelEnumeration.Warning:
                return WarningIcon;
            case MessageLevelEnumeration.Info:
                return InfoIcon;
            default:
                return InfoIcon;
        }
    };
    const getMessageStatusStyle = (message) => {
        var statusStyle = {};
        switch (message.level) {
            case MessageLevelEnumeration.Error:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.error.main,
                    color: (theme) => theme.palette.error.contrastText,
                    fill: (theme) => theme.palette.error.contrastText
                };
            case MessageLevelEnumeration.Warning:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.warning.main,
                    color: (theme) => theme.palette.warning.contrastText,
                    fill: (theme) => theme.palette.warning.contrastText
                };
            case MessageLevelEnumeration.Info:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.info.main,
                    color: (theme) => theme.palette.info.contrastText,
                    fill: (theme) => theme.palette.info.contrastText
                };
            default:
                return statusStyle = {
                    backgroundColor: (theme) => theme.palette.view.main,
                    color: (theme) => theme.palette.view.contrastText,
                    fill: (theme) => theme.palette.view.contrastText
                };
        }
        ;
    };
    const getClutchStatusStyle = (isEngaged) => {
        var statusStyle = {};
        if (isEngaged)
            return statusStyle = {
                backgroundColor: (theme) => theme.palette.success.main,
                color: (theme) => theme.palette.success.contrastText,
                fill: (theme) => theme.palette.success.contrastText
            };
        else
            return statusStyle = {
                backgroundColor: (theme) => theme.palette.view.main,
                color: (theme) => theme.palette.view.contrastText,
                fill: (theme) => theme.palette.view.contrastText
            };
    };
    const getContentContainerStyle = () => {
        var containerStyle = {
            display: 'flex',
            flexDirection: 'column',
            justifyContent: 'center'
        };
        if (props.type === 'MessageBar')
            containerStyle = {
                display: 'flex',
                flexDirection: 'row',
                justifyContent: 'center'
            };
        return containerStyle;
    };
    const renderStatusMessage = (message) => {
        const MessageIcon = getMessageIcon(message);
        var localizedMessageText = message.display.value;
        if (!message.hasCustomText)
            localizedMessageText = brudererContext.localize(message.display);
        var messageCount = 1;
        if (messageService)
            messageCount = messageService.activeErrorMessages.length;
        return (_jsxs(Box, { sx: {
                height: '100%',
                display: 'flex',
                flexDirection: 'row',
                justifyContent: 'center',
                justifyItems: 'center',
                alignContent: 'center',
                alignItems: 'center'
            }, children: [_jsx(Box, { sx: {
                        height: '100%',
                        minWidth: iconWrapperWidth,
                        width: iconWrapperWidth,
                        display: 'flex',
                        flexDirection: 'column',
                        justifyContent: 'center',
                        justifyItems: 'center',
                        alignContent: 'center',
                        alignItems: 'center'
                    }, children: _jsx(Badge, { badgeContent: messageCount, overlap: "circular", color: 'primary', children: _jsx(MessageIcon, { sx: {
                                minWidth: iconSizeValue,
                                minHeight: iconSizeValue,
                                width: iconSizeValue,
                                height: iconSizeValue
                            } }, void 0) }, void 0) }, void 0), _jsxs(SizeInjectorContainer, { containerStyle: getContentContainerStyle(), children: [_jsx(Typography, { variant: numberTypographyVariant, children: message.messageID.toString() }, void 0), _jsx(Box, { sx: { minWidth: (theme) => theme.spacing(1) } }, void 0), _jsx(Typography, { variant: textTypographyVariant, children: localizedMessageText }, void 0)] }, void 0)] }, void 0));
    };
    const renderEngagedIndicator = () => {
        const clutchStateText = isEngaged ?
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.engaged', 'Eingekuppelt') :
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.disengaged', 'Ausgekuppelt');
        return (_jsxs(Box, { sx: {
                height: '100%',
                display: 'flex',
                flexDirection: 'row',
                justifyContent: 'center',
                justifyItems: 'center',
                alignContent: 'center',
                alignItems: 'center'
            }, children: [_jsx(Box, { sx: {
                        height: '100%',
                        minWidth: iconWrapperWidth,
                        width: iconWrapperWidth,
                        display: 'flex',
                        flexDirection: 'column',
                        justifyContent: 'center',
                        justifyItems: 'center',
                        alignContent: 'center',
                        alignItems: 'center'
                    }, children: _jsx(Indicator2, { backgroundColor: themeStyles.palette.success.contrastText, size: iconSize }, void 0) }, void 0), _jsx(Box, { sx: {
                        flex: 'auto',
                        display: 'flex',
                        flexDirection: 'column'
                    }, children: _jsx(Typography, { variant: numberTypographyVariant, children: clutchStateText }, void 0) }, void 0)] }, void 0));
    };
    const renderDisengagedIndicator = () => {
        const clutchStateText = isEngaged ?
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.engaged', 'Eingekuppelt') :
            brudererContext.localizeRaw(LocalizationNamespaces.StampingPressServices, 'services.clutchservice.disengaged', 'Ausgekuppelt');
        return (_jsxs(Box, { sx: {
                height: '100%',
                display: 'flex',
                flexDirection: 'row',
                justifyContent: 'center',
                justifyItems: 'center',
                alignContent: 'center',
                alignItems: 'center'
            }, children: [_jsx(Box, { sx: {
                        height: '100%',
                        minWidth: iconWrapperWidth,
                        width: iconWrapperWidth,
                        display: 'flex',
                        flexDirection: 'column',
                        justifyContent: 'center',
                        justifyItems: 'center',
                        alignContent: 'center',
                        alignItems: 'center'
                    }, children: _jsx(Box, { sx: {
                            opacity: 0.6,
                            borderRadius: '50%',
                            height: iconSizeValue,
                            width: iconSizeValue,
                            backgroundColor: themeStyles.palette.success.contrastText
                        } }, void 0) }, void 0), _jsx(Box, { sx: {
                        flex: 'auto',
                        display: 'flex',
                        flexDirection: 'column'
                    }, children: _jsx(Typography, { variant: numberTypographyVariant, children: clutchStateText }, void 0) }, void 0)] }, void 0));
    };
    const renderContent = () => {
        const primaryMessage = (activeErrorMessages.length > 0) ? activeErrorMessages[0] : undefined;
        if (primaryMessage)
            return renderStatusMessage(primaryMessage);
        if (isEngaged)
            return renderEngagedIndicator();
        return renderDisengagedIndicator();
    };
    // Helpers
    var iconSize;
    var iconSizeValue;
    var iconWrapperWidth;
    var numberTypographyVariant;
    var textTypographyVariant;
    getSizes();
    return (_jsx(React.Fragment, { children: renderContent() }, void 0));
};
export const StatusRenderer = React.memo(StatusRendererComponent);
