import { jsx as _jsx } from "react/jsx-runtime";
import React from 'react';
import { TypeValidatorNET, BrudererUserRoleEnumeration } from '@brudererdev/web-extensions';
import { VarSelector, VarSwitch, VarStatus, VarTextIn, VarTextOut } from './../elements';
;
export const VariableRenderer = (props) => {
    // Helpers
    const variableProps = props;
    const isReadOnlyForced = (props.forceReadOnly !== undefined) ? props.forceReadOnly : false;
    const isReadOnly = isReadOnlyForced ? isReadOnlyForced : props.variable.isReadOnly;
    // Return empty fragment for a undefined variable
    if (!props.variable)
        return _jsx(React.Fragment, {}, void 0);
    // Check authorized visibility
    var isVisible = true;
    if (props.variable.visibilityAuthorization > BrudererUserRoleEnumeration.Undefined) {
        if (props.userRoles) {
            isVisible = props.userRoles.some(role => role >= props.variable.visibilityAuthorization);
            if (!isVisible)
                return _jsx(React.Fragment, {}, void 0);
        }
        ;
    }
    ;
    // Check authorized manipulation
    var canBeManipulated = true;
    if (props.variable.manipulationAuthorization > BrudererUserRoleEnumeration.Undefined) {
        if (props.userRoles) {
            canBeManipulated = props.userRoles.some(role => role >= props.variable.manipulationAuthorization);
        }
        ;
    }
    ;
    // Return the custom rendered variable
    if (props.onRenderVariable) {
        const CustomVariableElement = props.onRenderVariable(variableProps);
        if (CustomVariableElement)
            return (_jsx(React.Fragment, { children: CustomVariableElement }, void 0));
    }
    ;
    // Render a selector if the value is an enumeration
    if (props.variable.valueEnumerationKey &&
        canBeManipulated) {
        return (_jsx(VarSelector, { variable: props.variable, newValue: props.newValue, forceReadOnly: props.forceReadOnly, isDisabled: props.isDisabled, showInitialValueOnChange: props.showInitialValueOnChange, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onVariableChanged: props.onVariableChanged, onValidationError: props.onValidationError }, void 0));
    }
    // Render a text field for string or a number
    if (!isReadOnly &&
        canBeManipulated &&
        (TypeValidatorNET.isString(props.variable.valueType) ||
            TypeValidatorNET.isNumeric(props.variable.valueType))) {
        return (_jsx(VarTextIn, { variable: props.variable, newValue: props.newValue, isDisabled: props.isDisabled, showInitialValueOnChange: props.showInitialValueOnChange, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onVariableChanged: props.onVariableChanged, onValidationError: props.onValidationError }, void 0));
    }
    // Render a text out for a read-only string or number
    if ((isReadOnly || !canBeManipulated) &&
        (TypeValidatorNET.isString(props.variable.valueType) ||
            TypeValidatorNET.isNumeric(props.variable.valueType))) {
        return (_jsx(VarTextOut, { variable: props.variable, newValue: props.newValue, isDisabled: props.isDisabled, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onValidationError: props.onValidationError }, void 0));
    }
    // Render a switch for a boolean
    if (!isReadOnly &&
        canBeManipulated &&
        TypeValidatorNET.isBoolean(props.variable.valueType)) {
        return (_jsx(VarSwitch, { variable: props.variable, newValue: props.newValue, forceReadOnly: props.forceReadOnly, isDisabled: props.isDisabled, userRoles: props.userRoles, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, locale: props.locale, onLocalize: props.onLocalize, onVariableChanged: props.onVariableChanged, onValidationError: props.onValidationError }, void 0));
    }
    // Render a status for a read-only boolean 
    if ((isReadOnly || !canBeManipulated) &&
        TypeValidatorNET.isBoolean(props.variable.valueType)) {
        return (_jsx(VarStatus, { variable: props.variable, newValue: props.newValue, isDisabled: props.isDisabled, showStartAdornment: props.showStartAdornment, StartAdornmentIcon: props.StartAdornmentIcon, userRoles: props.userRoles, locale: props.locale, onLocalize: props.onLocalize, onValidationError: props.onValidationError }, void 0));
    }
    return _jsx(React.Fragment, {}, void 0);
};
