import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useEffect, useRef, useState } from 'react';
import { LocalizationNamespaces } from '@brudererdev/web-extensions';
import { Box, Stepper, Step, StepConnector, StepLabel, StepContent, Card, CardContent, Typography } from '@mui/material';
import CheckCircleIcon from '@mui/icons-material/CheckCircle';
import RadioButtonUncheckedIcon from '@mui/icons-material/RadioButtonUnchecked';
import RadioButtonCheckedIcon from '@mui/icons-material/RadioButtonChecked';
import { scrollVerticalStyle, scrollHorizontalStyle, cardStyle, cardContentStyle } from './../../styles/themeExtensions';
import { CommandBar } from '../bars/CommandBar';
export const GenericStepper = (props) => {
    // Refs
    const stepContentRef = useRef(null);
    // States
    var [internalCurrentStep, setActiveStep] = useState(0);
    var [actualCurrentStep, setActualCurrentStep] = useState(0);
    // Effects
    useEffect(() => {
        // Mount
        // Unmount
        return () => {
        };
    }, []);
    // update current step
    useEffect(() => {
        getCurrentStep();
    }, [internalCurrentStep, props.externalCurrentStep]);
    useEffect(() => {
        props.onStepChange(actualCurrentStep);
        stepContentRef.current?.scrollIntoView({ behavior: 'smooth', block: 'center', inline: 'center' });
    }, [actualCurrentStep]);
    const getSteps = () => {
        var localizedSteps = [];
        props.steps.forEach((step) => {
            var result = (props.onLocalize !== undefined) ? props.onLocalize(step) : step.value;
            if (result)
                localizedSteps.push(result);
        });
        return localizedSteps;
    };
    const handleNext = () => {
        setActiveStep(internalCurrentStep + 1);
    };
    const handleBack = () => {
        setActiveStep(internalCurrentStep - 1);
    };
    const handleReset = () => {
        props.onStepChange(0);
        setActiveStep((step) => step = 0);
        if (props.onReset)
            props.onReset();
    };
    const getScrollStyle = () => {
        if (props.orientation === 'vertical')
            return scrollVerticalStyle;
        return scrollHorizontalStyle;
    };
    const getCurrentStep = () => {
        props.externalCurrentStep ? setActualCurrentStep(props.externalCurrentStep) : setActualCurrentStep(internalCurrentStep);
        return actualCurrentStep;
    };
    const getCommandTiles = () => {
        // Setup "Reset" text
        var resetText = 'Reset';
        if (props.onLocalize)
            resetText = props.onLocalize({
                link: {
                    key: 'views.elements.stepper.reset',
                    path: 'views.elements.stepper',
                    name: 'reset',
                    seperator: '.'
                },
                keyNamespace: LocalizationNamespaces.UIGeneric,
                value: 'Reset'
            });
        // Setup "Next" text
        var nextText = 'Next';
        if (props.onLocalize)
            nextText = props.onLocalize({
                link: {
                    key: 'views.elements.stepper.next',
                    path: 'views.elements.stepper',
                    name: 'next',
                    seperator: '.'
                },
                keyNamespace: LocalizationNamespaces.UIGeneric,
                value: 'Next'
            });
        // Setup "Back" text
        var backText = 'Back';
        if (props.onLocalize)
            backText = props.onLocalize({
                link: {
                    key: 'views.elements.stepper.back',
                    path: 'views.elements.stepper',
                    name: 'back',
                    seperator: '.'
                },
                keyNamespace: LocalizationNamespaces.UIGeneric,
                value: 'Back'
            });
        //All possible tiles
        var result = [];
        var reset = {
            key: 'reset',
            onClick: (e) => handleReset(),
            tileStyle: {
                backgroundColor: (theme) => theme.palette.error.main,
                color: (theme) => theme.palette.error.contrastText,
                fill: (theme) => theme.palette.error.contrastText
            },
            minWidth: 128,
            content: _jsx(Typography, { variant: 'h6', children: resetText }, void 0)
        };
        var next = {
            key: 'next',
            onClick: (e) => handleNext(),
            tileStyle: {
                backgroundColor: (theme) => theme.palette.success.main,
                color: (theme) => theme.palette.success.contrastText,
                fill: (theme) => theme.palette.success.contrastText
            },
            minWidth: 128,
            content: _jsx(Typography, { variant: 'h6', children: nextText }, void 0)
        };
        var back = {
            key: 'back',
            onClick: (e) => handleBack(),
            tileStyle: {
                backgroundColor: (theme) => theme.palette.info.main,
                color: (theme) => theme.palette.info.contrastText,
                fill: (theme) => theme.palette.info.contrastText
            },
            minWidth: 128,
            content: _jsx(Typography, { variant: 'h6', children: backText }, void 0)
        };
        result.push(reset);
        if (props.externalCurrentStep === undefined) {
            if (actualCurrentStep > 0) {
                result.unshift(back);
            }
            result.unshift(next);
        }
        return result;
    };
    const getStepIcon = (props) => {
        const { active, completed } = props;
        const CompletedIcon = completed ?
            _jsx(CheckCircleIcon, { sx: {
                    height: 48,
                    width: 48,
                    color: (theme) => theme.palette.success.main,
                    fill: (theme) => theme.palette.success.main
                } }, void 0)
            :
                _jsx(RadioButtonUncheckedIcon, { sx: {
                        height: 48,
                        width: 48,
                        color: (theme) => theme.palette.grey[500],
                        fill: (theme) => theme.palette.grey[500]
                    } }, void 0);
        const ActiveIcon = _jsx(RadioButtonCheckedIcon, { sx: {
                height: 48,
                width: 48,
                color: (theme) => theme.palette.secondary.main,
                fill: (theme) => theme.palette.secondary.main
            } }, void 0);
        return active ? ActiveIcon : CompletedIcon;
    };
    const renderConnector = (props) => {
        return (_jsx(StepConnector, { sx: {
                '& .MuiStepConnector-root': {},
                '& .MuiStepConnector-line': {
                    borderWidth: 4
                },
                '& .MuiStepConnector-active': {},
                '& .MuiStepConnector-lineVertical': {
                    marginLeft: '8px'
                },
                '& .MuiStepConnector-lineHorizontal': {
                    marginTop: (theme) => theme.spacing(1),
                    marginLeft: (theme) => theme.spacing(1),
                    marginRight: (theme) => theme.spacing(1),
                },
                '&.MuiStepConnector-vertical': {},
                '&.MuiStepConnector-horizontal': {},
            } }, void 0));
    };
    const renderCardContent = () => {
        if (props.children) {
            return (_jsx(Card, { sx: { ...cardStyle }, elevation: 5, children: _jsx(CardContent, { sx: { ...cardContentStyle }, children: props.children }, void 0) }, void 0));
        }
        ;
    };
    //Helper
    var localizedSteps = getSteps();
    return (_jsxs(Box, { sx: {
            height: '100%',
            width: '100%',
            flex: 'auto',
            display: 'flex',
            flexDirection: 'column'
        }, children: [_jsx(Box, { sx: {
                    height: 'auto',
                    width: '100%',
                    display: 'flex',
                    flexDirection: 'column',
                    justifyContent: 'flex-start',
                    ...getScrollStyle()
                }, children: _jsx(Stepper, { sx: {
                        '&.MuiStepper-root': {
                            backgroundColor: 'inherit'
                        }
                    }, activeStep: actualCurrentStep, orientation: props.orientation, alternativeLabel: props.orientation === 'horizontal', connector: renderConnector(props.orientation), children: localizedSteps.map((label, index) => (_jsxs(Step, { children: [_jsx(Box, { ref: (index === actualCurrentStep) ? stepContentRef : null }, void 0), _jsx(StepLabel, { StepIconComponent: getStepIcon, children: _jsx(Typography, { variant: 'h5', children: label }, void 0) }, void 0), (props.orientation === 'vertical') &&
                                _jsx(StepContent, { sx: {
                                        '&.MuiStepContent-root': {
                                            width: '100%',
                                            borderWidth: (props.orientation === 'vertical') ? 4 : 0,
                                            padding: (props.orientation === 'vertical') ? undefined : 0,
                                            marginLeft: (props.orientation === 'vertical') ? '20px' : undefined,
                                        },
                                    }, children: renderCardContent() }, void 0)] }, label))) }, void 0) }, void 0), (props.orientation === 'horizontal') &&
                _jsx(Box, { sx: {
                        margin: (theme) => theme.spacing(1),
                        backgroundColor: 'red'
                    }, children: renderCardContent() }, void 0), _jsx(Box, { sx: { flex: 'auto' } }, void 0), _jsx(CommandBar, { commandTiles: getCommandTiles() }, void 0)] }, void 0));
};
