import { useEffect, useRef, useCallback } from 'react';
export const useDebounceFunction = (f, delay, options) => {
    // Refs
    const functionArgs = useRef([]);
    const functionTimeoutHandler = useRef(undefined);
    const isUnmounted = useRef(false);
    const debouncedFunction = useRef(f);
    debouncedFunction.current = f;
    // Effects
    useEffect(() => {
        // Mount
        // Unmount
        return () => {
            isUnmounted.current = true;
        };
    }, []);
    const debouncedCallback = useCallback((...args) => {
        functionArgs.current = args;
        window.clearTimeout(functionTimeoutHandler.current);
        functionTimeoutHandler.current = window.setTimeout(() => {
            cancelDebouncedFunction();
            if (!isUnmounted.current) {
                debouncedFunction.current(...args);
            }
        }, delay);
    }, []);
    const cancelDebouncedFunction = useCallback(() => {
        clearTimeout(functionTimeoutHandler.current);
        functionArgs.current = [];
    }, []);
    const callPendingDebouncedFunction = () => {
        // Call pending callback only if we have anything in our queue
        if (!functionTimeoutHandler.current) {
            return;
        }
        debouncedFunction.current.apply(null, functionArgs.current);
        cancelDebouncedFunction();
    };
    return [debouncedCallback, cancelDebouncedFunction, callPendingDebouncedFunction];
};
