import React from 'react';
import { IBaseService, IServiceDictionary, IResponse, IUserRegistration, IUserLogin, IUser, IAuthenticatedUser, ILocalizableContent, IDynamicValueDictionary, LocalizeMethod } from '@brudererdev/web-extensions';
export interface BrudererContextProps {
    activeUser?: IAuthenticatedUser;
    activeLanguage: string;
    activeThemeName: string;
    serviceDictionary: IServiceDictionary;
    failedServices: Array<IBaseService>;
    isStartingServices: boolean;
    isStoppingServices: boolean;
    isPLCConnected: boolean;
    isDeveloperModeActive: boolean;
    isServiceAutoReconnectionActive: boolean;
    registerUser: (registerCredentials: IUserRegistration) => Promise<IResponse<boolean>>;
    loginUser: (loginCredentials: IUserLogin) => Promise<IResponse<boolean>>;
    logoutUser: () => Promise<IResponse<boolean>>;
    getUsers: () => Promise<IResponse<Array<IUser>>>;
    addService: <T extends IBaseService>(service: T, serviceKey: string) => void;
    getService: <T extends IBaseService>(serviceKey: string) => T | undefined;
    startServices: () => Promise<boolean>;
    stopServices: () => Promise<boolean>;
    changeTheme: (themeName: string) => void;
    setLanguage: (locale: string) => void;
    localize: (content: ILocalizableContent) => string;
    localizeRaw: (keyNamespace: string, key: string, value: string, dynamicValueDictionary?: IDynamicValueDictionary) => string;
    onLocalize: (contextKey: string, callbackHandler: (content: ILocalizableContent) => string) => string;
    offLocalize: (registerKey: string) => boolean;
    setDeveloperMode: (enabled: boolean) => void;
    setServiceAutoReconnection: (enabled: boolean) => void;
}
export declare const BrudererContext: React.Context<BrudererContextProps>;
export interface BrudererProviderProps {
    onLocalize?: LocalizeMethod;
    onThemeChange?: (themeName: string) => void;
}
interface ILocalProps {
}
declare type Props = ILocalProps & BrudererProviderProps;
export declare const BrudererProvider: React.FC<Props>;
export {};
