/// <reference types="react" />
import { IModelBaseService, LocalizeMethod, BrudererUserRoleEnumeration, IModelVariable, IModelComponentContainer, IModelStructureContainer, ITextNotification, FilterTypeEnumeration } from '@brudererdev/web-extensions';
export interface IModelVariableProps {
    modelVariable: IModelVariable;
    /**
   * passed new value of the variable after invoking the method onRenderModelVariable.
   */
    newValue?: any;
    forceReadOnly?: boolean;
    isDisabled?: boolean;
    showInitialValueOnChange?: boolean;
    showStartAdornment?: boolean;
    StartAdornmentIcon?: React.FunctionComponent<any> | React.ComponentType<any> | string;
}
export interface IModelVariableContextProps {
    userRoles?: Array<BrudererUserRoleEnumeration>;
    locale?: string;
    onLocalize?: LocalizeMethod;
    onModelVariableChanged?: (modelVariable: IModelVariable) => void;
    onValidationError?: (notification: ITextNotification) => void;
}
export interface IModelVariableFilterProps {
    filterText?: string | Array<string>;
    filterType?: FilterTypeEnumeration;
    filterByLocalization?: boolean;
    filterByLinkKey?: boolean;
    minimumFilterLength?: number;
}
export interface IModelVariableStartAdornment {
    showStartAdornment?: boolean;
    StartAdornmentIcon?: React.FunctionComponent<any> | React.ComponentType<any> | string;
}
export interface IModelContainerEnumerationData {
    enumerationIndex: number;
    containerModelLink: string;
}
export interface IModelContainerData {
    modelContainer: IModelComponentContainer;
    renderIndex: number;
    enumerableParentData: Array<IModelContainerEnumerationData>;
}
export interface IGenericModelRenderingProps {
    filter?: Array<IModelVariableFilterProps>;
    filterFilledByStart?: boolean;
    renderingType?: 'auto' | 'flat';
    stackEnumerableItemVariables?: boolean;
    forceReadOnly?: boolean;
    userRoles: Array<BrudererUserRoleEnumeration>;
    locale?: string;
    onLocalize?: LocalizeMethod;
    onRenderModelContainerHeader?: (modelContainerData: IModelContainerData) => React.ReactElement | undefined | null;
    /**
      * Method for individual rendering of model container.
      */
    onRenderModelContainer?: (modelContainerData: IModelContainerData) => React.ComponentType<ModelServiceProps> | undefined | null;
    /**
      * Method for individual rendering of variable.
      *
      * @returns {React.ComponentType<ModelVariableProps> | undefined | null}  Componente to display variable, if undefined variable is rendered automatically.
      */
    onRenderModelVariable?: (modelVariable: IModelVariable) => React.ComponentType<ModelVariableProps> | undefined | null;
    onStartAdornment?: (modelVariable: IModelVariable) => IModelVariableStartAdornment;
}
export interface IModelServiceProps {
    changedModelVariables?: Array<IModelVariable>;
    onModelVariableChanged?: (modelVariable: IModelVariable, service: IModelBaseService) => void;
}
export interface IModelVariableChangeProps {
    modelVariables?: Array<IModelVariable>;
    modelStructureContainer?: IModelStructureContainer;
    changedModelVariables?: Array<IModelVariable>;
    onValidationError?: (notification: ITextNotification) => void;
    /**
     * Method must be invoked if variable has changed to keep changes and rerender componente.
     */
    onModelVariableChanged?: (variable: IModelVariable) => void;
}
export interface IModelServiceChangeProps {
    service: IModelBaseService;
    serviceModelVariables?: Array<IModelVariable>;
    saveChangesCommand: boolean;
    discardChangesCommand?: string;
    onModelVariableChanged?: (modelVariable: IModelVariable, service: IModelBaseService) => void;
    onSaveChanges?: (changedModelVariables: Array<IModelVariable>) => Promise<boolean>;
    onChangedModelVariablesUpdated?: (changedModelVariables: Array<IModelVariable>, service?: IModelBaseService) => void;
    onChangesSaved: (result: boolean, notification?: ITextNotification, service?: IModelBaseService) => void;
}
export declare type ModelVariableProps = IModelVariableProps & IModelVariableContextProps;
export declare type ModelVariableRenderingProps = IModelVariableChangeProps & IGenericModelRenderingProps;
export declare type ModelServiceRenderingProps = IModelServiceChangeProps & IGenericModelRenderingProps;
export declare type ModelServiceProps = IModelServiceProps;
export declare const getFilteredVariables: (sourceVariables: Array<IModelVariable>, filters: Array<IModelVariableFilterProps>, onLocalize?: LocalizeMethod | undefined, filledByStart?: boolean | undefined) => IModelVariable[];
