import { Theme as MUITheme } from '@mui/material/styles';
import { SxProps } from '@mui/system';
import { Interpolation } from '@emotion/react';

// Module augmentation MUI
declare module '@mui/material/styles' {
  interface Palette {
    view: Palette['primary'];
    container: Palette['primary'];
    card: Palette['primary'];
    menu: Palette['primary'];
    command: Palette['primary'];
    markColor1: Palette['primary'];
    markColor2: Palette['primary'];
    markColor3: Palette['primary'];
    markColor4: Palette['primary'];
    markColor5: Palette['primary'];
  }
  interface PaletteOptions {
    view: PaletteOptions['primary'];
    container: PaletteOptions['primary'];
    card: PaletteOptions['primary'];
    menu: PaletteOptions['primary'];
    command: PaletteOptions['primary'];
    markColor1: PaletteOptions['primary'];
    markColor2: PaletteOptions['primary'];
    markColor3: PaletteOptions['primary'];
    markColor4: PaletteOptions['primary'];
    markColor5: PaletteOptions['primary'];
  }
};

// Module augmentation emotion
declare module "@emotion/react" {
  export interface Theme extends MUITheme { }
};

export const scrollVerticalStyle: SxProps<MUITheme> = {
  overflowX: 'hidden',
  overflowY: 'auto',
  scrollbarWidth: 'thin',
  scrollbarColor: (theme) => theme.palette.primary.main,
  '&::-webkit-scrollbar': {
    width: '0.4rem'
  },
  '&::-webkit-scrollbar-thumb': {
    backgroundColor: (theme) => theme.palette.primary.main
  }
};

export const scrollHorizontalStyle: SxProps<MUITheme> = {
  overflowX: 'auto',
  overflowY: 'hidden',
  scrollbarWidth: 'thin',
  scrollbarColor: (theme) => theme.palette.primary.main,
  '&::-webkit-scrollbar': {
    width: '0.4rem'
  },
  '&::-webkit-scrollbar-thumb': {
    backgroundColor: (theme) => theme.palette.primary.main
  }
};

export const textBreakWordStyle: SxProps<MUITheme> = {
  overflow: 'hidden',
  overflowWrap: 'break-word',
  whiteSpace: 'pre-wrap',
  wordBreak: 'break-word'
};

export const horizontalLineStyle: SxProps<MUITheme> = {
  width: '100%',
  border: 'none',
  borderTop: '1px solid',
  opacity: 0.6,
  margin: 0
};

export const cardStyle: SxProps<MUITheme> = {
  width: '100%',
  borderRadius: 0
};

export const cardContentStyle: SxProps<MUITheme> = {
  width: '100%',
  height: '100%',
  overflow: 'hidden',
  backgroundColor: (theme) => theme.palette.card.main,
  color: (theme) => theme.palette.card.contrastText,
  fill: (theme: MUITheme) => theme.palette.card.contrastText
};

export const cardMediaStyle: SxProps<MUITheme> = {
  width: '100%',
  height: '100%',
  overflow: 'hidden',
  backgroundColor: (theme) => theme.palette.card.main,
  color: (theme) => theme.palette.card.contrastText,
  fill: (theme: MUITheme) => theme.palette.card.contrastText
};

export const accordionSummaryRootStyle: SxProps<MUITheme> = {
  padding: 0,
  margin: 0,
  backgroundColor: (theme) => theme.palette.card.main,
  color: (theme) => theme.palette.card.contrastText,
  fill: (theme: MUITheme) => theme.palette.card.contrastText
};

export const accordionSummaryExpandedStyle: SxProps<MUITheme> = {
  backgroundColor: (theme) => theme.palette.primary.main,
  color: (theme) => theme.palette.primary.contrastText,
  fill: (theme: MUITheme) => theme.palette.primary.contrastText
};

export const accordionSummaryContentStyle: SxProps<MUITheme> = {
  height: '100%',
  width: '100%'
};

export const accordionDetailsRootStyle: SxProps<MUITheme> = {
  display: 'flex',
  flexDirection: 'column',
  backgroundColor: (theme) => theme.palette.container.main,
  color: (theme) => theme.palette.container.contrastText,
  fill: (theme: MUITheme) => theme.palette.container.contrastText,
  // borderLeftStyle: 'solid',
  // borderLeftWidth: 2,
  // borderLeftColor: theme.palette.primary.main,
  borderBottomStyle: 'solid',
  borderBottomWidth: 2,
  borderBottomColor: (theme) => theme.palette.primary.main,
  // borderRightStyle: 'solid',
  // borderRightWidth: 2,
  // borderRightColor: theme.palette.primary.main
};

export const dialogPaperStyle: SxProps<MUITheme> = {
  backgroundColor: (theme) => theme.palette.view.main,
  color: (theme) => theme.palette.view.contrastText,
  fill: (theme: MUITheme) => theme.palette.view.contrastText,
};

export const dialogTitleStyle: SxProps<MUITheme> = {
  backgroundColor: (theme) => theme.palette.primary.main,
  color: (theme) => theme.palette.primary.contrastText,
  fill: (theme: MUITheme) => theme.palette.primary.contrastText
};

export const dialogTitleContainerStyle: SxProps<MUITheme> = {
  display: 'flex',
  flexDirection: 'row',
  alignContent: 'center',
  alignItems: 'center',
  margin: (theme) => theme.spacing(1)
};

export const dialogContainerStyle: SxProps<MUITheme> = {
  flex: 'auto',
  display: 'flex',
  flexDirection: 'column',
  overflow: 'hidden',
  margin: (theme) => theme.spacing(1)
};

export const tileIconStyle: SxProps<MUITheme> = {
  minWidth: 48,
  minHeight: 48,
  width: 48,
  height: 48
};

export const tileIconStyleCSS: Interpolation<MUITheme> = {
  minWidth: 48,
  minHeight: 48,
  width: 48,
  height: 48
};

export const itemIconStyle: SxProps<MUITheme> = {
  minWidth: 40,
  minHeight: 40,
  width: 40,
  height: 40
};

export const itemIconStyleCSS: Interpolation<MUITheme> = {
  minWidth: 40,
  minHeight: 40,
  width: 40,
  height: 40
};

export const textWhiteSpaceStyle: SxProps<MUITheme> = {
  whiteSpace: 'pre',
  ':before': {
    content: '" "'
  },
  ':after': {
    content: '" "'
  }
};
